@@ -22,7 +22,9 @@ from ufl.corealg.traversal import unique_post_traversal
 from ufl.conditional import Conditional
 from ufl.algorithms import expand_derivatives
 
+
 class ExpressionCollection(object):
+
     def __init__(self, cell):
         self.cell = cell
 
@@ -32,7 +34,7 @@ class ExpressionCollection(object):
         c = CellVolume(cell)
         h = Circumradius(cell)
         f = FacetArea(cell)
-        #s = CellSurfaceArea(cell)
+        # s = CellSurfaceArea(cell)
         # FIXME: Add all new geometry types here!
 
         I = Identity(d)
@@ -69,16 +71,16 @@ class ExpressionCollection(object):
             u+2*u, v+2*v, w+2*w,
             2/u, u/2, v/2, w/2,
             u**3, 3**u,
-            ])
+        ])
         self.mathfunctions = ([
             abs(u), sqrt(u), exp(u), ln(u),
             cos(u), sin(u), tan(u), acos(u), asin(u), atan(u),
             erf(u), bessel_I(1, u), bessel_J(1, u), bessel_K(1, u), bessel_Y(1, u),
-            ])
+        ])
         self.variables = ([
             variable(u), variable(v), variable(w),
             variable(w*u), 3*variable(w*u),
-            ])
+        ])
 
         if d == 1:
             w2 = as_matrix(((u**2,),))
@@ -107,7 +109,7 @@ class ExpressionCollection(object):
                 as_matrix(((u**2, u**3), (u**4, u**5))),
                 as_vector((u, 2, 3))[i],
                 w2[i, j]*w[i, j],
-                ])
+        ])
         self.conditionals = ([
             conditional(le(u, 1.0), 1, 0),
             conditional(eq(3.0, u), 1, 0),
@@ -122,7 +124,7 @@ class ExpressionCollection(object):
             conditional(Not(ge(u, 0.0)), 1, 2),
             conditional(And(Not(ge(u, 0.0)), lt(u, 1.0)), 1, 2),
             conditional(le(u, 0.0), u**3, ln(u)),
-            ])
+        ])
         self.restrictions = [u('+'), u('-'), v('+'), v('-'), w('+'), w('-')]
         if d > 1:
             i, j = indices(2)
@@ -131,7 +133,7 @@ class ExpressionCollection(object):
                 v[i]('+')*v[i]('+'),
                 (v[i]*v[i])('+'),
                 (v[i]*v[j])('+')*w[i, j]('+'),
-                ])
+            ])
 
         self.noncompounds = []
         self.noncompounds += self.algebra
@@ -154,7 +156,7 @@ class ExpressionCollection(object):
                 outer(w, v),
                 outer(v, w),
                 outer(w, w),
-                ])
+            ])
 
         if d == 1:
             self.tensoralgebra = []
@@ -162,7 +164,7 @@ class ExpressionCollection(object):
             self.tensoralgebra = ([
                 w.T, sym(w), skew(w), dev(w),
                 det(w), tr(w), cofac(w), inv(w),
-                ])
+            ])
 
         if d != 3:
             self.crossproducts = []
@@ -173,7 +175,7 @@ class ExpressionCollection(object):
                 cross(v, w[0,:]),
                 cross(v, w[:, 1]),
                 cross(w[:, 0], v),
-                ])
+            ])
 
         self.compounds = []
         self.compounds += self.tensorproducts
@@ -185,17 +187,19 @@ class ExpressionCollection(object):
         self.all_expressions += self.noncompounds
         self.all_expressions += self.compounds
 
-@pytest.fixture(params=(1,2,3))
+
+@pytest.fixture(params=(1, 2, 3))
 def d_expr(request):
     d = request.param
-    cell = { 1: interval, 2: triangle, 3: tetrahedron }[d]
+    cell = {1: interval, 2: triangle, 3: tetrahedron}[d]
     expr = ExpressionCollection(cell)
     return d, expr
 
+
 def ad_algorithm(expr):
-    #alt = 1
-    #alt = 4
-    #alt = 6
+    # alt = 1
+    # alt = 4
+    # alt = 6
     alt = 0
     if alt == 0:
         return expand_derivatives(expr)
@@ -225,40 +229,47 @@ def ad_algorithm(expr):
             apply_expand_compounds_after=False,
             use_alternative_wrapper_algorithm=False)
 
+
 def _test_no_derivatives_no_change(self, collection):
     for expr in collection:
         before = expr
         after = ad_algorithm(before)
-        #print '\n', str(before), '\n', str(after), '\n'
+        # print '\n', str(before), '\n', str(after), '\n'
         self.assertEqualTotalShape(before, after)
         assert before == after
 
+
 def _test_no_derivatives_but_still_changed(self, collection):
     # Planning to fix these:
     for expr in collection:
         before = expr
         after = ad_algorithm(before)
-        #print '\n', str(before), '\n', str(after), '\n'
+        # print '\n', str(before), '\n', str(after), '\n'
         self.assertEqualTotalShape(before, after)
-        #assert before == after # Without expand_compounds
+        # assert before == after # Without expand_compounds
         self.assertNotEqual(before, after) # With expand_compounds
 
+
 def test_only_terminals_no_change(self, d_expr):
     d, ex = d_expr
     _test_no_derivatives_no_change(self, ex.terminals)
 
+
 def test_no_derivatives_no_change(self, d_expr):
     d, ex = d_expr
     _test_no_derivatives_no_change(self, ex.noncompounds)
 
+
 def xtest_compounds_no_derivatives_no_change(self, d_expr): # This test fails with expand_compounds enabled
     d, ex = d_expr
     _test_no_derivatives_no_change(self, ex.compounds)
 
+
 def test_zero_derivatives_of_terminals_produce_the_right_types_and_shapes(self, d_expr):
     d, ex = d_expr
     _test_zero_derivatives_of_terminals_produce_the_right_types_and_shapes(self, ex)
 
+
 def _test_zero_derivatives_of_terminals_produce_the_right_types_and_shapes(self, collection):
     c = Constant(collection.shared_objects.cell)
 
@@ -271,19 +282,21 @@ def _test_zero_derivatives_of_terminals_produce_the_right_types_and_shapes(self,
             before = derivative(t, var) # This will often get preliminary simplified to zero
             after = ad_algorithm(before)
             expected = 0*t
-            #print '\n', str(expected), '\n', str(after), '\n', str(before), '\n'
+            # print '\n', str(expected), '\n', str(after), '\n', str(before), '\n'
             assert after == expected
 
             before = derivative(c*t, var) # This will usually not get simplified to zero
             after = ad_algorithm(before)
             expected = 0*t
-            #print '\n', str(expected), '\n', str(after), '\n', str(before), '\n'
+            # print '\n', str(expected), '\n', str(after), '\n', str(before), '\n'
             assert after == expected
 
+
 def test_zero_diffs_of_terminals_produce_the_right_types_and_shapes(self, d_expr):
     d, ex = d_expr
     _test_zero_diffs_of_terminals_produce_the_right_types_and_shapes(self, ex)
 
+
 def _test_zero_diffs_of_terminals_produce_the_right_types_and_shapes(self, collection):
     c = Constant(collection.shared_objects.cell)
 
@@ -299,19 +312,21 @@ def _test_zero_diffs_of_terminals_produce_the_right_types_and_shapes(self, colle
             before = diff(t, var) # This will often get preliminary simplified to zero
             after = ad_algorithm(before)
             expected = 0*outer(t, var)
-            #print '\n', str(expected), '\n', str(after), '\n', str(before), '\n'
+            # print '\n', str(expected), '\n', str(after), '\n', str(before), '\n'
             assert after == expected
 
             before = diff(c*t, var) # This will usually not get simplified to zero
             after = ad_algorithm(before)
             expected = 0*outer(t, var)
-            #print '\n', str(expected), '\n', str(after), '\n', str(before), '\n'
+            # print '\n', str(expected), '\n', str(after), '\n', str(before), '\n'
             assert after == expected
 
+
 def test_zero_derivatives_of_noncompounds_produce_the_right_types_and_shapes(self, d_expr):
     d, ex = d_expr
     _test_zero_derivatives_of_noncompounds_produce_the_right_types_and_shapes(self, ex)
 
+
 def _test_zero_derivatives_of_noncompounds_produce_the_right_types_and_shapes(self, collection):
     debug = 0
 
@@ -319,26 +334,35 @@ def _test_zero_derivatives_of_noncompounds_produce_the_right_types_and_shapes(se
     v = Coefficient(collection.shared_objects.V)
     w = Coefficient(collection.shared_objects.W)
 
-    #for t in chain(collection.noncompounds, collection.compounds):
-    #debug = True
+    # for t in chain(collection.noncompounds, collection.compounds):
+    # debug = True
     for t in collection.noncompounds:
         for var in (u, v, w):
-            if debug: print('\n', 'shapes:   ', t.ufl_shape, var.ufl_shape, '\n')
-            if debug: print('\n', 't:        ', str(t), '\n')
-            if debug: print('\n', 't ind:    ', str(t.ufl_free_indices), '\n')
-            if debug: print('\n', 'var:      ', str(var), '\n')
+            if debug:
+                print('\n', 'shapes:   ', t.ufl_shape, var.ufl_shape, '\n')
+            if debug:
+                print('\n', 't:        ', str(t), '\n')
+            if debug:
+                print('\n', 't ind:    ', str(t.ufl_free_indices), '\n')
+            if debug:
+                print('\n', 'var:      ', str(var), '\n')
             before = derivative(t, var)
-            if debug: print('\n', 'before:   ', str(before), '\n')
+            if debug:
+                print('\n', 'before:   ', str(before), '\n')
             after = ad_algorithm(before)
-            if debug: print('\n', 'after:    ', str(after), '\n')
+            if debug:
+                print('\n', 'after:    ', str(after), '\n')
             expected = 0*t
-            if debug: print('\n', 'expected: ', str(expected), '\n')
+            if debug:
+                print('\n', 'expected: ', str(expected), '\n')
             assert after == expected
 
+
 def test_zero_diffs_of_noncompounds_produce_the_right_types_and_shapes(self, d_expr):
     d, ex = d_expr
     _test_zero_diffs_of_noncompounds_produce_the_right_types_and_shapes(self, ex)
 
+
 def _test_zero_diffs_of_noncompounds_produce_the_right_types_and_shapes(self, collection):
     debug = 0
 
@@ -350,22 +374,27 @@ def _test_zero_diffs_of_noncompounds_produce_the_right_types_and_shapes(self, co
     vv = variable(v)
     vw = variable(w)
 
-    #for t in chain(collection.noncompounds, collection.compounds):
+    # for t in chain(collection.noncompounds, collection.compounds):
     for t in collection.noncompounds:
         for var in (vu, vv, vw):
             before = diff(t, var)
-            if debug: print('\n', 'before:   ', str(before), '\n')
+            if debug:
+                print('\n', 'before:   ', str(before), '\n')
             after = ad_algorithm(before)
-            if debug: print('\n', 'after:    ', str(after), '\n')
+            if debug:
+                print('\n', 'after:    ', str(after), '\n')
             expected = 0*outer(t, var)
-            if debug: print('\n', 'expected: ', str(expected), '\n')
-            #print '\n', str(expected), '\n', str(after), '\n', str(before), '\n'
+            if debug:
+                print('\n', 'expected: ', str(expected), '\n')
+            # print '\n', str(expected), '\n', str(after), '\n', str(before), '\n'
             assert after == expected
 
+
 def test_nonzero_derivatives_of_noncompounds_produce_the_right_types_and_shapes(self, d_expr):
     d, ex = d_expr
     _test_nonzero_derivatives_of_noncompounds_produce_the_right_types_and_shapes(self, ex)
 
+
 def _test_nonzero_derivatives_of_noncompounds_produce_the_right_types_and_shapes(self, collection):
     debug = 0
 
@@ -373,22 +402,27 @@ def _test_nonzero_derivatives_of_noncompounds_produce_the_right_types_and_shapes
     v = collection.shared_objects.v
     w = collection.shared_objects.w
 
-    #for t in chain(collection.noncompounds, collection.compounds):
+    # for t in chain(collection.noncompounds, collection.compounds):
     for t in collection.noncompounds:
         for var in (u, v, w):
             # Include d/dx [z ? y: x] but not d/dx [x ? f: z]
             if isinstance(t, Conditional) and (var in unique_post_traversal(t.ufl_operands[0])):
-                if debug: print(("Depends on %s :: %s" % (str(var), str(t))))
+                if debug:
+                    print(("Depends on %s :: %s" % (str(var), str(t))))
                 continue
 
-            if debug: print(('\n', '...:   ', t.ufl_shape, var.ufl_shape, '\n'))
+            if debug:
+                print(('\n', '...:   ', t.ufl_shape, var.ufl_shape, '\n'))
             before = derivative(t, var)
-            if debug: print(('\n', 'before:   ', str(before), '\n'))
+            if debug:
+                print(('\n', 'before:   ', str(before), '\n'))
             after = ad_algorithm(before)
-            if debug: print(('\n', 'after:    ', str(after), '\n'))
+            if debug:
+                print(('\n', 'after:    ', str(after), '\n'))
             expected_shape = 0*t
-            if debug: print(('\n', 'expected_shape: ', str(expected_shape), '\n'))
-            #print '\n', str(expected_shape), '\n', str(after), '\n', str(before), '\n'
+            if debug:
+                print(('\n', 'expected_shape: ', str(expected_shape), '\n'))
+            # print '\n', str(expected_shape), '\n', str(after), '\n', str(before), '\n'
 
             if var in unique_post_traversal(t):
                 self.assertEqualTotalShape(after, expected_shape)
@@ -396,10 +430,12 @@ def _test_nonzero_derivatives_of_noncompounds_produce_the_right_types_and_shapes
             else:
                 assert after == expected_shape
 
+
 def test_nonzero_diffs_of_noncompounds_produce_the_right_types_and_shapes(self, d_expr):
     d, ex = d_expr
     _test_nonzero_diffs_of_noncompounds_produce_the_right_types_and_shapes(self, ex)
 
+
 def _test_nonzero_diffs_of_noncompounds_produce_the_right_types_and_shapes(self, collection):
     debug = 0
     u = collection.shared_objects.u
@@ -410,22 +446,26 @@ def _test_nonzero_diffs_of_noncompounds_produce_the_right_types_and_shapes(self,
     vv = variable(v)
     vw = variable(w)
 
-    #for t in chain(collection.noncompounds, collection.compounds):
+    # for t in chain(collection.noncompounds, collection.compounds):
     for t in collection.noncompounds:
         t = replace(t, {u:vu, v:vv, w:vw})
         for var in (vu, vv, vw):
             # Include d/dx [z ? y: x] but not d/dx [x ? f: z]
             if isinstance(t, Conditional) and (var in unique_post_traversal(t.ufl_operands[0])):
-                if debug: print(("Depends on %s :: %s" % (str(var), str(t))))
+                if debug:
+                    print(("Depends on %s :: %s" % (str(var), str(t))))
                 continue
 
             before = diff(t, var)
-            if debug: print(('\n', 'before:   ', str(before), '\n'))
+            if debug:
+                print(('\n', 'before:   ', str(before), '\n'))
             after = ad_algorithm(before)
-            if debug: print(('\n', 'after:    ', str(after), '\n'))
+            if debug:
+                print(('\n', 'after:    ', str(after), '\n'))
             expected_shape = 0*outer(t, var) # expected shape, not necessarily value
-            if debug: print(('\n', 'expected_shape: ', str(expected_shape), '\n'))
-            #print '\n', str(expected_shape), '\n', str(after), '\n', str(before), '\n'
+            if debug:
+                print(('\n', 'expected_shape: ', str(expected_shape), '\n'))
+            # print '\n', str(expected_shape), '\n', str(after), '\n', str(before), '\n'
 
             if var in unique_post_traversal(t):
                 self.assertEqualTotalShape(after, expected_shape)
@@ -433,6 +473,7 @@ def _test_nonzero_diffs_of_noncompounds_produce_the_right_types_and_shapes(self,
             else:
                 assert after == expected_shape
 
+
 def test_grad_coeff(self, d_expr):
     d, collection = d_expr
 
@@ -454,12 +495,13 @@ def test_grad_coeff(self, d_expr):
         before = grad(grad(f))
         after = ad_algorithm(before)
         self.assertEqualTotalShape(before, after)
-        #assert before == after # Differing by being wrapped in indexing types
+        # assert before == after # Differing by being wrapped in indexing types
 
         before = grad(grad(grad(f)))
         after = ad_algorithm(before)
         self.assertEqualTotalShape(before, after)
-        #assert before == after # Differing by being wrapped in indexing types
+        # assert before == after # Differing by being wrapped in indexing types
+
 
 def test_derivative_grad_coeff(self, d_expr):
     d, collection = d_expr
@@ -471,22 +513,23 @@ def test_derivative_grad_coeff(self, d_expr):
         before = derivative(grad(f), f)
         after = ad_algorithm(before)
         self.assertEqualTotalShape(before, after)
-        #assert after == expected
+        # assert after == expected
 
         before = derivative(grad(grad(f)), f)
         after = ad_algorithm(before)
         self.assertEqualTotalShape(before, after)
-        #assert after == expected
+        # assert after == expected
 
         before = derivative(grad(grad(grad(f))), f)
         after = ad_algorithm(before)
         self.assertEqualTotalShape(before, after)
-        #assert after == expected
+        # assert after == expected
         if 0:
             print()
             print(('B', f, "::", before))
             print(('A', f, "::", after))
 
+
 def xtest_derivative_grad_coeff_with_variation_components(self, d_expr):
     d, collection = d_expr
 
@@ -503,17 +546,17 @@ def xtest_derivative_grad_coeff_with_variation_components(self, d_expr):
         before = derivative(grad(g), f, df)
         after = ad_algorithm(before)
         self.assertEqualTotalShape(before, after)
-        #assert after == expected
+        # assert after == expected
 
         before = derivative(grad(grad(g)), f, df)
         after = ad_algorithm(before)
         self.assertEqualTotalShape(before, after)
-        #assert after == expected
+        # assert after == expected
 
         before = derivative(grad(grad(grad(g))), f, df)
         after = ad_algorithm(before)
         self.assertEqualTotalShape(before, after)
-        #assert after == expected
+        # assert after == expected
         if 0:
             print()
             print(('B', f, "::", before))
