@@ -1,6 +1,8 @@
 #!/usr/bin/env py.test
 # -*- coding: utf-8 -*-
 
+from __future__ import print_function
+
 __authors__ = "Martin Sandve Alnæs"
 __date__ = "2009-02-17 -- 2009-02-17"
 
@@ -20,9 +22,10 @@ def assertEqualBySampling(actual, expected):
     b = bd.preprocessed_form.integrals_by_type("cell")[0].integrand()
 
     assert ([ad.function_replace_map[ac] for ac in ad.reduced_coefficients]
-           == [bd.function_replace_map[bc] for bc in bd.reduced_coefficients])
+            == [bd.function_replace_map[bc] for bc in bd.reduced_coefficients])
 
     n = ad.num_coefficients
+
     def make_value(c):
         if isinstance(c, Coefficient):
             z = 0.3
@@ -77,7 +80,7 @@ def _test(self, f, df):
     wv = 11.0
 
     x = xv
-    mapping = { v: vv, u: uv, w: wv }
+    mapping = {v: vv, u: uv, w: wv}
 
     dfv1 = derivative(f(w), w, v)
     dfv2 = df(w, v)
@@ -93,124 +96,168 @@ def _test(self, f, df):
 
 # --- Literals
 
+
 def testScalarLiteral(self):
-    def f(w):     return as_ufl(1)
+    def f(w): return as_ufl(1)
+
     def df(w, v): return zero()
     _test(self, f, df)
 
+
 def testIdentityLiteral(self):
-    def f(w):     return Identity(2)[i, i]
+    def f(w): return Identity(2)[i, i]
+
     def df(w, v): return zero()
     _test(self, f, df)
 
 # --- Form arguments
 
+
 def testCoefficient(self):
-    def f(w):     return w
+    def f(w): return w
+
     def df(w, v): return v
     _test(self, f, df)
 
+
 def testArgument(self):
-    def f(w):     return TestFunction(FiniteElement("CG", triangle, 1))
+    def f(w): return TestFunction(FiniteElement("CG", triangle, 1))
+
     def df(w, v): return zero()
     _test(self, f, df)
 
 # --- Geometry
 
+
 def testSpatialCoordinate(self):
-    def f(w):     return SpatialCoordinate(triangle)[0]
+    def f(w): return SpatialCoordinate(triangle)[0]
+
     def df(w, v): return zero()
     _test(self, f, df)
 
+
 def testFacetNormal(self):
-    def f(w):     return FacetNormal(triangle)[0]
+    def f(w): return FacetNormal(triangle)[0]
+
     def df(w, v): return zero()
     _test(self, f, df)
 
-#def testCellSurfaceArea(self):
+# def testCellSurfaceArea(self):
 #    def f(w):     return CellSurfaceArea(triangle)
 #    def df(w, v): return zero()
 #    _test(self, f, df)
 
+
 def testFacetArea(self):
-    def f(w):     return FacetArea(triangle)
+    def f(w): return FacetArea(triangle)
+
     def df(w, v): return zero()
     _test(self, f, df)
 
+
 def testCircumradius(self):
-    def f(w):     return Circumradius(triangle)
+    def f(w): return Circumradius(triangle)
+
     def df(w, v): return zero()
     _test(self, f, df)
 
+
 def testCellVolume(self):
-    def f(w):     return CellVolume(triangle)
+    def f(w): return CellVolume(triangle)
+
     def df(w, v): return zero()
     _test(self, f, df)
 
 # --- Basic operators
 
+
 def testSum(self):
-    def f(w):     return w + 1
+    def f(w): return w + 1
+
     def df(w, v): return v
     _test(self, f, df)
 
+
 def testProduct(self):
-    def f(w):     return 3*w
+    def f(w): return 3*w
+
     def df(w, v): return 3*v
     _test(self, f, df)
 
+
 def testPower(self):
-    def f(w):     return w**3
+    def f(w): return w**3
+
     def df(w, v): return 3*w**2*v
     _test(self, f, df)
 
+
 def testDivision(self):
-    def f(w):     return w / 3.0
+    def f(w): return w / 3.0
+
     def df(w, v): return v / 3.0
     _test(self, f, df)
 
+
 def testDivision2(self):
-    def f(w):     return 3.0 / w
+    def f(w): return 3.0 / w
+
     def df(w, v): return -3.0 * v / w**2
     _test(self, f, df)
 
+
 def testExp(self):
-    def f(w):     return exp(w)
+    def f(w): return exp(w)
+
     def df(w, v): return v*exp(w)
     _test(self, f, df)
 
+
 def testLn(self):
-    def f(w):     return ln(w)
+    def f(w): return ln(w)
+
     def df(w, v): return v / w
     _test(self, f, df)
 
+
 def testCos(self):
-    def f(w):     return cos(w)
+    def f(w): return cos(w)
+
     def df(w, v): return -v*sin(w)
     _test(self, f, df)
 
+
 def testSin(self):
-    def f(w):     return sin(w)
+    def f(w): return sin(w)
+
     def df(w, v): return v*cos(w)
     _test(self, f, df)
 
+
 def testTan(self):
-    def f(w):     return tan(w)
+    def f(w): return tan(w)
+
     def df(w, v): return v*2.0/(cos(2.0*w) + 1.0)
     _test(self, f, df)
 
+
 def testAcos(self):
-    def f(w):     return acos(w/1000)
+    def f(w): return acos(w/1000)
+
     def df(w, v): return -(v/1000)/sqrt(1.0 - (w/1000)**2)
     _test(self, f, df)
 
+
 def testAsin(self):
-    def f(w):     return asin(w/1000)
+    def f(w): return asin(w/1000)
+
     def df(w, v): return (v/1000)/sqrt(1.0 - (w/1000)**2)
     _test(self, f, df)
 
+
 def testAtan(self):
-    def f(w):     return atan(w)
+    def f(w): return atan(w)
+
     def df(w, v): return v/(1.0 + w**2)
     _test(self, f, df)
 
@@ -218,31 +265,42 @@ def testAtan(self):
 
 # --- Abs and conditionals
 
+
 def testAbs(self):
-    def f(w):     return abs(w)
+    def f(w): return abs(w)
+
     def df(w, v): return sign(w)*v
     _test(self, f, df)
 
+
 def testConditional(self):
     def cond(w): return lt(1.0, 2.0)
-    def f(w):     return conditional(cond(w), 2*w, 3*w)
+
+    def f(w): return conditional(cond(w), 2*w, 3*w)
+
     def df(w, v): return 2*v
     _test(self, f, df)
 
     def cond(w): return lt(2.0, 1.0)
-    def f(w):     return conditional(cond(w), 2*w, 3*w)
+
+    def f(w): return conditional(cond(w), 2*w, 3*w)
+
     def df(w, v): return 3*v
     _test(self, f, df)
 
-def testConditional(self): # This will fail without bugfix in derivative
+
+def testConditional(self):  # This will fail without bugfix in derivative
     def cond(w): return lt(w, 1.0)
-    def f(w):     return conditional(cond(w), 2*w, 3*w)
+
+    def f(w): return conditional(cond(w), 2*w, 3*w)
+
     def df(w, v): return (conditional(cond(w), 1, 0) * 2*v +
                           conditional(cond(w), 0, 1) * 3*v)
     _test(self, f, df)
 
 # --- Tensor algebra basics
 
+
 def testIndexSum(self):
     def f(w):
         # 3*w + 4*w**2 + 5*w**3
@@ -250,22 +308,24 @@ def testIndexSum(self):
         b = as_vector((3, 4, 5))
         i, = indices(1)
         return a[i]*b[i]
+
     def df(w, v): return 3*v + 4*2*w*v + 5*3*w**2*v
     _test(self, f, df)
 
+
 def testListTensor(self):
     v = variable(as_ufl(42))
     f = as_tensor((
-            ( (0,      0), (0,   0) ),
-            ( (v,    2*v), (0,   0) ),
-            ( (v**2,   1), (2, v/2) ),
-            ))
+        ((0, 0), (0, 0)),
+            ((v, 2*v), (0, 0)),
+            ((v**2, 1), (2, v/2)),
+    ))
     assert f.ufl_shape == (3, 2, 2)
     g = as_tensor((
-            ( (0, 0), (0, 0) ),
-            ( (1, 2), (0, 0) ),
-            ( (84, 0), (0, 0.5) ),
-            ))
+        ((0, 0), (0, 0)),
+            ((1, 2), (0, 0)),
+            ((84, 0), (0, 0.5)),
+    ))
     assert g.ufl_shape == (3, 2, 2)
     dfv = diff(f, v)
     x = None
@@ -276,6 +336,7 @@ def testListTensor(self):
 
 # --- Coefficient and argument input configurations
 
+
 def test_single_scalar_coefficient_derivative(self):
     cell = triangle
     V = FiniteElement("CG", cell, 1)
@@ -285,6 +346,7 @@ def test_single_scalar_coefficient_derivative(self):
     b = derivative(a, u, v)
     self.assertEqualAfterPreprocessing(b, 3*(u*(2*v)))
 
+
 def test_single_vector_coefficient_derivative(self):
     cell = triangle
     V = VectorElement("CG", cell, 1)
@@ -295,6 +357,7 @@ def test_single_vector_coefficient_derivative(self):
     expected = 3*(2*(u[i]*v[i]))
     assertEqualBySampling(actual, expected)
 
+
 def test_multiple_coefficient_derivative(self):
     cell = triangle
     V = FiniteElement("CG", cell, 1)
@@ -315,6 +378,7 @@ def test_multiple_coefficient_derivative(self):
     expected = cos(uv)*vv * (uw[i]*uw[i]) + (uw[j]*vw[j])*2 * sin(uv)
     assertEqualBySampling(actual, expected)
 
+
 def test_indexed_coefficient_derivative(self):
     cell = triangle
     I = Identity(cell.geometric_dimension())
@@ -324,7 +388,7 @@ def test_indexed_coefficient_derivative(self):
     v = TestFunction(V)
 
     w = dot(u, nabla_grad(u))
-    #a = dot(w, w)
+    # a = dot(w, w)
     a = (u[i]*u[k].dx(i)) * w[k]
 
     actual = derivative(a, u[0], v)
@@ -334,6 +398,7 @@ def test_indexed_coefficient_derivative(self):
 
     assertEqualBySampling(actual, expected)
 
+
 def test_multiple_indexed_coefficient_derivative(self):
     cell = tetrahedron
     I = Identity(cell.geometric_dimension())
@@ -350,6 +415,7 @@ def test_multiple_indexed_coefficient_derivative(self):
 
     assertEqualBySampling(actual, expected)
 
+
 def test_segregated_derivative_of_convection(self):
     cell = tetrahedron
     V = FiniteElement("CG", cell, 1)
@@ -375,7 +441,7 @@ def test_segregated_derivative_of_convection(self):
             fd = compute_form_data(form)
             pf = fd.preprocessed_form
             a = expand_indices(pf)
-            #print (i,j), str(a)
+            # print (i,j), str(a)
 
     k = Index()
     for i in range(cell.geometric_dimension()):
@@ -386,6 +452,7 @@ def test_segregated_derivative_of_convection(self):
 
 # --- User provided derivatives of coefficients
 
+
 def test_coefficient_derivatives(self):
     V = FiniteElement("Lagrange", triangle, 1)
 
@@ -396,7 +463,7 @@ def test_coefficient_derivatives(self):
     df = Coefficient(V, count=2)
     dg = Coefficient(V, count=3)
     u = Coefficient(V, count=4)
-    cd = { f: df, g: dg }
+    cd = {f: df, g: dg}
 
     integrand = inner(f, g)
     expected = (df*dv)*g + f*(dg*dv)
@@ -408,6 +475,7 @@ def test_coefficient_derivatives(self):
     assert (actual*dx).signature() == (expected*dx).signature()
     self.assertEqual(replace(actual, fd.function_replace_map), expected)
 
+
 def test_vector_coefficient_derivatives(self):
     V = VectorElement("Lagrange", triangle, 1)
     VV = TensorElement("Lagrange", triangle, 1)
@@ -418,7 +486,7 @@ def test_vector_coefficient_derivatives(self):
     g = Coefficient(V, count=1)
     f = Coefficient(V, count=2)
     u = Coefficient(V, count=3)
-    cd = { f: df }
+    cd = {f: df}
 
     integrand = inner(f, g)
 
@@ -430,7 +498,8 @@ def test_vector_coefficient_derivatives(self):
     fd = compute_form_data(J)
     actual = fd.preprocessed_form.integrals()[0].integrand()
     assert (actual*dx).signature() == (expected*dx).signature()
-    #self.assertEqual(replace(actual, fd.function_replace_map), expected)
+    # self.assertEqual(replace(actual, fd.function_replace_map), expected)
+
 
 def test_vector_coefficient_derivatives_of_product(self):
     V = VectorElement("Lagrange", triangle, 1)
@@ -443,35 +512,19 @@ def test_vector_coefficient_derivatives_of_product(self):
     dg = Coefficient(VV, count=2)
     f = Coefficient(V, count=3)
     u = Coefficient(V, count=4)
-    cd = { f: df, g: dg }
+    cd = {f: df, g: dg}
 
     integrand = f[i]*g[i]
 
     i0, i1, i2, i3, i4 = [Index(count=c) for c in range(5)]
     expected = as_tensor(df[i2, i1]*dv[i1], (i2,))[i0]*g[i0] +\
-               f[i0]*as_tensor(dg[i4, i3]*dv[i3], (i4,))[i0]
+        f[i0]*as_tensor(dg[i4, i3]*dv[i3], (i4,))[i0]
 
     F = integrand*dx
     J = derivative(F, u, dv, cd)
     fd = compute_form_data(J)
     actual = fd.preprocessed_form.integrals()[0].integrand()
 
-    # Keeping this snippet here for a while for debugging purposes
-    if 0:
-        print(('\n', 'str:'))
-        print((str(actual)))
-        print((str(expected)))
-        print(('\n', 'repr:'))
-        print((repr(actual)))
-        print((repr(expected)))
-        from ufl.algorithms import tree_format
-        open('actual.txt', 'w').write(tree_format(actual))
-        open('expected.txt', 'w').write(tree_format(expected))
-        print(('\n', 'equal:'))
-        print((str(actual) == str(expected)))
-        print((repr(actual) == repr(expected)))
-        print((actual == expected))
-
     # Tricky case! These are equal in representation except
     # that the outermost sum/indexsum are swapped.
     # Sampling the expressions instead of comparing representations.
@@ -483,6 +536,7 @@ def test_vector_coefficient_derivatives_of_product(self):
 
 # --- Some actual forms
 
+
 def testHyperElasticity(self):
     cell = interval
     element = FiniteElement("CG", cell, 2)
@@ -517,7 +571,7 @@ def testHyperElasticity(self):
     F_expression = strip_variables(F.integrals_by_type("cell")[0].integrand())
     J_expression = strip_variables(J.integrals_by_type("cell")[0].integrand())
 
-    #classes = set(c.__class__ for c in post_traversal(f_expression))
+    # classes = set(c.__class__ for c in post_traversal(f_expression))
 
     Kv = .2
     bv = .3
@@ -545,22 +599,23 @@ def testHyperElasticity(self):
         return dw
 
     w, b, K = form_data_f.original_form.coefficients()
-    mapping = { K: Kv, b: bv, w: Nw }
+    mapping = {K: Kv, b: bv, w: Nw}
     fv2 = f_expression((0,), mapping)
     self.assertAlmostEqual(fv, fv2)
 
     w, b, K = form_data_F.original_form.coefficients()
     v, = form_data_F.original_form.arguments()
-    mapping = { K: Kv, b: bv, v: Nv, w: Nw }
+    mapping = {K: Kv, b: bv, v: Nv, w: Nw}
     Fv2 = F_expression((0,), mapping)
     self.assertAlmostEqual(Fv, Fv2)
 
     w, b, K = form_data_J.original_form.coefficients()
     v, u = form_data_J.original_form.arguments()
-    mapping = { K: Kv, b: bv, v: Nv, u: Nu, w: Nw }
+    mapping = {K: Kv, b: bv, v: Nv, u: Nu, w: Nw}
     Jv2 = J_expression((0,), mapping)
     self.assertAlmostEqual(Jv, Jv2)
 
+
 def test_mass_derived_from_functional(self):
     cell = triangle
     V = FiniteElement("CG", cell, 1)
@@ -572,13 +627,14 @@ def test_mass_derived_from_functional(self):
     f = (w**2/2)*dx
     L = w*v*dx
     a = u*v*dx
-    F  = derivative(f, w, v)
+    F = derivative(f, w, v)
     J1 = derivative(L, w, u)
     J2 = derivative(F, w, u)
     # TODO: assert something
 
 # --- Interaction with replace
 
+
 def test_derivative_replace_works_together(self):
     cell = triangle
     V = FiniteElement("CG", cell, 1)
@@ -591,7 +647,7 @@ def test_derivative_replace_works_together(self):
     M = cos(f)*sin(g)
     F = derivative(M, f, v)
     J = derivative(F, f, u)
-    JR = replace(J, { f: g })
+    JR = replace(J, {f: g})
 
     F2 = -sin(f)*v*sin(g)
     J2 = -cos(f)*u*v*sin(g)
@@ -603,6 +659,7 @@ def test_derivative_replace_works_together(self):
 
 # --- Scratch space
 
+
 def test_foobar(self):
     element = VectorElement("Lagrange", triangle, 1)
     v = TestFunction(element)
@@ -613,7 +670,7 @@ def test_foobar(self):
 
     def planarGrad(u):
         return as_matrix([[u[0].dx(0), 0, u[0].dx(1)],
-                          [ 0, 0, 0 ],
+                          [0, 0, 0],
                           [u[1].dx(0), 0, u[1].dx(1)]])
 
     def epsilon(u):
