@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """Algorithms for building canonical data structure for integrals over subdomains."""
 
-# Copyright (C) 2009-2015 Anders Logg and Martin Sandve Alnæs
+# Copyright (C) 2009-2016 Anders Logg and Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -18,20 +18,21 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 
+# import six
 from collections import defaultdict
 from six.moves import zip
-from six import iteritems
 
 import ufl
 from ufl.log import error
-from ufl.assertions import ufl_assert
-from ufl.measure import Measure
+from ufl.utils.py23 import as_native_strings
 from ufl.integral import Integral
 from ufl.form import Form
 from ufl.sorting import cmp_expr, sorted_expr
 from ufl.utils.sorting import canonicalize_metadata, sorted_by_key, sorted_by_tuple_key
+import numbers
 
 
+# @six.python_2_unicode_compatible
 class IntegralData(object):
     """Utility class with the members
         (domain, integral_type, subdomain_id, integrals, metadata)
@@ -39,15 +40,17 @@ class IntegralData(object):
     where metadata is an empty dictionary that may be used for
     associating metadata with each object.
     """
-    __slots__ = ('domain', 'integral_type', 'subdomain_id', 'integrals', 'metadata',
-                 'integral_coefficients', 'enabled_coefficients')
-    def __init__(self, domain, integral_type, subdomain_id, integrals, metadata):
-        ufl_assert(len(set(itg.ufl_domain() for itg in integrals)) == 1,
-                   "Multiple domains mismatch in integral data.")
-        ufl_assert(all(integral_type == itg.integral_type() for itg in integrals),
-                   "Integral type mismatch in integral data.")
-        ufl_assert(all(subdomain_id == itg.subdomain_id() for itg in integrals),
-                   "Subdomain id mismatch in integral data.")
+    __slots__ = as_native_strings(('domain', 'integral_type', 'subdomain_id', 'integrals',
+                 'metadata', 'integral_coefficients', 'enabled_coefficients'))
+
+    def __init__(self, domain, integral_type, subdomain_id, integrals,
+                 metadata):
+        if 1 != len(set(itg.ufl_domain() for itg in integrals)):
+            error("Multiple domains mismatch in integral data.")
+        if not all(integral_type == itg.integral_type() for itg in integrals):
+            error("Integral type mismatch in integral data.")
+        if not all(subdomain_id == itg.subdomain_id() for itg in integrals):
+            error("Subdomain id mismatch in integral data.")
 
         self.domain = domain
         self.integral_type = integral_type
@@ -55,17 +58,21 @@ class IntegralData(object):
 
         self.integrals = integrals
 
-        # This is populated in preprocess using data not available at this stage:
+        # This is populated in preprocess using data not available at
+        # this stage:
         self.integral_coefficients = None
         self.enabled_coefficients = None
 
-        # TODO: I think we can get rid of this with some refactoring in ffc:
+        # TODO: I think we can get rid of this with some refactoring
+        # in ffc:
         self.metadata = metadata
 
     def __lt__(self, other):
         # To preserve behaviour of extract_integral_data:
-        return ((self.integral_type, self.subdomain_id, self.integrals, self.metadata)
-                < (other.integral_type, other.subdomain_id, other.integrals, other.metadata))
+        return ((self.integral_type, self.subdomain_id,
+                 self.integrals, self.metadata) <
+                (other.integral_type, other.subdomain_id, other.integrals,
+                 other.metadata))
 
     def __eq__(self, other):
         # Currently only used for tests:
@@ -74,10 +81,16 @@ class IntegralData(object):
                 self.integrals == other.integrals and
                 self.metadata == other.metadata)
 
+    def __unicode__(self):
+        # Only in python 2
+        return str(self).decode("utf-8")
+
     def __str__(self):
-        return "IntegralData object over domain (%s, %s), with integrals:\n%s\nand metadata:\n%s" % (
+        s = "IntegralData over domain(%s, %s), with integrals:\n%s\nand metadata:\n%s" % (
             self.integral_type, self.subdomain_id,
             '\n\n'.join(map(str, self.integrals)), self.metadata)
+        return s
+
 
 def dicts_lt(a, b):
     na = 0 if a is None else len(a)
@@ -87,16 +100,19 @@ def dicts_lt(a, b):
     for ia, ib in zip(sorted_by_key(a), sorted_by_key(b)):
         # Assuming keys are sortable (usually str)
         if ia[0] != ib[0]:
-            return (ia[0].__class__.__name__, ia[0]) < (ib[0].__class__.__name__, ib[0]) # Hack to preserve type sorting in py3
+            return (ia[0].__class__.__name__, ia[0]) < (ib[0].__class__.__name__, ib[0])  # Hack to preserve type sorting in py3
         # Assuming values are sortable
         if ia[1] != ib[1]:
-            return (ia[1].__class__.__name__, ia[1]) < (ib[1].__class__.__name__, ib[1]) # Hack to preserve type sorting in py3
+            return (ia[1].__class__.__name__, ia[1]) < (ib[1].__class__.__name__, ib[1])  # Hack to preserve type sorting in py3
+
 
 # Tuple comparison helper
 class ExprTupleKey(object):
-    __slots__ = ('x',)
+    __slots__ = as_native_strings(('x',))
+
     def __init__(self, x):
         self.x = x
+
     def __lt__(self, other):
         # Comparing expression first
         c = cmp_expr(self.x[0], other.x[0])
@@ -110,6 +126,7 @@ class ExprTupleKey(object):
             mdo = canonicalize_metadata(other.x[1])
             return mds < mdo
 
+
 def group_integrals_by_domain_and_type(integrals, domains):
     """
     Input:
@@ -119,10 +136,10 @@ def group_integrals_by_domain_and_type(integrals, domains):
     Output:
         integrals_by_domain_and_type: dict: (domain, integral_type) -> list(Integral)
     """
-    integral_data = []
     integrals_by_domain_and_type = defaultdict(list)
     for itg in integrals:
-        ufl_assert(itg.ufl_domain() is not None, "Integrals without a domain is now illegal.")
+        if itg.ufl_domain() is None:
+            error("Integral has no domain.")
         key = (itg.ufl_domain(), itg.integral_type())
 
         # Append integral to list of integrals with shared key
@@ -130,14 +147,15 @@ def group_integrals_by_domain_and_type(integrals, domains):
 
     return integrals_by_domain_and_type
 
+
 def integral_subdomain_ids(integral):
     "Get a tuple of integer subdomains or a valid string subdomain from integral."
     did = integral.subdomain_id()
-    if isinstance(did, int):
+    if isinstance(did, numbers.Integral):
         return (did,)
     elif isinstance(did, tuple):
-        ufl_assert(all(isinstance(d, int) for d in did),
-                   "Expecting only integer subdomains in tuple.")
+        if not all(isinstance(d, numbers.Integral) for d in did):
+            error("Expecting only integer subdomains in tuple.")
         return did
     elif did in ("everywhere", "otherwise"):
         # TODO: Define list of valid strings somewhere more central
@@ -145,6 +163,7 @@ def integral_subdomain_ids(integral):
     else:
         error("Invalid domain id %s." % did)
 
+
 def rearrange_integrals_by_single_subdomains(integrals):
     """Rearrange integrals over multiple subdomains to single subdomain integrals.
 
@@ -167,7 +186,8 @@ def rearrange_integrals_by_single_subdomains(integrals):
             subdomain_integrals.append((dids, itg))
 
     # Fill single_subdomain_integrals with lists of integrals from
-    # subdomain_integrals, but split and restricted to single subdomain ids
+    # subdomain_integrals, but split and restricted to single
+    # subdomain ids
     single_subdomain_integrals = defaultdict(list)
     for dids, itg in subdomain_integrals:
         # Region or single subdomain id
@@ -175,7 +195,8 @@ def rearrange_integrals_by_single_subdomains(integrals):
             # Restrict integral to this subdomain!
             single_subdomain_integrals[did].append(itg.reconstruct(subdomain_id=did))
 
-    # Add everywhere integrals to each single subdomain id integral list
+    # Add everywhere integrals to each single subdomain id integral
+    # list
     otherwise_integrals = []
     for ev_itg in everywhere_integrals:
         # Restrict everywhere integral to 'otherwise'
@@ -214,7 +235,8 @@ def accumulate_integrands_with_same_metadata(integrals):
             by_cdid[cdid] = ([], cd)
         by_cdid[cdid][0].append(itg)
 
-    # Accumulate integrands separately for each compiler data object id
+    # Accumulate integrands separately for each compiler data object
+    # id
     for cdid in by_cdid:
         integrals, cd = by_cdid[cdid]
         # Ensure canonical sorting of more than two integrands
@@ -222,7 +244,8 @@ def accumulate_integrands_with_same_metadata(integrals):
         integrands_sum = sum(integrands[1:], integrands[0])
         by_cdid[cdid] = (integrands_sum, cd)
 
-    # Sort integrands canonically by integrand first then compiler data
+    # Sort integrands canonically by integrand first then compiler
+    # data
     return sorted(by_cdid.values(), key=ExprTupleKey)
 
 
@@ -284,7 +307,8 @@ def group_form_integrals(form, domains):
                 rearrange_integrals_by_single_subdomains(ddt_integrals)
 
             for subdomain_id, ss_integrals in sorted_by_key(single_subdomain_integrals):
-                # Accumulate integrands of integrals that share the same compiler data
+                # Accumulate integrands of integrals that share the
+                # same compiler data
                 integrands_and_cds = \
                     accumulate_integrands_with_same_metadata(ss_integrals)
 
@@ -293,6 +317,7 @@ def group_form_integrals(form, domains):
                                               subdomain_id, metadata, None))
     return Form(integrals)
 
+
 def reconstruct_form_from_integral_data(integral_data):
     integrals = []
     for ida in integral_data:
