@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """Algorithms for estimating polynomial degrees of expressions."""
 
-# Copyright (C) 2008-2015 Martin Sandve Alnæs and Anders Logg
+# Copyright (C) 2008-2016 Martin Sandve Alnæs and Anders Logg
 #
 # This file is part of UFL.
 #
@@ -21,7 +21,6 @@
 # Modified by Anders Logg, 2009-2010
 # Modified by Jan Blechta, 2012
 
-from ufl.assertions import ufl_assert
 from ufl.log import warning, error
 from ufl.form import Form
 from ufl.integral import Integral
@@ -30,6 +29,14 @@ from ufl.corealg.map_dag import map_expr_dags
 from ufl.checks import is_cellwise_constant
 
 
+class IrreducibleInt(int):
+    """Degree type used by quadrilaterals.
+
+    Unlike int, values of this type are not decremeneted by _reduce_degree.
+    """
+    pass
+
+
 class SumDegreeEstimator(MultiFunction):
     "This algorithm is exact for a few operators and heuristic for many."
 
@@ -61,44 +68,56 @@ class SumDegreeEstimator(MultiFunction):
     def argument(self, v):
         """A form argument provides a degree depending on the element,
         or the default degree if the element has no degree."""
-        return v.ufl_element().degree() # FIXME: Use component to improve accuracy for mixed elements
+        return v.ufl_element().degree()  # FIXME: Use component to improve accuracy for mixed elements
 
     def coefficient(self, v):
         """A form argument provides a degree depending on the element,
         or the default degree if the element has no degree."""
         e = v.ufl_element()
         e = self.element_replace_map.get(e, e)
-        d = e.degree() # FIXME: Use component to improve accuracy for mixed elements
+        d = e.degree()  # FIXME: Use component to improve accuracy for mixed elements
         if d is None:
             d = self.default_degree
         return d
 
     def _reduce_degree(self, v, f):
         """Reduces the estimated degree by one; used when derivatives
-        are taken. Does not reduce the degree when OuterProduct elements
-        are involved."""
-        if isinstance(f, int):
+        are taken. Does not reduce the degree when TensorProduct elements
+        or quadrilateral elements are involved."""
+        if isinstance(f, int) and not isinstance(f, IrreducibleInt):
             return max(f-1, 0)
         else:
             # if tuple, do not reduce
             return f
 
     def _add_degrees(self, v, *ops):
-        if all(isinstance(o, int) for o in ops):
-            return sum(ops)
-        else:
+        def add_single(ops):
+            if any(isinstance(o, IrreducibleInt) for o in ops):
+                return IrreducibleInt(sum(ops))
+            else:
+                return sum(ops)
+
+        if any(isinstance(o, tuple) for o in ops):
             # we can add a slight hack here to handle things
             # like adding 0 to (3, 3) [by expanding
             # 0 to (0, 0) when making tempops]
             tempops = [foo if isinstance(foo, tuple) else (foo, foo) for foo in ops]
-            return tuple(map(sum, zip(*tempops)))
+            return tuple(map(add_single, zip(*tempops)))
+        else:
+            return add_single(ops)
 
     def _max_degrees(self, v, *ops):
-        if all(isinstance(o, int) for o in ops):
-            return max(ops + (0,))
-        else:
+        def max_single(ops):
+            if any(isinstance(o, IrreducibleInt) for o in ops):
+                return IrreducibleInt(max(ops))
+            else:
+                return max(ops)
+
+        if any(isinstance(o, tuple) for o in ops):
             tempops = [foo if isinstance(foo, tuple) else (foo, foo) for foo in ops]
-            return tuple(map(max, zip(*tempops)))
+            return tuple(map(max_single, zip(*tempops)))
+        else:
+            return max_single(ops + (0,))
 
     def _not_handled(self, v, *args):
         error("Missing degree handler for type %s" % v._ufl_class_.__name__)
@@ -111,35 +130,47 @@ class SumDegreeEstimator(MultiFunction):
     # Utility types with no degree concept
     def multi_index(self, v):
         return None
+
     def label(self, v):
         return None
+
     # Fall-through, indexing and similar types
     def reference_value(self, rv, f):
         return f
+
     def variable(self, v, e, l):
         return e
+
     def transposed(self, v, A):
         return A
+
     def index_sum(self, v, A, ii):
         return A
+
     def indexed(self, v, A, ii):
         return A
+
     def component_tensor(self, v, A, ii):
         return A
+
     list_tensor = _max_degrees
+
     def positive_restricted(self, v, a):
         return a
+
     def negative_restricted(self, v, a):
         return a
 
     # A sum takes the max degree of its operands:
     sum = _max_degrees
 
-    # TODO: Need a new algorithm which considers direction of derivatives of form arguments
-    # A spatial derivative reduces the degree with one
+    # TODO: Need a new algorithm which considers direction of
+    # derivatives of form arguments A spatial derivative reduces the
+    # degree with one
     grad = _reduce_degree
     reference_grad = _reduce_degree
-    # Handling these types although they should not occur... please apply preprocessing before using this algorithm:
+    # Handling these types although they should not occur... please
+    # apply preprocessing before using this algorithm:
     nabla_grad = _reduce_degree
     div = _reduce_degree
     reference_div = _reduce_degree
@@ -157,16 +188,18 @@ class SumDegreeEstimator(MultiFunction):
 
     # A product accumulates the degrees of its operands:
     product = _add_degrees
-    # Handling these types although they should not occur... please apply preprocessing before using this algorithm:
+    # Handling these types although they should not occur... please
+    # apply preprocessing before using this algorithm:
     inner = _add_degrees
     dot = _add_degrees
     outer = _add_degrees
     cross = _add_degrees
 
-    # Explicitly not handling these types, please apply preprocessing before using this algorithm:
-    derivative = _not_handled # base type
-    compound_derivative = _not_handled # base type
-    compound_tensor_operator = _not_handled # base class
+    # Explicitly not handling these types, please apply preprocessing
+    # before using this algorithm:
+    derivative = _not_handled  # base type
+    compound_derivative = _not_handled  # base type
+    compound_tensor_operator = _not_handled  # base class
     variable_derivative = _not_handled
     trace = _not_handled
     determinant = _not_handled
@@ -201,7 +234,8 @@ class SumDegreeEstimator(MultiFunction):
                 return tuple(foo*gi for foo in a)
         except:
             pass
-        # Something to a non-integer power, this is just a heuristic with no background
+        # Something to a non-integer power, this is just a heuristic
+        # with no background
         if isinstance(a, int):
             return a*2
         else:
@@ -214,13 +248,11 @@ class SumDegreeEstimator(MultiFunction):
         which can be wildly inaccurate but at least
         gives a somewhat high integration degree.
         """
-        #print "estimate",a,b
         if a or b:
             return self._add_degrees(v, self._max_degrees(v, a, b), 2)
         else:
             return self._max_degrees(v, a, b)
 
-
     def math_function(self, v, a):
         """Using the heuristic
         degree(sin(const)) == 0
@@ -266,7 +298,8 @@ class SumDegreeEstimator(MultiFunction):
     max_value = min_value
 
 
-def estimate_total_polynomial_degree(e, default_degree=1, element_replace_map={}):
+def estimate_total_polynomial_degree(e, default_degree=1,
+                                     element_replace_map={}):
     """Estimate total polynomial degree of integrand.
 
     NB! Although some compound types are supported here,
@@ -279,7 +312,8 @@ def estimate_total_polynomial_degree(e, default_degree=1, element_replace_map={}
     """
     de = SumDegreeEstimator(default_degree, element_replace_map)
     if isinstance(e, Form):
-        ufl_assert(e.integrals(), "Got form with no integrals!")
+        if not e.integrals():
+            error("Got form with no integrals!")
         degrees = map_expr_dags(de, [it.integrand() for it in e.integrals()])
     elif isinstance(e, Integral):
         degrees = map_expr_dags(de, [e.integrand()])
@@ -287,31 +321,3 @@ def estimate_total_polynomial_degree(e, default_degree=1, element_replace_map={}
         degrees = map_expr_dags(de, [e])
     degree = max(degrees) if degrees else default_degree
     return degree
-
-
-# TODO: Do these contain useful ideas or should we just delete them?
-
-
-def __unused__extract_max_quadrature_element_degree(integral):
-    """Extract quadrature integration order from quadrature
-    elements in integral. Returns None if not found."""
-    quadrature_elements = [e for e in extract_elements(integral) if "Quadrature" in e.family()]
-    degrees = [element.degree() for element in quadrature_elements]
-    degrees = [q for q in degrees if q is not None]
-    if not degrees:
-        return None
-    max_degree = quadrature_elements[0].degree()
-    ufl_assert(all(max_degree == q for q in degrees),
-               "Incompatible quadrature elements specified (orders must be equal).")
-    return max_degree
-
-
-def __unused__estimate_quadrature_degree(integral):
-    "Estimate the necessary quadrature order for integral using the sum of argument degrees."
-    arguments = extract_arguments(integral)
-    degrees = [v.ufl_element().degree() for v in arguments]
-    if len(arguments) == 0:
-        return None
-    if len(arguments) == 1:
-        return 2*degrees[0]
-    return sum(degrees)
