@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 "Algorithms for renumbering of counted objects, currently variables and indices."
 
-# Copyright (C) 2008-2015 Martin Sandve Alnæs and Anders Logg
+# Copyright (C) 2008-2016 Martin Sandve Alnæs and Anders Logg
 #
 # This file is part of UFL.
 #
@@ -19,13 +19,13 @@
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 
 from six.moves import zip
-from ufl.utils.stacks import Stack, StackDict
 from ufl.log import error
 from ufl.core.expr import Expr
 from ufl.core.multiindex import Index, FixedIndex, MultiIndex
 from ufl.variable import Label, Variable
 from ufl.algorithms.transformer import ReuseTransformer, apply_transformer
-from ufl.assertions import ufl_assert
+from ufl.classes import Zero
+
 
 class VariableRenumberingTransformer(ReuseTransformer):
     def __init__(self):
@@ -42,6 +42,7 @@ class VariableRenumberingTransformer(ReuseTransformer):
             self.variable_map[l] = v
         return v
 
+
 class IndexRenumberingTransformer(VariableRenumberingTransformer):
     "This is a poorly designed algorithm. It is used in some tests, please do not use for anything else."
     def __init__(self):
@@ -71,14 +72,14 @@ class IndexRenumberingTransformer(VariableRenumberingTransformer):
         new_indices = tuple(self.index(i) for i in o.indices())
         return MultiIndex(new_indices)
 
+
 def renumber_indices(expr):
     if isinstance(expr, Expr):
         num_free_indices = len(expr.ufl_free_indices)
-        #error("Not expecting any free indices left in expression.")
 
     result = apply_transformer(expr, IndexRenumberingTransformer())
 
     if isinstance(expr, Expr):
-        ufl_assert(num_free_indices == len(result.ufl_free_indices),
-                   "The number of free indices left in expression should be invariant w.r.t. renumbering.")
+        if num_free_indices != len(result.ufl_free_indices):
+            error("The number of free indices left in expression should be invariant w.r.t. renumbering.")
     return result
