@@ -2,7 +2,7 @@
 """This module defines the class Argument and a number of related
 classes (functions), including TestFunction and TrialFunction."""
 
-# Copyright (C) 2008-2015 Martin Sandve Alnæs
+# Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -20,19 +20,21 @@ classes (functions), including TestFunction and TrialFunction."""
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008-2009.
+# Modified by Massimiliano Leoni, 2016.
 
-from ufl.log import deprecate
-from ufl.assertions import ufl_assert
+import numbers
+from ufl.utils.py23 import as_native_str
+from ufl.utils.py23 import as_native_strings
+from ufl.log import error
 from ufl.core.ufl_type import ufl_type
-from ufl.core.terminal import Terminal, FormArgument
+from ufl.core.terminal import FormArgument
 from ufl.split_functions import split
 from ufl.finiteelement import FiniteElementBase
 from ufl.domain import default_domain
 from ufl.functionspace import AbstractFunctionSpace, FunctionSpace
 
-
 # Export list for ufl.classes (TODO: not actually classes: drop? these are in ufl.*)
-__all_classes__ = ["TestFunction", "TrialFunction", "TestFunctions", "TrialFunctions"]
+__all_classes__ = as_native_strings(["TestFunction", "TrialFunction", "TestFunctions", "TrialFunctions"])
 
 
 # --- Class representing an argument (basis function) in a form ---
@@ -40,13 +42,20 @@ __all_classes__ = ["TestFunction", "TrialFunction", "TestFunctions", "TrialFunct
 @ufl_type()
 class Argument(FormArgument):
     """UFL value: Representation of an argument to a form."""
-    __slots__ = ("_ufl_function_space", "_ufl_shape", "_number", "_part", "_repr")
+    __slots__ = as_native_strings((
+        "_ufl_function_space",
+        "_ufl_shape",
+        "_number",
+        "_part",
+        "_repr",
+    ))
 
     def __init__(self, function_space, number, part=None):
         FormArgument.__init__(self)
 
         if isinstance(function_space, FiniteElementBase):
-            # For legacy support for .ufl files using cells, we map the cell to The Default Mesh
+            # For legacy support for .ufl files using cells, we map the cell to
+            # the default Mesh
             element = function_space
             domain = default_domain(element.cell())
             function_space = FunctionSpace(domain, element)
@@ -56,17 +65,19 @@ class Argument(FormArgument):
         self._ufl_function_space = function_space
         self._ufl_shape = function_space.ufl_element().value_shape()
 
-        ufl_assert(isinstance(number, int),
-                   "Expecting an int for number, not %s" % (number,))
-        ufl_assert(part is None or isinstance(part, int),
-                   "Expecting None or an int for part, not %s" % (part,))
+        if not isinstance(number, numbers.Integral):
+            error("Expecting an int for number, not %s" % (number,))
+        if part is not None and not isinstance(part, numbers.Integral):
+            error("Expecting None or an int for part, not %s" % (part,))
         self._number = number
         self._part = part
 
-        self._repr = "Argument(%r, %r, %r)" % (self._ufl_function_space, self._number, self._part)
+        self._repr = as_native_str("Argument(%s, %s, %s)" % (
+            repr(self._ufl_function_space), repr(self._number), repr(self._part)))
 
     @property
     def ufl_shape(self):
+        "Return the associated UFL shape."
         return self._ufl_shape
 
     def ufl_function_space(self):
@@ -74,18 +85,24 @@ class Argument(FormArgument):
         return self._ufl_function_space
 
     def ufl_domain(self):
-        #TODO: deprecate("Argument.ufl_domain() is deprecated, please use .ufl_function_space().ufl_domain() instead.")
+        "Deprecated, please use .ufl_function_space().ufl_domain() instead."
+        # TODO: deprecate("Argument.ufl_domain() is deprecated, please
+        # use .ufl_function_space().ufl_domain() instead.")
         return self._ufl_function_space.ufl_domain()
 
     def ufl_element(self):
-        #TODO: deprecate("Argument.ufl_domain() is deprecated, please use .ufl_function_space().ufl_element() instead.")
+        "Deprecated, please use .ufl_function_space().ufl_element() instead."
+        # TODO: deprecate("Argument.ufl_domain() is deprecated, please
+        # use .ufl_function_space().ufl_element() instead.")
         return self._ufl_function_space.ufl_element()
 
-    def element(self):
-        deprecate("Argument.element() is deprecated, please use Argument.ufl_element() instead.")
-        return self.ufl_element()
+    # def element(self):
+    #    "Deprecated, please use .ufl_function_space().ufl_element() instead."
+    #    deprecate("Argument.element() is deprecated, please use Argument.ufl_element() instead.")
+    #    return self.ufl_element()
 
     def number(self):
+        "Return the Argument number."
         return self._number
 
     def part(self):
@@ -100,8 +117,9 @@ class Argument(FormArgument):
         return False
 
     def ufl_domains(self):
-        "Return tuple of domains related to this terminal object."
-        #TODO: deprecate("Argument.ufl_domains() is deprecated, please use .ufl_function_space().ufl_domains() instead.")
+        "Deprecated, please use .ufl_function_space().ufl_domains() instead."
+        # TODO: deprecate("Argument.ufl_domains() is deprecated,
+        # please use .ufl_function_space().ufl_domains() instead.")
         return self._ufl_function_space.ufl_domains()
 
     def _ufl_signature_data_(self, renumbering):
@@ -143,16 +161,19 @@ class Argument(FormArgument):
                 self._part == other._part and
                 self._ufl_function_space == other._ufl_function_space)
 
+
 # --- Helper functions for pretty syntax ---
 
 def TestFunction(function_space, part=None):
     """UFL value: Create a test function argument to a form."""
     return Argument(function_space, 0, part)
 
+
 def TrialFunction(function_space, part=None):
     """UFL value: Create a trial function argument to a form."""
     return Argument(function_space, 1, part)
 
+
 # --- Helper functions for creating subfunctions on mixed elements ---
 
 def Arguments(function_space, number):
@@ -160,11 +181,13 @@ def Arguments(function_space, number):
     tuple with the function components corresponding to the subelements."""
     return split(Argument(function_space, number))
 
+
 def TestFunctions(function_space):
     """UFL value: Create a TestFunction in a mixed space, and return a
     tuple with the function components corresponding to the subelements."""
     return Arguments(function_space, 0)
 
+
 def TrialFunctions(function_space):
     """UFL value: Create a TrialFunction in a mixed space, and return a
     tuple with the function components corresponding to the subelements."""
