@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 "Types for representing a cell."
 
-# Copyright (C) 2008-2015 Martin Sandve Alnæs
+# Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -22,32 +22,39 @@
 # Modified by Kristian B. Oelgaard, 2009
 # Modified by Marie E. Rognes 2012
 # Modified by Andrew T. T. McRae, 2014
+# Modified by Massimiliano Leoni, 2016
 
-from ufl.log import warning, error
-from ufl.assertions import ufl_assert
-from ufl.utils.formatting import istr
-from ufl.utils.dicts import EmptyDict
-from ufl.core.terminal import Terminal
+from six.moves import reduce
+from six import string_types
+import numbers
+
+from ufl.utils.py23 import as_native_str
+from ufl.utils.py23 import as_native_strings
+from ufl.log import error
 from ufl.core.ufl_type import attach_operators_from_hash_data
 
 
 # Export list for ufl.classes
-__all_classes__ = ["AbstractCell", "Cell", "TensorProductCell", "OuterProductCell"]
+__all_classes__ = as_native_strings(["AbstractCell", "Cell", "TensorProductCell"])
 
 
 # --- The most abstract cell class, base class for other cell types
 
 class AbstractCell(object):
     "Representation of an abstract finite element cell with only the dimensions known."
-    __slots__ = ("_topological_dimension", "_geometric_dimension")
+    __slots__ = as_native_strings((
+        "_topological_dimension",
+        "_geometric_dimension",
+        ))
+
     def __init__(self, topological_dimension, geometric_dimension):
         # Validate dimensions
-        ufl_assert(isinstance(geometric_dimension, int),
-                   "Expecting integer geometric dimension, not '%r'" % (geometric_dimension,))
-        ufl_assert(isinstance(topological_dimension, int),
-                   "Expecting integer topological dimension, not '%r'" % (topological_dimension,))
-        ufl_assert(topological_dimension <= geometric_dimension,
-                   "Topological dimension cannot be larger than geometric dimension.")
+        if not isinstance(geometric_dimension, numbers.Integral):
+            error("Expecting integer geometric_dimension.")
+        if not isinstance(topological_dimension, numbers.Integral):
+            error("Expecting integer topological_dimension.")
+        if topological_dimension > geometric_dimension:
+            error("Topological dimension cannot be larger than geometric dimension.")
 
         # Store validated dimensions
         self._topological_dimension = topological_dimension
@@ -73,55 +80,60 @@ class AbstractCell(object):
         "Define an arbitrarily chosen but fixed sort order for all cells."
         if not isinstance(other, AbstractCell):
             return NotImplemented
-        # Sort by gdim first, tdim next, then whatever's left depending on the subclass
+        # Sort by gdim first, tdim next, then whatever's left
+        # depending on the subclass
         s = (self.geometric_dimension(), self.topological_dimension())
         o = (other.geometric_dimension(), other.topological_dimension())
         if s != o:
             return s < o
         return self._ufl_hash_data_() < other._ufl_hash_data_()
 
+    def __unicode__(self):
+        # Only in python 2
+        return str(self).decode("utf-8")
+
 
 # --- Basic topological properties of known basic cells
 
-# Mapping from cell name to number of cell entities of each topological dimension
-num_cell_entities = {
-    "vertex":        (1,),
-    "interval":      (2,  1),
-    "triangle":      (3,  3, 1),
-    "quadrilateral": (4,  4, 1),
-    "tetrahedron":   (4,  6, 4, 1),
-    "hexahedron":    (8, 12, 6, 1),
-    }
+# Mapping from cell name to number of cell entities of each
+# topological dimension
+num_cell_entities = {"vertex": (1,),
+                     "interval": (2, 1),
+                     "triangle": (3, 3, 1),
+                     "quadrilateral": (4, 4, 1),
+                     "tetrahedron": (4, 6, 4, 1),
+                     "hexahedron": (8, 12, 6, 1)}
 
 # Mapping from cell name to topological dimension
-cellname2dim = dict((k, len(v)-1) for k,v in num_cell_entities.items())
+cellname2dim = dict((k, len(v)-1) for k, v in num_cell_entities.items())
 
 # Mapping from cell name to facet name
-# Note: This is not generalizable to product elements but it's still in use a couple of places.
-cellname2facetname = {
-    "interval":      "vertex",
-    "triangle":      "interval",
-    "quadrilateral": "interval",
-    "tetrahedron":   "triangle",
-    "hexahedron":    "quadrilateral",
-    }
+# Note: This is not generalizable to product elements but it's still
+# in use a couple of places.
+cellname2facetname = {"interval": "vertex",
+                      "triangle": "interval",
+                      "quadrilateral": "interval",
+                      "tetrahedron": "triangle",
+                      "hexahedron": "quadrilateral"}
 
 
 # --- Basic cell representation classes
 
+# @six.python_2_unicode_compatible
 @attach_operators_from_hash_data
 class Cell(AbstractCell):
     "Representation of a named finite element cell with known structure."
-    __slots__ = ("_cellname",)
+    __slots__ = as_native_strings(("_cellname",))
+
     def __init__(self, cellname, geometric_dimension=None):
         "Initialize basic cell description."
 
         self._cellname = cellname
 
-        # The topological dimension is defined by the cell type,
-        # so the cellname must be among the known ones,
-        # so we can find the known dimension, unless we have
-        # a product cell, in which the given dimension is used
+        # The topological dimension is defined by the cell type, so
+        # the cellname must be among the known ones, so we can find
+        # the known dimension, unless we have a product cell, in which
+        # the given dimension is used
         topological_dimension = len(num_cell_entities[cellname]) - 1
 
         # The geometric dimension defaults to equal the topological
@@ -134,10 +146,17 @@ class Cell(AbstractCell):
 
     # --- Overrides of AbstractCell methods ---
 
+    def reconstruct(self, geometric_dimension=None):
+        if geometric_dimension is None:
+            geometric_dimension = self._geometric_dimension
+        return Cell(self._cellname, geometric_dimension=geometric_dimension)
+
     def is_simplex(self):
+        "    Return True if this is a simplex cell."
         return self.num_vertices() == self.topological_dimension() + 1
 
     def has_simplex_facets(self):
+        "Return True if all the facets of this cell are simplex cells."
         return self.is_simplex() or self.cellname() == "quadrilateral"
 
     # --- Specific cell properties ---
@@ -178,32 +197,54 @@ class Cell(AbstractCell):
         return s
 
     def __repr__(self):
-        # For standard cells, return name of builtin cell object if possible.
-        # This reduces the size of the repr strings for domains, elements, etc. as well
+        # For standard cells, return name of builtin cell object if
+        # possible.  This reduces the size of the repr strings for
+        # domains, elements, etc. as well
         gdim = self.geometric_dimension()
         tdim = self.topological_dimension()
         name = self.cellname()
         if gdim == tdim and name in cellname2dim:
-            return name
+            r = name
         else:
-            return "Cell(%r, %r)" % (name, gdim)
+            r = "Cell(%s, %s)" % (repr(name), repr(gdim))
+        return as_native_str(r)
 
     def _ufl_hash_data_(self):
-        return (self._geometric_dimension, self._topological_dimension, self._cellname)
+        return (self._geometric_dimension, self._topological_dimension,
+                self._cellname)
 
+
+# @six.python_2_unicode_compatible
 @attach_operators_from_hash_data
 class TensorProductCell(AbstractCell):
-    __slots__ = ("_cells",)
-    def __init__(self, cells):
+    __slots__ = as_native_strings(("_cells",))
+
+    def __init__(self, *cells, **kwargs):
+        keywords = list(kwargs.keys())
+        if keywords and keywords != ["geometric_dimension"]:
+            raise ValueError(
+                "TensorProductCell got an unexpected keyword argument '%s'" %
+                keywords[0])
+
         self._cells = tuple(as_cell(cell) for cell in cells)
 
-        gdims = [cell.geometric_dimension() for cell in self._cells]
-        tdims = [cell.topological_dimension() for cell in self._cells]
-        gdim = sum(gdims)
-        tdim = sum(tdims)
+        tdim = sum([cell.topological_dimension() for cell in self._cells])
+        if kwargs:
+            gdim = kwargs["geometric_dimension"]
+        else:
+            gdim = sum([cell.geometric_dimension() for cell in self._cells])
 
         AbstractCell.__init__(self, tdim, gdim)
 
+    def cellname(self):
+        "Return the cellname of the cell."
+        return " * ".join([cell._cellname for cell in self._cells])
+
+    def reconstruct(self, geometric_dimension=None):
+        if geometric_dimension is None:
+            geometric_dimension = self._geometric_dimension
+        return TensorProductCell(*(self._cells), geometric_dimension=geometric_dimension)
+
     def is_simplex(self):
         "Return True if this is a simplex cell."
         if len(self._cells) == 1:
@@ -218,7 +259,7 @@ class TensorProductCell(AbstractCell):
 
     def num_vertices(self):
         "The number of cell vertices."
-        return product(c.num_vertices() for c in self._cells)
+        return reduce(lambda x, y: x * y, [c.num_vertices() for c in self._cells])
 
     def num_edges(self):
         "The number of cell edges."
@@ -233,107 +274,51 @@ class TensorProductCell(AbstractCell):
         return self._cells
 
     def __str__(self):
-        return repr(self)
-
-    def __repr__(self):
-        return "TensorProductCell(%s)" % ", ".join(repr(c) for c in self._cells)
-
-    def _ufl_hash_data_(self):
-        return tuple(c._ufl_hash_data_() for c in self._cells)
-
-
-@attach_operators_from_hash_data
-class OuterProductCell(AbstractCell): # TODO: Remove this and use TensorProductCell instead
-    """Representation of a cell formed as the Cartesian product of
-    two existing cells"""
-    __slots__ = ("_A", "_B", "facet_horiz", "facet_vert")
-
-    def __init__(self, A, B, gdim=None):
-        self._A = A
-        self._B = B
-
-        tdim = A.topological_dimension() + B.topological_dimension()
-        # default gdim -- "only as big as it needs to be, but not smaller than A or B"
-        gdim_temp = max(A.geometric_dimension(),
-                        B.geometric_dimension(),
-                        A.topological_dimension() + B.topological_dimension())
-        if gdim is None:
-            # default gdim
-            gdim = gdim_temp
+        gdim = self.geometric_dimension()
+        tdim = self.topological_dimension()
+        reprs = ", ".join(repr(c) for c in self._cells)
+        if gdim == tdim:
+            gdimstr = ""
         else:
-            # otherwise, validate custom gdim
-            if not isinstance(gdim, int):
-                raise TypeError("gdim must be an integer")
-            if gdim < gdim_temp:
-                raise ValueError("gdim must be at least %d" % gdim_temp)
-
-        AbstractCell.__init__(self, tdim, gdim)
-
-        # facets for extruded cells
-        if B.cellname() == "interval":
-            self.facet_horiz = A
-            if A.topological_dimension() == 2:
-                self.facet_vert = OuterProductCell(Cell("interval"), Cell("interval"))
-            elif A.topological_dimension() == 1:
-                # Terminate this recursion somewhere!
-                self.facet_vert = Cell("interval")
-            else:
-                # Don't know how to extrude this
-                self.facet_vert = None
-
-    def is_simplex(self):
-        "Return True if this is a simplex cell."
-        return False
-
-    def has_simplex_facets(self):
-        "Return True if all the facets of this cell are simplex cells."
-        # Actually sometimes true
-        return False
-
-    def num_vertices(self):
-        "The number of cell vertices."
-        return self._A.num_vertices() * self._B.num_vertices()
-
-    def num_edges(self):
-        "The number of cell edges."
-        error("Not defined for OuterProductCell.")
-
-    def num_facets(self):
-        "The number of cell facets."
-        return self._A.num_facets() + self._B.num_facets()
+            gdimstr = ", geometric_dimension=%d" % gdim
+        r = "TensorProductCell(%s%s)" % (reprs, gdimstr)
+        return r
 
     def __repr__(self):
-        return "OuterProductCell(*%r)" % list([self._A, self._B])
+        return str(self)
 
     def _ufl_hash_data_(self):
-        return tuple(c._ufl_hash_data_() for c in (self._A, self._B))
+        return tuple(c._ufl_hash_data_() for c in self._cells) + (self._geometric_dimension,)
 
 
 # --- Utility conversion functions
 
-# Mapping from topological dimension to reference cell name for simplices
-_simplex_dim2cellname = {
-    0: "vertex",
-    1: "interval",
-    2: "triangle",
-    3: "tetrahedron",
-    }
+# Mapping from topological dimension to reference cell name for
+# simplices
+_simplex_dim2cellname = {0: "vertex",
+                         1: "interval",
+                         2: "triangle",
+                         3: "tetrahedron"}
+
+# Mapping from topological dimension to reference cell name for
+# hypercubes
+_hypercube_dim2cellname = {0: "vertex",
+                           1: "interval",
+                           2: "quadrilateral",
+                           3: "hexahedron"}
 
-# Mapping from topological dimension to reference cell name for hypercubes
-_hypercube_dim2cellname = {
-    0: "vertex",
-    1: "interval",
-    2: "quadrilateral",
-    3: "hexahedron",
-    }
 
 def simplex(topological_dimension, geometric_dimension=None):
     "Return a simplex cell of given dimension."
-    return Cell(_simplex_dim2cellname[topological_dimension], geometric_dimension)
+    return Cell(_simplex_dim2cellname[topological_dimension],
+                geometric_dimension)
+
 
 def hypercube(topological_dimension, geometric_dimension=None):
     "Return a hypercube cell of given dimension."
-    return Cell(_hypercube_dim2cellname[topological_dimension], geometric_dimension)
+    return Cell(_hypercube_dim2cellname[topological_dimension],
+                geometric_dimension)
+
 
 def as_cell(cell):
     """Convert any valid object to a Cell or return cell if it is already a Cell.
@@ -342,7 +327,7 @@ def as_cell(cell):
     """
     if isinstance(cell, AbstractCell):
         return cell
-    elif isinstance(cell, str):
+    elif isinstance(cell, string_types):
         return Cell(cell)
     elif isinstance(cell, tuple):
         return TensorProductCell(cell)
