@@ -2,7 +2,7 @@
 """This module defines the Coefficient class and a number
 of related classes, including Constant."""
 
-# Copyright (C) 2008-2015 Martin Sandve Alnæs
+# Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -20,11 +20,12 @@ of related classes, including Constant."""
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008-2009.
+# Modified by Massimiliano Leoni, 2016.
 
-from ufl.log import deprecate
-from ufl.assertions import ufl_assert
+from ufl.utils.py23 import as_native_str
+from ufl.log import error
 from ufl.core.ufl_type import ufl_type
-from ufl.core.terminal import Terminal, FormArgument
+from ufl.core.terminal import FormArgument
 from ufl.finiteelement import FiniteElementBase, FiniteElement, VectorElement, TensorElement
 from ufl.domain import as_domain, default_domain
 from ufl.functionspace import AbstractFunctionSpace, FunctionSpace
@@ -33,12 +34,14 @@ from ufl.utils.counted import counted_init
 
 # --- The Coefficient class represents a coefficient in a form ---
 
+
 @ufl_type()
 class Coefficient(FormArgument):
     """UFL form argument type: Representation of a form coefficient."""
 
-    # Slots are disabled here because they cause trouble in PyDOLFIN multiple inheritance pattern:
-    #__slots__ = ("_count", "_ufl_function_space", "_repr", "_ufl_shape")
+    # Slots are disabled here because they cause trouble in PyDOLFIN
+    # multiple inheritance pattern:
+    # __slots__ = ("_count", "_ufl_function_space", "_repr", "_ufl_shape")
     _ufl_noslots_ = True
     _globalcount = 0
 
@@ -47,7 +50,8 @@ class Coefficient(FormArgument):
         counted_init(self, count, Coefficient)
 
         if isinstance(function_space, FiniteElementBase):
-            # For legacy support for .ufl files using cells, we map the cell to The Default Mesh
+            # For legacy support for .ufl files using cells, we map
+            # the cell to The Default Mesh
             element = function_space
             domain = default_domain(element.cell())
             function_space = FunctionSpace(domain, element)
@@ -57,13 +61,15 @@ class Coefficient(FormArgument):
         self._ufl_function_space = function_space
         self._ufl_shape = function_space.ufl_element().value_shape()
 
-        self._repr = "Coefficient(%r, %r)" % (self._ufl_function_space, self._count)
+        self._repr = as_native_str("Coefficient(%s, %s)" % (
+            repr(self._ufl_function_space), repr(self._count)))
 
     def count(self):
         return self._count
 
     @property
     def ufl_shape(self):
+        "Return the associated UFL shape."
         return self._ufl_shape
 
     def ufl_function_space(self):
@@ -78,9 +84,10 @@ class Coefficient(FormArgument):
         "Shortcut to get the finite element of the function space of this coefficient."
         return self._ufl_function_space.ufl_element()
 
-    def element(self):
-        deprecate("Coefficient.element() is deprecated, please use Coefficient.ufl_element() instead.")
-        return self.ufl_element()
+    # def element(self):
+    #    "Deprecated, please use Coefficient.ufl_element() instead."
+    #    deprecate("Coefficient.element() is deprecated, please use Coefficient.ufl_element() instead.")
+    #    return self.ufl_element()
 
     def is_cellwise_constant(self):
         "Return whether this expression is spatially constant over each cell."
@@ -114,7 +121,9 @@ class Coefficient(FormArgument):
         return (self._count == other._count and
                 self._ufl_function_space == other._ufl_function_space)
 
-# --- Helper functions for defining constant coefficients without specifying element ---
+
+# --- Helper functions for defining constant coefficients without
+# --- specifying element ---
 
 def Constant(domain, count=None):
     """UFL value: Represents a globally constant scalar valued coefficient."""
@@ -123,6 +132,7 @@ def Constant(domain, count=None):
     fs = FunctionSpace(domain, element)
     return Coefficient(fs, count=count)
 
+
 def VectorConstant(domain, dim=None, count=None):
     """UFL value: Represents a globally constant vector valued coefficient."""
     domain = as_domain(domain)
@@ -130,13 +140,16 @@ def VectorConstant(domain, dim=None, count=None):
     fs = FunctionSpace(domain, element)
     return Coefficient(fs, count=count)
 
+
 def TensorConstant(domain, shape=None, symmetry=None, count=None):
     """UFL value: Represents a globally constant tensor valued coefficient."""
     domain = as_domain(domain)
-    element = TensorElement("Real", domain.ufl_cell(), 0, shape=shape, symmetry=symmetry)
+    element = TensorElement("Real", domain.ufl_cell(), 0, shape=shape,
+                            symmetry=symmetry)
     fs = FunctionSpace(domain, element)
     return Coefficient(fs, count=count)
 
+
 # --- Helper functions for subfunctions on mixed elements ---
 
 def Coefficients(function_space):
