@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-"Base class for all operators, i.e. non-terminal expr types."
 
-# Copyright (C) 2008-2015 Martin Sandve Alnæs
+# Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -19,26 +18,28 @@
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008
+# Modified by Massimiliano Leoni, 2016
 
-from six import iteritems
-
-from ufl.log import error
+from ufl.utils.py23 import as_native_str
+from ufl.utils.py23 import as_native_strings
 from ufl.core.expr import Expr
 from ufl.core.ufl_type import ufl_type
 
 
-#--- Base class for operator objects ---
+# --- Base class for operator objects ---
 
 @ufl_type(is_abstract=True, is_terminal=False)
 class Operator(Expr):
-    __slots__ = ("ufl_operands",)
+    "Base class for all operators, i.e. non-terminal expression types."
+    __slots__ = as_native_strings(("ufl_operands",))
 
     def __init__(self, operands=None):
         Expr.__init__(self)
 
-        # If operands is None, the type sets this itself. This is to get around
-        # some tricky too-fancy __new__/__init__ design in algebra.py, for now.
-        # It would be nicer to make the classes in algebra.py pass operands here.
+        # If operands is None, the type sets this itself. This is to
+        # get around some tricky too-fancy __new__/__init__ design in
+        # algebra.py, for now.  It would be nicer to make the classes
+        # in algebra.py pass operands here.
         if operands is not None:
             self.ufl_operands = operands
 
@@ -52,3 +53,10 @@ class Operator(Expr):
     def _ufl_compute_hash_(self):
         "Compute a hash code for this expression. Used by sets and dicts."
         return hash((self._ufl_typecode_,) + tuple(hash(o) for o in self.ufl_operands))
+
+    def __repr__(self):
+        "Default repr string construction for operators."
+        # This should work for most cases
+        r = "%s(%s)" % (self._ufl_class_.__name__,
+            ", ".join(repr(op) for op in self.ufl_operands))
+        return as_native_str(r)
