@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-"""This module defines the Terminal class, the superclass
-for all types that are terminal nodes in the expression trees."""
+"""This module defines the ``Terminal`` class, the superclass
+for all types that are terminal nodes in an expression tree."""
 
-# Copyright (C) 2008-2015 Martin Sandve Alnæs
+# Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -20,13 +20,14 @@ for all types that are terminal nodes in the expression trees."""
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008
+# Modified by Massimiliano Leoni, 2016
 
 from ufl.log import error, warning
-from ufl.assertions import ufl_assert
 from ufl.core.expr import Expr
 from ufl.core.ufl_type import ufl_type
 
-#--- Base class for terminal objects ---
+
+# --- Base class for terminal objects ---
 
 @ufl_type(is_abstract=True, is_terminal=True)
 class Terminal(Expr):
@@ -51,7 +52,7 @@ class Terminal(Expr):
         raise NotImplementedError("Missing implementation of domains().")
 
     def evaluate(self, x, mapping, component, index_values, derivatives=()):
-        "Get self from mapping and return the component asked for."
+        "Get *self* from *mapping* and return the component asked for."
         f = mapping.get(self)
         # No mapping, trying to evaluate self as a constant
         if f is None:
@@ -100,10 +101,12 @@ class Terminal(Expr):
         return repr(self) == repr(other)
 
 
-#--- Subgroups of terminals ---
+# --- Subgroups of terminals ---
 
 @ufl_type(is_abstract=True)
 class FormArgument(Terminal):
+    "An abstract class for a form argument."
     __slots__ = ()
+
     def __init__(self):
         Terminal.__init__(self)
