@@ -1,8 +1,9 @@
+# -*- coding: utf-8 -*-
 """This module attaches special functions to Expr.
 This way we avoid circular dependencies between e.g.
 Sum and its superclass Expr."""
 
-# Copyright (C) 2008-2014 Martin Sandve Alnes
+# Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -18,14 +19,15 @@ Sum and its superclass Expr."""
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
+#
+# Modified by Massimiliano Leoni, 2016.
 
 from itertools import chain
+import numbers
 
 from ufl.log import error
-from ufl.assertions import ufl_assert
-from ufl.common import mergedicts, subdict, StackDict
+from ufl.utils.stacks import StackDict
 from ufl.core.expr import Expr
-from ufl.core.operator import Operator
 from ufl.constantvalue import Zero, as_ufl
 from ufl.algebra import Sum, Product, Division, Power, Abs
 from ufl.tensoralgebra import Transposed, Inner
@@ -37,54 +39,74 @@ from ufl.restriction import PositiveRestricted, NegativeRestricted
 from ufl.differentiation import Grad
 from ufl.index_combination_utils import create_slice_indices, merge_overlapping_indices
 
+from ufl.exprequals import expr_equals
 
-#--- Boolean operators ---
+# --- Boolean operators ---
+
+from ufl.conditional import LE, GE, LT, GT
 
-from ufl.conditional import EQ, NE, LE, GE, LT, GT
 
 def _le(left, right):
     "UFL operator: A boolean expresion (left <= right) for use with conditional."
     return LE(left, right)
 
+
 def _ge(left, right):
     "UFL operator: A boolean expresion (left >= right) for use with conditional."
     return GE(left, right)
 
+
 def _lt(left, right):
     "UFL operator: A boolean expresion (left < right) for use with conditional."
     return LT(left, right)
 
+
 def _gt(left, right):
     "UFL operator: A boolean expresion (left > right) for use with conditional."
     return GT(left, right)
 
-# '==' needs to implement comparison of expression representations for use in
-# hashmaps (dict and set), but the others can be overloaded in the language.
-# It is possible that we can overload eq as well, but we'll need to fix some
-# issues first and also check for a possible significant performance hit with
-# compilation of complex forms. Replacing a==b with equiv(a,b) all over the
-# code could be one way to reduce such a performance hit, but we cannot do
-# anything about dict and set calling __eq__...
-from ufl.exprequals import expr_equals
+
+# '==' needs to implement comparison of expression representations for
+# use in hashmaps (dict and set), but the others can be overloaded in
+# the language.  It is possible that we can overload eq as well, but
+# we'll need to fix some issues first and also check for a possible
+# significant performance hit with compilation of complex
+# forms. Replacing a==b with equiv(a,b) all over the code could be one
+# way to reduce such a performance hit, but we cannot do anything
+# about dict and set calling __eq__...
 Expr.__eq__ = expr_equals
 
-# != is used at least by tests, possibly in code as well, and must mean
-# the opposite of ==, i.e. when evaluated as bool it must mean 'not equal representation'.
+
+# != is used at least by tests, possibly in code as well, and must
+# mean the opposite of ==, i.e. when evaluated as bool it must mean
+# 'not equal representation'.
 def _ne(self, other):
     return not self.__eq__(other)
-Expr.__ne__ = _ne
 
+
+Expr.__ne__ = _ne
 Expr.__lt__ = _lt
 Expr.__gt__ = _gt
 Expr.__le__ = _le
 Expr.__ge__ = _ge
 
-#Expr.__and__ = And
-#Expr.__or__ = Or
-#Expr.__xor__ = Xor
+# Python operators 'and'/'or' cannot be overloaded, and bitwise
+# operators &/| don't have the right precedence levels
+# Expr.__and__ = _and
+# Expr.__or__ = _or
 
 
-#--- Helper functions for product handling ---
+def _as_tensor(self, indices):
+    "UFL operator: A^indices := as_tensor(A, indices)."
+    if not isinstance(indices, tuple):
+        error("Expecting a tuple of Index objects to A^indices := as_tensor(A, indices).")
+    if not all(isinstance(i, Index) for i in indices):
+        error("Expecting a tuple of Index objects to A^indices := as_tensor(A, indices).")
+    return as_tensor(self, indices)
+Expr.__xor__ = _as_tensor
+
+
+# --- Helper functions for product handling ---
 
 def _mult(a, b):
     # Discover repeated indices, which results in index sums
@@ -116,14 +138,15 @@ def _mult(a, b):
             return Zero(shape, fi, fid)
 
         # Repeated indices are allowed, like in:
-        #v[i]*M[i,:]
+        # v[i]*M[i,:]
 
         # Apply product to scalar components
-        ti = indices(b.rank())
+        ti = indices(len(b.ufl_shape))
         p = Product(a, b[ti])
 
-    elif r1 == 2 and r2 in (1, 2): # Matrix-matrix or matrix-vector
-        ufl_assert(not ri, "Not expecting repeated indices in non-scalar product.")
+    elif r1 == 2 and r2 in (1, 2):  # Matrix-matrix or matrix-vector
+        if ri:
+            error("Not expecting repeated indices in non-scalar product.")
 
         # Check for zero, simplifying early if possible
         if isinstance(a, Zero) or isinstance(b, Zero):
@@ -131,8 +154,8 @@ def _mult(a, b):
             return Zero(shape, fi, fid)
 
         # Return dot product in index notation
-        ai = indices(a.rank() - 1)
-        bi = indices(b.rank() - 1)
+        ai = indices(len(a.ufl_shape) - 1)
+        bi = indices(len(b.ufl_shape) - 1)
         k = indices(1)
 
         p = a[ai + k] * b[k + bi]
@@ -141,21 +164,24 @@ def _mult(a, b):
     else:
         error("Invalid ranks {0} and {1} in product.".format(r1, r2))
 
-    # TODO: I think applying as_tensor after index sums results in cleaner expression graphs.
+    # TODO: I think applying as_tensor after index sums results in
+    # cleaner expression graphs.
     # Wrap as tensor again
     if ti:
         p = as_tensor(p, ti)
 
-    # If any repeated indices were found, apply implicit summation over those
+    # If any repeated indices were found, apply implicit summation
+    # over those
     for i in ri:
         mi = MultiIndex((Index(count=i),))
         p = IndexSum(p, mi)
 
     return p
 
-#--- Extend Expr with algebraic operators ---
+# --- Extend Expr with algebraic operators ---
+
+_valid_types = (Expr, numbers.Real, numbers.Integral)
 
-_valid_types = (Expr,) + (int, float)
 
 def _mul(self, o):
     if not isinstance(o, _valid_types):
@@ -164,6 +190,7 @@ def _mul(self, o):
     return _mult(self, o)
 Expr.__mul__ = _mul
 
+
 def _rmul(self, o):
     if not isinstance(o, _valid_types):
         return NotImplemented
@@ -171,30 +198,39 @@ def _rmul(self, o):
     return _mult(o, self)
 Expr.__rmul__ = _rmul
 
+
 def _add(self, o):
     if not isinstance(o, _valid_types):
         return NotImplemented
     return Sum(self, o)
 Expr.__add__ = _add
 
+
 def _radd(self, o):
     if not isinstance(o, _valid_types):
         return NotImplemented
+    if isinstance(o, numbers.Number) and o == 0:
+        # Allow adding scalar int 0 as a no-op, even for shaped self,
+        # needed for sum([a,b])
+        return self
     return Sum(o, self)
 Expr.__radd__ = _radd
 
+
 def _sub(self, o):
     if not isinstance(o, _valid_types):
         return NotImplemented
     return Sum(self, -o)
 Expr.__sub__ = _sub
 
+
 def _rsub(self, o):
     if not isinstance(o, _valid_types):
         return NotImplemented
     return Sum(o, -self)
 Expr.__rsub__ = _rsub
 
+
 def _div(self, o):
     if not isinstance(o, _valid_types):
         return NotImplemented
@@ -207,6 +243,7 @@ def _div(self, o):
 Expr.__div__ = _div
 Expr.__truediv__ = _div
 
+
 def _rdiv(self, o):
     if not isinstance(o, _valid_types):
         return NotImplemented
@@ -214,6 +251,7 @@ def _rdiv(self, o):
 Expr.__rdiv__ = _rdiv
 Expr.__rtruediv__ = _rdiv
 
+
 def _pow(self, o):
     if not isinstance(o, _valid_types):
         return NotImplemented
@@ -222,33 +260,38 @@ def _pow(self, o):
     return Power(self, o)
 Expr.__pow__ = _pow
 
+
 def _rpow(self, o):
     if not isinstance(o, _valid_types):
         return NotImplemented
     return Power(o, self)
 Expr.__rpow__ = _rpow
 
+
 # TODO: Add Negated class for this? Might simplify reductions in Add.
 def _neg(self):
     return -1*self
 Expr.__neg__ = _neg
 
+
 def _abs(self):
     return Abs(self)
 Expr.__abs__ = _abs
 
-#--- Extend Expr with restiction operators a("+"), a("-") ---
+
+# --- Extend Expr with restiction operators a("+"), a("-") ---
 
 def _restrict(self, side):
     if side == "+":
         return PositiveRestricted(self)
     if side == "-":
         return NegativeRestricted(self)
-    error("Invalid side %r in restriction operator." % side)
+    error("Invalid side '%s' in restriction operator." % (side,))
+
 
 def _eval(self, coord, mapping=None, component=()):
-    # Evaluate expression at this particular coordinate,
-    # with provided values for other terminals in mapping
+    # Evaluate expression at this particular coordinate, with provided
+    # values for other terminals in mapping
 
     # Evaluate derivatives first
     from ufl.algorithms import expand_derivatives
@@ -260,24 +303,28 @@ def _eval(self, coord, mapping=None, component=()):
     index_values = StackDict()
     return f.evaluate(coord, mapping, component, index_values)
 
+
 def _call(self, arg, mapping=None, component=()):
     # Taking the restriction or evaluating depending on argument
     if arg in ("+", "-"):
-        ufl_assert(mapping is None, "Not expecting a mapping when taking restriction.")
+        if mapping is not None:
+            error("Not expecting a mapping when taking restriction.")
         return _restrict(self, arg)
     else:
         return _eval(self, arg, mapping, component)
 Expr.__call__ = _call
 
-#--- Extend Expr with the transpose operation A.T ---
+
+# --- Extend Expr with the transpose operation A.T ---
 
 def _transpose(self):
-    """Transposed a rank two tensor expression. For more general transpose
+    """Transpose a rank-2 tensor expression. For more general transpose
     operations of higher order tensor expressions, use indexing and Tensor."""
     return Transposed(self)
 Expr.T = property(_transpose)
 
-#--- Extend Expr with indexing operator a[i] ---
+
+# --- Extend Expr with indexing operator a[i] ---
 
 def analyse_key(ii, rank):
     """Takes something the user might input as an index tuple
@@ -302,7 +349,8 @@ def analyse_key(ii, rank):
     if not isinstance(ii, (tuple, MultiIndex)):
         ii = (ii,)
     else:
-        # Flatten nested tuples, happens with f[...,ii] where ii is a tuple of indices
+        # Flatten nested tuples, happens with f[...,ii] where ii is a
+        # tuple of indices
         jj = []
         for j in ii:
             if isinstance(j, (tuple, MultiIndex)):
@@ -311,20 +359,22 @@ def analyse_key(ii, rank):
                 jj.append(j)
         ii = tuple(jj)
 
-    # Convert all indices to Index or FixedIndex objects.
-    # If there is an ellipsis, split the indices into before and after.
+    # Convert all indices to Index or FixedIndex objects.  If there is
+    # an ellipsis, split the indices into before and after.
     axis_indices = set()
-    pre  = []
+    pre = []
     post = []
     indexlist = pre
     for i in ii:
         if i == Ellipsis:
-            # Switch from pre to post list when an ellipsis is encountered
-            ufl_assert(indexlist is pre, "Found duplicate ellipsis.")
+            # Switch from pre to post list when an ellipsis is
+            # encountered
+            if indexlist is not pre:
+                error("Found duplicate ellipsis.")
             indexlist = post
         else:
             # Convert index to a proper type
-            if isinstance(i, int):
+            if isinstance(i, numbers.Integral):
                 idx = FixedIndex(i)
             elif isinstance(i, IndexBase):
                 idx = i
@@ -336,13 +386,13 @@ def analyse_key(ii, rank):
                     # TODO: Use ListTensor to support partial slices?
                     error("Partial slices not implemented, only complete slices like [:]")
             else:
-                error("Can't convert this object to index: %r" % i)
+                error("Can't convert this object to index: %s" % (i,))
 
             # Store index in pre or post list
             indexlist.append(idx)
 
-    # Handle ellipsis as a number of complete slices,
-    # that is create a number of new axis indices
+    # Handle ellipsis as a number of complete slices, that is create a
+    # number of new axis indices
     num_axis = rank - len(pre) - len(post)
     if indexlist is post:
         ellipsis_indices = indices(num_axis)
@@ -355,6 +405,7 @@ def analyse_key(ii, rank):
     axis_indices = tuple(i for i in all_indices if i in axis_indices)
     return all_indices, axis_indices
 
+
 def _getitem(self, component):
 
     # Treat component consistently as tuple below
@@ -366,11 +417,13 @@ def _getitem(self, component):
     # Analyse slices (:) and Ellipsis (...)
     all_indices, slice_indices, repeated_indices = create_slice_indices(component, shape, self.ufl_free_indices)
 
-    # Check that we have the right number of indices for a tensor with this shape
+    # Check that we have the right number of indices for a tensor with
+    # this shape
     if len(shape) != len(all_indices):
         error("Invalid number of indices {0} for expression of rank {1}.".format(len(all_indices), len(shape)))
 
-    # Special case for simplifying foo[...] => foo, foo[:] => foo or similar
+    # Special case for simplifying foo[...] => foo, foo[:] => foo or
+    # similar
     if len(slice_indices) == len(all_indices):
         return self
 
@@ -379,23 +432,27 @@ def _getitem(self, component):
         if all_indices == self.indices().indices():
             return self.ufl_operands[0]
 
-    # Apply all indices to index self, yielding a scalar valued expression
+    # Apply all indices to index self, yielding a scalar valued
+    # expression
     mi = MultiIndex(all_indices)
     a = Indexed(self, mi)
 
-    # TODO: I think applying as_tensor after index sums results in cleaner expression graphs.
+    # TODO: I think applying as_tensor after index sums results in
+    # cleaner expression graphs.
 
-    # If the Ellipsis or any slices were found, wrap as tensor
-    # valued with the slice indices created at the top here
+    # If the Ellipsis or any slices were found, wrap as tensor valued
+    # with the slice indices created at the top here
     if slice_indices:
         a = as_tensor(a, slice_indices)
 
-    # If any repeated indices were found, apply implicit summation over those
+    # If any repeated indices were found, apply implicit summation
+    # over those
     for i in repeated_indices:
         mi = MultiIndex((i,))
         a = IndexSum(a, mi)
 
-    # Check for zero (last so we can get indices etc from a, could possibly be done faster by checking early instead)
+    # Check for zero (last so we can get indices etc from a, could
+    # possibly be done faster by checking early instead)
     if isinstance(self, Zero):
         shape = a.ufl_shape
         fi = a.ufl_free_indices
@@ -406,14 +463,19 @@ def _getitem(self, component):
 
 Expr.__getitem__ = _getitem
 
-#--- Extend Expr with spatial differentiation operator a.dx(i) ---
+
+# --- Extend Expr with spatial differentiation operator a.dx(i) ---
 
 def _dx(self, *ii):
-    "Return the partial derivative with respect to spatial variable number i."
+    "Return the partial derivative with respect to spatial variable number *ii*."
     d = self
+    # Unwrap ii to allow .dx(i,j) and .dx((i,j))
+    if len(ii) == 1 and isinstance(ii[0], tuple):
+        ii = ii[0]
     # Apply all derivatives
     for i in ii:
         d = Grad(d)
+
     # Take all components, applying repeated index sums in the [] operation
     return d.__getitem__((Ellipsis,) + ii)
 
