@@ -15,15 +15,20 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
+#
+# Modified by Massimiliano Leoni, 2016
 
+# import six
+from ufl.utils.py23 import as_native_str
 from ufl.finiteelement.finiteelementbase import FiniteElementBase
 
 
+# @six.python_2_unicode_compatible
 class BrokenElement(FiniteElementBase):
-    """The discontinuous version of an existing Finite Element space"""
+    """The discontinuous version of an existing Finite Element space."""
     def __init__(self, element):
         self._element = element
-        self._repr = "BrokenElement(%r)" % (element,)
+        self._repr = as_native_str("BrokenElement(%s)" % repr(element))
 
         family = "BrokenElement"
         cell = element.cell()
@@ -41,7 +46,5 @@ class BrokenElement(FiniteElementBase):
         return "BrokenElement(%s)" % str(self._element)
 
     def shortstr(self):
+        "Format as string for pretty printing."
         return "BrokenElement(%s)" % str(self._element.shortstr())
-
-    def __repr__(self):
-        return self._repr
