@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 "This module defines the UFL finite element classes."
 
-# Copyright (C) 2008-2015 Martin Sandve Alnæs
+# Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -20,56 +20,62 @@
 #
 # Modified by Kristian B. Oelgaard
 # Modified by Marie E. Rognes 2010, 2012
+# Modified by Massimiliano Leoni, 2016
 
+# import six
+from ufl.utils.py23 import as_native_str
 from six.moves import zip
-from ufl.assertions import ufl_assert
-from ufl.log import info_blue, warning, warning_blue, error
-
+from ufl.log import error
 from ufl.finiteelement.finiteelementbase import FiniteElementBase
 
+
+# @six.python_2_unicode_compatible
 class EnrichedElement(FiniteElementBase):
     """The vector sum of two finite element spaces:
 
-        EnrichedElement(V, Q) = {v + q | v in V, q in Q}.
+        .. math:: \\textrm{EnrichedElement}(V, Q) = \\{v + q | v \\in V, q \\in Q\\}.
     """
     def __init__(self, *elements):
         self._elements = elements
 
         cell = elements[0].cell()
-        ufl_assert(all(e.cell() == cell for e in elements),
-                   "Cell mismatch for sub elements of enriched element.")
+        if not all(e.cell() == cell for e in elements[1:]):
+            error("Cell mismatch for sub elements of enriched element.")
 
         if isinstance(elements[0].degree(), int):
-            degrees = { e.degree() for e in elements } - { None }
+            degrees = {e.degree() for e in elements} - {None}
             degree = max(degrees) if degrees else None
         else:
             degree = tuple(map(max, zip(*[e.degree() for e in elements])))
 
-        # We can allow the scheme not to be defined, but all defined should be equal
+        # We can allow the scheme not to be defined, but all defined
+        # should be equal
         quad_schemes = [e.quadrature_scheme() for e in elements]
         quad_schemes = [qs for qs in quad_schemes if qs is not None]
         quad_scheme = quad_schemes[0] if quad_schemes else None
-        ufl_assert(all(qs == quad_scheme for qs in quad_schemes),
-            "Quadrature scheme mismatch.")
+        if not all(qs == quad_scheme for qs in quad_schemes):
+            error("Quadrature scheme mismatch.")
 
         value_shape = elements[0].value_shape()
-        ufl_assert(all(e.value_shape() == value_shape for e in elements),
-                   "Element value shape mismatch.")
+        if not all(e.value_shape() == value_shape for e in elements[1:]):
+            error("Element value shape mismatch.")
 
         reference_value_shape = elements[0].reference_value_shape()
-        ufl_assert(all(e.reference_value_shape() == reference_value_shape for e in elements),
-                   "Element reference value shape mismatch.")
+        if not all(e.reference_value_shape() == reference_value_shape for e in elements[1:]):
+            error("Element reference value shape mismatch.")
 
-        #mapping = elements[0].mapping() # FIXME: This fails for a mixed subelement here.
-        #ufl_assert(all(e.mapping() == mapping for e in elements),
-        #           "Element mapping mismatch.")
+        # mapping = elements[0].mapping() # FIXME: This fails for a mixed subelement here.
+        # if not all(e.mapping() == mapping for e in elements[1:]):
+        #    error("Element mapping mismatch.")
 
         # Initialize element data
         FiniteElementBase.__init__(self, "EnrichedElement", cell, degree,
-                                   quad_scheme, value_shape, reference_value_shape)
+                                   quad_scheme, value_shape,
+                                   reference_value_shape)
 
         # Cache repr string
-        self._repr = "EnrichedElement(%s)" % ", ".join(repr(e) for e in self._elements)
+        self._repr = as_native_str("EnrichedElement(%s)" %
+            ", ".join(repr(e) for e in self._elements))
 
     def is_cellwise_constant(self):
         """Return whether the basis functions of this
