@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 "This module defines the UFL finite element classes."
 
-# Copyright (C) 2008-2015 Martin Sandve Alnæs
+# Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -21,35 +21,37 @@
 # Modified by Kristian B. Oelgaard
 # Modified by Marie E. Rognes 2010, 2012
 # Modified by Anders Logg 2014
+# Modified by Massimiliano Leoni, 2016
 
-from itertools import chain
-
+# import six
 from six import iteritems
 from six.moves import zip
 from six.moves import xrange as range
 
-from ufl.assertions import ufl_assert
+from ufl.log import error
+from ufl.utils.py23 import as_native_strings
 from ufl.permutation import compute_indices
 from ufl.utils.sequences import product
-from ufl.utils.formatting import istr
 from ufl.utils.dicts import EmptyDict
 from ufl.utils.indexflattening import flatten_multiindex, unflatten_index, shape_to_strides
 from ufl.cell import as_cell
-from ufl.log import info_blue, warning, warning_blue, error
 
 from ufl.finiteelement.finiteelementbase import FiniteElementBase
 from ufl.finiteelement.finiteelement import FiniteElement
 
 
+# @six.python_2_unicode_compatible
 class MixedElement(FiniteElementBase):
-    "A finite element composed of a nested hierarchy of mixed or simple elements"
-    __slots__ = ("_sub_elements", "_cells")
+    """A finite element composed of a nested hierarchy of mixed or simple
+    elements."""
+    __slots__ = as_native_strings(("_sub_elements", "_cells"))
 
     def __init__(self, *elements, **kwargs):
         "Create mixed finite element from given list of elements"
 
         if type(self) is MixedElement:
-            ufl_assert(not kwargs, "Not expecting keyword arguments to MixedElement constructor.")
+            if kwargs:
+                error("Not expecting keyword arguments to MixedElement constructor.")
 
         # Un-nest arguments if we get a single argument with a list of elements
         if len(elements) == 1 and isinstance(elements[0], (tuple, list)):
@@ -65,57 +67,59 @@ class MixedElement(FiniteElementBase):
         if cells:
             cell = cells[0]
             # Require that all elements are defined on the same cell
-            ufl_assert(all(c == cell for c in cells[1:]),
-                       "Sub elements must live on the same cell.")
+            if not all(c == cell for c in cells[1:]):
+                error("Sub elements must live on the same cell.")
         else:
             cell = None
 
-        # Check that all elements use the same quadrature scheme
-        # TODO: We can allow the scheme not to be defined.
+        # Check that all elements use the same quadrature scheme TODO:
+        # We can allow the scheme not to be defined.
         quad_scheme = elements[0].quadrature_scheme()
-        ufl_assert(all(e.quadrature_scheme() == quad_scheme for e in elements),
-            "Quadrature scheme mismatch for sub elements of mixed element.")
+        if not all(e.quadrature_scheme() == quad_scheme for e in elements):
+            error("Quadrature scheme mismatch for sub elements of mixed element.")
 
         # Compute value sizes in global and reference configurations
         value_size_sum = sum(product(s.value_shape()) for s in self._sub_elements)
         reference_value_size_sum = sum(product(s.reference_value_shape()) for s in self._sub_elements)
 
-        # Default value shape: Treated simply as all subelement values unpacked in a vector.
+        # Default value shape: Treated simply as all subelement values
+        # unpacked in a vector.
         value_shape = kwargs.get('value_shape', (value_size_sum,))
 
-        # Default reference value shape: Treated simply as all subelement reference values unpacked in a vector.
+        # Default reference value shape: Treated simply as all
+        # subelement reference values unpacked in a vector.
         reference_value_shape = kwargs.get('reference_value_shape', (reference_value_size_sum,))
 
-        # Validate value_shape (deliberately not for subclasses VectorElement and TensorElement)
+        # Validate value_shape (deliberately not for subclasses
+        # VectorElement and TensorElement)
         if type(self) is MixedElement:
             # This is not valid for tensor elements with symmetries,
             # assume subclasses deal with their own validation
-            ufl_assert(product(value_shape) == value_size_sum,
-                "Provided value_shape doesn't match the total "
-                "value size of all subelements.")
+            if product(value_shape) != value_size_sum:
+                error("Provided value_shape doesn't match the "
+                      "total value size of all subelements.")
 
         # Initialize element data
-        degrees = { e.degree() for e in self._sub_elements } - { None }
+        degrees = {e.degree() for e in self._sub_elements} - {None}
         degree = max(degrees) if degrees else None
         FiniteElementBase.__init__(self, "Mixed", cell, degree, quad_scheme,
                                    value_shape, reference_value_shape)
 
         # Cache repr string
         if type(self) is MixedElement:
-            self._repr = "MixedElement(%s)" % ", ".join(repr(e) for e in self._sub_elements)
+            self._repr = "MixedElement(%s)" % (
+                ", ".join(repr(e) for e in self._sub_elements),)
 
     def reconstruct_from_elements(self, *elements):
         "Reconstruct a mixed element from new subelements."
         if all(a == b for (a, b) in zip(elements, self._sub_elements)):
             return self
-        ufl_assert(all(a.value_shape() == b.value_shape()
-                       for (a, b) in zip(elements, self._sub_elements)),
-            "Expecting new elements to have same value shape as old ones.")
         return MixedElement(*elements)
 
     def symmetry(self):
-        """Return the symmetry dict, which is a mapping c0 -> c1
-        meaning that component c0 is represented by component c1.
+        """Return the symmetry dict, which is a mapping :math:`c_0 \\to c_1`
+        meaning that component :math:`c_0` is represented by component
+        :math:`c_1`.
         A component is a tuple of one or more ints."""
         # Build symmetry map from symmetries of subelements
         sm = {}
@@ -124,15 +128,16 @@ class MixedElement(FiniteElementBase):
         for e in self._sub_elements:
             sh = e.value_shape()
             st = shape_to_strides(sh)
-            # Map symmetries of subelement into index space of this element
+            # Map symmetries of subelement into index space of this
+            # element
             for c0, c1 in iteritems(e.symmetry()):
                 j0 = flatten_multiindex(c0, st) + j
                 j1 = flatten_multiindex(c1, st) + j
                 sm[(j0,)] = (j1,)
             # Update base index for next element
             j += product(sh)
-        ufl_assert(j == product(self.value_shape()),
-                   "Size mismatch in symmetry algorithm.")
+        if j != product(self.value_shape()):
+            error("Size mismatch in symmetry algorithm.")
         return sm or EmptyDict
 
     def mapping(self):
@@ -151,14 +156,15 @@ class MixedElement(FiniteElementBase):
 
     def extract_subelement_component(self, i):
         """Extract direct subelement index and subelement relative
-        component index for a given component index"""
+        component index for a given component index."""
         if isinstance(i, int):
             i = (i,)
         self._check_component(i)
 
         # Select between indexing modes
         if len(self.value_shape()) == 1:
-            # Indexing into a long vector of flattened subelement shapes
+            # Indexing into a long vector of flattened subelement
+            # shapes
             j, = i
 
             # Find subelement for this index
@@ -168,29 +174,30 @@ class MixedElement(FiniteElementBase):
                 if j < si:
                     break
                 j -= si
-            ufl_assert(j >= 0, "Moved past last value component!")
+            if j < 0:
+                error("Moved past last value component!")
 
             # Convert index into a shape tuple
             st = shape_to_strides(sh)
             component = unflatten_index(j, st)
         else:
-            # Indexing into a multidimensional tensor
-            # where subelement index is first axis
+            # Indexing into a multidimensional tensor where subelement
+            # index is first axis
             sub_element_index = i[0]
-            ufl_assert(sub_element_index < len(self._sub_elements),
-                       "Illegal component index (dimension %d)." % sub_element_index)
+            if sub_element_index >= len(self._sub_elements):
+                error("Illegal component index (dimension %d)." % sub_element_index)
             component = i[1:]
         return (sub_element_index, component)
 
     def extract_component(self, i):
         """Recursively extract component index relative to a (simple) element
-        and that element for given value component index"""
+        and that element for given value component index."""
         sub_element_index, component = self.extract_subelement_component(i)
         return self._sub_elements[sub_element_index].extract_component(component)
 
     def extract_subelement_reference_component(self, i):
         """Extract direct subelement index and subelement relative
-        reference_component index for a given reference_component index"""
+        reference_component index for a given reference_component index."""
         if isinstance(i, int):
             i = (i,)
         self._check_reference_component(i)
@@ -207,7 +214,8 @@ class MixedElement(FiniteElementBase):
             if j < si:
                 break
             j -= si
-        ufl_assert(j >= 0, "Moved past last value reference_component!")
+        if j < 0:
+            error("Moved past last value reference_component!")
 
         # Convert index into a shape tuple
         st = shape_to_strides(sh)
@@ -216,7 +224,7 @@ class MixedElement(FiniteElementBase):
 
     def extract_reference_component(self, i):
         """Recursively extract reference_component index relative to a (simple) element
-        and that element for given value reference_component index"""
+        and that element for given value reference_component index."""
         sub_element_index, reference_component = self.extract_subelement_reference_component(i)
         return self._sub_elements[sub_element_index].extract_reference_component(reference_component)
 
@@ -230,9 +238,9 @@ class MixedElement(FiniteElementBase):
             return e.is_cellwise_constant()
 
     def degree(self, component=None):
-        "Return polynomial degree of finite element"
+        "Return polynomial degree of finite element."
         if component is None:
-            return self._degree # from FiniteElementBase, computed as max of subelements in __init__
+            return self._degree  # from FiniteElementBase, computed as max of subelements in __init__
         else:
             i, e = self.extract_component(component)
             return e.degree()
@@ -248,8 +256,9 @@ class MixedElement(FiniteElementBase):
         return "Mixed<" + tmp + ">"
 
 
+# @six.python_2_unicode_compatible
 class VectorElement(MixedElement):
-    "A special case of a mixed finite element where all elements are equal"
+    "A special case of a mixed finite element where all elements are equal."
 
     def __init__(self, family, cell=None, degree=None, dim=None,
                  form_degree=None, quad_scheme=None):
@@ -285,8 +294,8 @@ class VectorElement(MixedElement):
 
         # Set default size if not specified
         if dim is None:
-            ufl_assert(cell is not None,
-                       "Cannot infer vector dimension without a cell.")
+            if cell is None:
+                error("Cannot infer vector dimension without a cell.")
             dim = cell.geometric_dimension()
 
         # Create list of sub elements for mixed element constructor
@@ -297,31 +306,37 @@ class VectorElement(MixedElement):
         reference_value_shape = (dim,) + sub_element.reference_value_shape()
 
         # Initialize element data
-        MixedElement.__init__(self, sub_elements, value_shape=value_shape, reference_value_shape=reference_value_shape)
-        # FIXME: Storing this here is strange, isn't that handled by subclass?
+        MixedElement.__init__(self, sub_elements, value_shape=value_shape,
+                              reference_value_shape=reference_value_shape)
+        # FIXME: Storing this here is strange, isn't that handled by
+        # subclass?
         self._family = sub_element.family()
         self._degree = sub_element.degree()
         self._sub_element = sub_element
-        self._form_degree = form_degree # Storing for signature_data, not sure if it's needed
+        self._form_degree = form_degree  # Storing for signature_data, not sure if it's needed
 
         # Cache repr string
-        self._repr = "VectorElement(%r, dim=%d)" % (sub_element, len(self._sub_elements))
+        self._repr = "VectorElement(%s, dim=%d)" % (
+            repr(sub_element), len(self._sub_elements))
 
     def __str__(self):
         "Format as string for pretty printing."
         return ("<vector element with %d components of %s>" %
-               (len(self._sub_elements), self._sub_element))
+                (len(self._sub_elements), self._sub_element))
 
     def shortstr(self):
         "Format as string for pretty printing."
         return "Vector<%d x %s>" % (len(self._sub_elements),
                                     self._sub_element.shortstr())
 
+
+# @six.python_2_unicode_compatible
 class TensorElement(MixedElement):
-    "A special case of a mixed finite element where all elements are equal"
-    __slots__ = ("_sub_element", "_shape", "_symmetry",
+    "A special case of a mixed finite element where all elements are equal."
+    __slots__ = as_native_strings(("_sub_element", "_shape", "_symmetry",
                  "_sub_element_mapping", "_flattened_sub_element_mapping",
-                 "_mapping")
+                 "_mapping"))
+
     def __init__(self, family, cell=None, degree=None, shape=None,
                  symmetry=None, quad_scheme=None):
         """Create tensor element (repeated mixed element with optional symmetries).
@@ -344,13 +359,13 @@ class TensorElement(MixedElement):
             # Create scalar sub element
             sub_element = FiniteElement(family, cell, degree, quad_scheme)
 
-        ufl_assert(sub_element.value_shape() == (),
-                   "Expecting only scalar valued subelement for TensorElement.")
+        if sub_element.value_shape() != ():
+            error("Expecting only scalar valued subelement for TensorElement.")
 
         # Set default shape if not specified
         if shape is None:
-            ufl_assert(cell is not None,
-                       "Cannot infer tensor shape without a cell.")
+            if cell is None:
+                error("Cannot infer tensor shape without a cell.")
             dim = cell.geometric_dimension()
             shape = (dim, dim)
 
@@ -358,26 +373,28 @@ class TensorElement(MixedElement):
             symmetry = EmptyDict
         elif symmetry is True:
             # Construct default symmetry dict for matrix elements
-            ufl_assert(len(shape) == 2 and shape[0] == shape[1],
-                       "Cannot set automatic symmetry for non-square tensor.")
-            symmetry = dict( ((i, j), (j, i)) for i in range(shape[0])
-                             for j in range(shape[1]) if i > j )
+            if not (len(shape) == 2 and shape[0] == shape[1]):
+                error("Cannot set automatic symmetry for non-square tensor.")
+            symmetry = dict(((i, j), (j, i)) for i in range(shape[0])
+                            for j in range(shape[1]) if i > j)
         else:
-            ufl_assert(isinstance(symmetry, dict), "Expecting symmetry to be None (unset), True, or dict.")
+            if not isinstance(symmetry, dict):
+                error("Expecting symmetry to be None (unset), True, or dict.")
 
         # Validate indices in symmetry dict
         for i, j in iteritems(symmetry):
-            ufl_assert(len(i) == len(j),
-                       "Non-matching length of symmetry index tuples.")
+            if len(i) != len(j):
+                error("Non-matching length of symmetry index tuples.")
             for k in range(len(i)):
-                ufl_assert(i[k] >= 0 and j[k] >= 0 and
-                           i[k] < shape[k] and j[k] < shape[k],
-                           "Symmetry dimensions out of bounds.")
+                if not (i[k] >= 0 and j[k] >= 0
+                        and i[k] < shape[k] and j[k] < shape[k]):
+                    error("Symmetry dimensions out of bounds.")
 
         # Compute all index combinations for given shape
         indices = compute_indices(shape)
 
-        # Compute mapping from indices to sub element number, accounting for symmetry
+        # Compute mapping from indices to sub element number,
+        # accounting for symmetry
         sub_elements = []
         sub_element_mapping = {}
         for index in indices:
@@ -390,7 +407,8 @@ class TensorElement(MixedElement):
         for index in indices:
             if index in symmetry:
                 sub_element_mapping[index] = sub_element_mapping[symmetry[index]]
-        flattened_sub_element_mapping = [sub_element_mapping[index] for i, index in enumerate(indices)]
+        flattened_sub_element_mapping = [sub_element_mapping[index] for i,
+                                         index in enumerate(indices)]
 
         # Compute value shape
         value_shape = shape
@@ -406,7 +424,8 @@ class TensorElement(MixedElement):
             self._mapping = "identity"
 
         # Initialize element data
-        MixedElement.__init__(self, sub_elements, value_shape=value_shape, reference_value_shape=reference_value_shape)
+        MixedElement.__init__(self, sub_elements, value_shape=value_shape,
+                              reference_value_shape=reference_value_shape)
         self._family = sub_element.family()
         self._degree = sub_element.degree()
         self._sub_element = sub_element
@@ -416,7 +435,8 @@ class TensorElement(MixedElement):
         self._flattened_sub_element_mapping = flattened_sub_element_mapping
 
         # Cache repr string
-        self._repr = "TensorElement(%r, shape=%r, symmetry=%r)" % (sub_element, self._shape, self._symmetry)
+        self._repr = "TensorElement(%s, shape=%s, symmetry=%s)" % (
+            repr(sub_element), repr(self._shape), repr(self._symmetry))
 
     def mapping(self):
         if self._symmetry:
@@ -429,7 +449,7 @@ class TensorElement(MixedElement):
 
     def extract_subelement_component(self, i):
         """Extract direct subelement index and subelement relative
-        component index for a given component index"""
+        component index for a given component index."""
         if isinstance(i, int):
             i = (i,)
         self._check_component(i)
@@ -438,14 +458,16 @@ class TensorElement(MixedElement):
         l = len(self._shape)
         ii = i[:l]
         jj = i[l:]
-        ufl_assert(ii in self._sub_element_mapping,
-                   "Illegal component index %s." % repr(i))
+        if ii not in self._sub_element_mapping:
+            error("Illegal component index %s." % (i,))
         k = self._sub_element_mapping[ii]
         return (k, jj)
 
     def symmetry(self):
-        """Return the symmetry dict, which is a mapping c0 -> c1
-        meaning that component c0 is represented by component c1."""
+        """Return the symmetry dict, which is a mapping :math:`c_0 \\to c_1`
+        meaning that component :math:`c_0` is represented by component
+        :math:`c_1`.
+        A component is a tuple of one or more ints."""
         return self._symmetry
 
     def __str__(self):
