@@ -1,6 +1,7 @@
+# -*- coding: utf-8 -*-
 "The Form class."
 
-# Copyright (C) 2008-2014 Martin Sandve Alnes
+# Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -18,31 +19,38 @@
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2009-2011.
+# Modified by Massimiliano Leoni, 2016.
 
-import hashlib
+# import six
 from itertools import chain
 from collections import defaultdict
-from ufl.log import error, deprecate
-from ufl.assertions import ufl_assert
-import ufl.measure
-from ufl.integral import Integral, Measure
+
+from ufl.log import error, warning
+from ufl.integral import Integral
 from ufl.checks import is_scalar_constant_expression
 from ufl.equation import Equation
 from ufl.core.expr import Expr
+from ufl.core.expr import ufl_err_str
 from ufl.constantvalue import Zero
-from ufl.protocols import id_or_none
+from ufl.utils.py23 import as_native_strings, as_native_str
+
+# Export list for ufl.classes
+__all_classes__ = as_native_strings(["Form"])
 
 # --- The Form class, representing a complete variational form or functional ---
 
+
 def _sorted_integrals(integrals):
     """Sort integrals by domain id, integral type, subdomain id
     for a more stable signature computation."""
 
-    # Group integrals in multilevel dict by keys [domain][integral_type][subdomain_id]
-    integrals_dict = defaultdict(lambda:defaultdict(lambda:defaultdict(list)))
+    # Group integrals in multilevel dict by keys
+    # [domain][integral_type][subdomain_id]
+    integrals_dict = defaultdict(lambda: defaultdict(lambda: defaultdict(list)))
     for integral in integrals:
-        d = integral.domain()
-        ufl_assert(d is not None, "An Integral without a Domain is now illegal.")
+        d = integral.ufl_domain()
+        if d is None:
+            error("Each integral in a form must have a uniquely defined integration domain.")
         it = integral.integral_type()
         si = integral.subdomain_id()
         integrals_dict[d][it][si] += [integral]
@@ -50,19 +58,25 @@ def _sorted_integrals(integrals):
     all_integrals = []
 
     # Order integrals canonically to increase signature stability
-    for d in sorted(integrals_dict): # Assuming Domain is sortable
-        for it in sorted(integrals_dict[d]): # str is sortable
-            for si in sorted(integrals_dict[d][it], key=lambda x: (type(x).__name__, x)): # int/str are sortable
+    for d in sorted(integrals_dict):  # Assuming Domain is sortable
+        for it in sorted(integrals_dict[d]):  # str is sortable
+            for si in sorted(integrals_dict[d][it],
+                             key=lambda x: (type(x).__name__, x)):  # int/str are sortable
                 unsorted_integrals = integrals_dict[d][it][si]
-                # TODO: At this point we could order integrals by metadata and integrand,
-                #       or even add the integrands with the same metadata. This is done
-                #       in accumulate_integrands_with_same_metadata in algorithms/domain_analysis.py
-                #       and would further increase the signature stability.
+                # TODO: At this point we could order integrals by
+                #       metadata and integrand, or even add the
+                #       integrands with the same metadata. This is
+                #       done in
+                #       accumulate_integrands_with_same_metadata in
+                #       algorithms/domain_analysis.py and would
+                #       further increase the signature stability.
                 all_integrals.extend(unsorted_integrals)
-                #integrals_dict[d][it][si] = unsorted_integrals
+                # integrals_dict[d][it][si] = unsorted_integrals
 
-    return tuple(all_integrals) #, integrals_dict
+    return tuple(all_integrals)  # integrals_dict
 
+
+# @six.python_2_unicode_compatible
 class Form(object):
     """Description of a weak form consisting of a sum of integrals over subdomains."""
     __slots__ = (
@@ -81,14 +95,16 @@ class Form(object):
         #     data in to be carried with the form
         #     Never use this internally in ufl!
         "_cache",
-        )
+    )
 
     def __init__(self, integrals):
-        # Basic input checking (further compatibilty analysis happens later)
-        ufl_assert(all(isinstance(itg, Integral) for itg in integrals),
-                   "Expecting list of integrals.")
+        # Basic input checking (further compatibilty analysis happens
+        # later)
+        if not all(isinstance(itg, Integral) for itg in integrals):
+            error("Expecting list of integrals.")
 
-        # Store integrals sorted canonically to increase signature stability
+        # Store integrals sorted canonically to increase signature
+        # stability
         self._integrals = _sorted_integrals(integrals)
 
         # Internal variables for caching domain data
@@ -103,7 +119,8 @@ class Form(object):
         self._coefficients = None
         self._coefficient_numbering = None
 
-        # Internal variables for caching of hash and signature after first request
+        # Internal variables for caching of hash and signature after
+        # first request
         self._hash = None
         self._signature = None
 
@@ -121,15 +138,16 @@ class Form(object):
         return tuple(integral for integral in self.integrals()
                      if integral.integral_type() == integral_type)
 
-    #def integrals_dict(self):
-    #    "Returns a mapping on the form { domain: { integral_type: { subdomain_id: integral_list } } }."
-    #    return self._integrals_dict
-
     def empty(self):
         "Returns whether the form has no integrals."
         return self.integrals() == ()
 
-    def domains(self):
+    #def domains(self):
+    #    "Deprecated, please use .ufl_domains() instead."
+    #    deprecate("Form.domains() is deprecated, please use .ufl_domains() instead.")
+    #    return self.ufl_domains()
+
+    def ufl_domains(self):
         """Return the geometric integration domains occuring in the form.
 
         NB! This does not include domains of coefficients defined on other meshes.
@@ -140,16 +158,21 @@ class Form(object):
             self._analyze_domains()
         return self._integration_domains
 
-    def cell(self):
-        "Return the single cell this form is defined on, fails if multiple cells are found."
-        domains = self.domains()
-        ufl_assert(all(domain.cell() == domains[0].cell() for domain in domains),
-                   "Calling Form.domain() is only valid if all integrals share domain.")
-        # Need to support missing domain to allow
-        # assemble(Constant(1)*dx, mesh=mesh) in dolfin
-        return domains[0].cell() if domains else None
+    #def cell(self):
+    #    "Deprecated, please use .ufl_cell() instead."
+    #    deprecate("Form.cell() is deprecated, please use .ufl_cell() instead.")
+    #    return self.ufl_cell()
 
-    def domain(self):
+    #def domain(self):
+    #    "Deprecated, please use .ufl_domain() instead."
+    #    deprecate("Form.domain() is deprecated, please use .ufl_domain() instead.")
+    #    return self.ufl_domain()
+
+    def ufl_cell(self):
+        "Return the single cell this form is defined on, fails if multiple cells are found."
+        return self.ufl_domain().ufl_cell()
+
+    def ufl_domain(self):
         """Return the single geometric integration domain occuring in the form.
 
         Fails if multiple domains are found.
@@ -157,45 +180,60 @@ class Form(object):
         NB! This does not include domains of coefficients defined on other
         meshes, look at form data for that additional information.
         """
-        domains = self.domains()
-        ufl_assert(all(domain == domains[0] for domain in domains),
-                   "Calling Form.domain() is only valid if all integrals share domain.")
-        # Need to support missing domain to allow
-        # assemble(Constant(1)*dx, mesh=mesh) in dolfin
-        return domains[0] if domains else None
+        # Collect all domains
+        domains = self.ufl_domains()
+        # Check that all are equal TODO: don't return more than one if
+        # all are equal?
+        if not all(domain == domains[0] for domain in domains):
+            error("Calling Form.ufl_domain() is only valid if all integrals share domain.")
+        # Return the one and only domain
+        return domains[0]
+
+    def geometric_dimension(self):
+        "Return the geometric dimension shared by all domains and functions in this form."
+        gdims = tuple(set(domain.geometric_dimension()
+                          for domain in self.ufl_domains()))
+        if len(gdims) != 1:
+            error("Expecting all domains and functions in a form "
+                  "to share geometric dimension, got %s." % str(tuple(sorted(gdims))))
+        return gdims[0]
 
     def domain_numbering(self):
-        "Return a contiguous numbering of domains in a mapping { domain: number }."
+        """Return a contiguous numbering of domains in a mapping
+        ``{domain:number}``."""
         if self._domain_numbering is None:
             self._analyze_domains()
         return self._domain_numbering
 
     def subdomain_data(self):
-        "Returns a mapping on the form { domain: { integral_type: subdomain_data } }."
+        """Returns a mapping on the form ``{domain:{integral_type:
+            subdomain_data}}``."""
         if self._subdomain_data is None:
             self._analyze_subdomain_data()
         return self._subdomain_data
 
     def max_subdomain_ids(self):
-        "Returns a mapping on the form { domain: { integral_type: max_subdomain_id } }."
+        """Returns a mapping on the form
+        ``{domain:{integral_type:max_subdomain_id}}``."""
         if self._max_subdomain_ids is None:
             self._analyze_subdomain_data()
         return self._max_subdomain_ids
 
     def arguments(self):
-        "Return all Argument objects found in form."
+        "Return all ``Argument`` objects found in form."
         if self._arguments is None:
             self._analyze_form_arguments()
         return self._arguments
 
     def coefficients(self):
-        "Return all Coefficient objects found in form."
+        "Return all ``Coefficient`` objects found in form."
         if self._coefficients is None:
             self._analyze_form_arguments()
         return self._coefficients
 
     def coefficient_numbering(self):
-        "Return a contiguous numbering of coefficients in a mapping { coefficient: number }."
+        """Return a contiguous numbering of coefficients in a mapping
+        ``{coefficient:number}``."""
         if self._coefficient_numbering is None:
             self._analyze_form_arguments()
         return self._coefficient_numbering
@@ -215,7 +253,7 @@ class Form(object):
         return self._hash
 
     def __eq__(self, other):
-        """Delayed evaluation of the __eq__ operator!
+        """Delayed evaluation of the == operator!
 
         Just 'lhs_form == rhs_form' gives an Equation,
         while 'bool(lhs_form == rhs_form)' delegates
@@ -223,8 +261,12 @@ class Form(object):
         """
         return Equation(self, other)
 
+    def __ne__(self, other):
+        "Immediate evaluation of the != operator (as opposed to the == operator)."
+        return not self.equals(other)
+
     def equals(self, other):
-        "Evaluate 'bool(lhs_form == rhs_form)'."
+        "Evaluate ``bool(lhs_form == rhs_form)``."
         if type(other) != Form:
             return False
         if len(self._integrals) != len(other._integrals):
@@ -258,6 +300,10 @@ class Form(object):
         "Subtract other form from this one."
         return self + (-other)
 
+    def __rsub__(self, other):
+        "Subtract this form from other."
+        return other + (-self)
+
     def __neg__(self):
         """Negate all integrals in form.
 
@@ -274,49 +320,109 @@ class Form(object):
 
     def __mul__(self, coefficient):
         "UFL form operator: Take the action of this form on the given coefficient."
-        if isinstance(coefficient, Expr): #Coefficient): # TODO: Check whatever makes sense
+        if isinstance(coefficient, Expr):
             from ufl.formoperators import action
             return action(self, coefficient)
         return NotImplemented
 
+    def __call__(self, *args, **kwargs):
+        """UFL form operator: Evaluate form by replacing arguments and coefficients.
+
+        Replaces form.arguments() with given positional arguments
+        in same number and ordering. Number of positional arguments
+        must be 0 or equal to the number of Arguments in the form.
+
+        The optional keyword argument coefficients can be set to a dict
+        to replace Coefficients with expressions of matching shapes.
+
+        Example:
+
+          V = FiniteElement("CG", triangle, 1)
+          v = TestFunction(V)
+          u = TrialFunction(V)
+          f = Coefficient(V)
+          g = Coefficient(V)
+          a = g*inner(grad(u), grad(v))*dx
+          M = a(f, f, coefficients={ g: 1 })
+
+        Is equivalent to M == grad(f)**2*dx.
+        """
+        repdict = {}
+
+        if args:
+            arguments = self.arguments()
+            if len(arguments) != len(args):
+                error("Need %d arguments to form(), got %d." % (len(arguments), len(args)))
+            repdict.update(zip(arguments, args))
+
+        coefficients = kwargs.pop("coefficients")
+        if kwargs:
+            error("Unknown kwargs %s." % str(list(kwargs)))
+
+        if coefficients is not None:
+            coeffs = self.coefficients()
+            for f in coefficients:
+                if f in coeffs:
+                    repdict[f] = coefficients[f]
+                else:
+                    warning("Coefficient %s is not in form." % ufl_err_str(f))
+        if repdict:
+            from ufl.formoperators import replace
+            return replace(self, repdict)
+        else:
+            return self
+
+    # "a @ f" notation in python 3.5
+    __matmul__ = __mul__
+
     # --- String conversion functions, for UI purposes only ---
 
+    def __unicode__(self):
+        # Only in python 2
+        return str(self).decode("utf-8")
+
     def __str__(self):
         "Compute shorter string representation of form. This can be huge for complicated forms."
-        # TODO: Add warning here to check if anyone actually calls it in libraries
+        # Warning used for making sure we don't use this in the general pipeline:
+        #warning("Calling str on form is potentially expensive and should be avoided except during debugging.")
+        # Not caching this because it can be huge
         s = "\n  +  ".join(str(itg) for itg in self.integrals())
         return s or "<empty Form>"
 
     def __repr__(self):
         "Compute repr string of form. This can be huge for complicated forms."
-        # TODO: Add warning here to check if anyone actually calls it in libraries
+        # Warning used for making sure we don't use this in the general pipeline:
+        #warning("Calling repr on form is potentially expensive and should be avoided except during debugging.")
         # Not caching this because it can be huge
-        r = "Form([%s])" % ", ".join(repr(itg) for itg in self.integrals())
-        return r
+        itgs = ", ".join(repr(itg) for itg in self.integrals())
+        r = "Form([" + itgs + "])"
+        return as_native_str(r)
 
-    def x_repr_latex_(self): # TODO: This works, but enable when form latex rendering is fixed
+    def x_repr_latex_(self):  # TODO: This works, but enable when form latex rendering is fixed
         from ufl.algorithms import ufl2latex
         return "$$%s$$" % ufl2latex(self)
 
-    def x_repr_png_(self): # TODO: This works, but enable when form latex rendering is fixed
+    def x_repr_png_(self):  # TODO: This works, but enable when form latex rendering is fixed
         from IPython.lib.latextools import latex_to_png
         return latex_to_png(self._repr_latex_())
 
-    # --- Analysis functions, precomputation and caching of various quantities ---
+    # --- Analysis functions, precomputation and caching of various quantities
 
     def _analyze_domains(self):
-        # TODO: join_domains function needs work, later when dolfin integration of a Domain or ufl.Mesh class is finished.
-        from ufl.geometry import join_domains
+        from ufl.domain import join_domains, sort_domains
 
-        # Collect integration domains and make canonical list of them
-        integration_domains = join_domains([itg.domain() for itg in self._integrals])
-        self._integration_domains = tuple(sorted(integration_domains, key=lambda x: x.label()))
+        # Collect unique integration domains
+        integration_domains = join_domains([itg.ufl_domain() for itg in self._integrals])
 
-        # TODO: Not including domains from coefficients and arguments here, may need that later
+        # Make canonically ordered list of the domains
+        self._integration_domains = sort_domains(integration_domains)
+
+        # TODO: Not including domains from coefficients and arguments
+        # here, may need that later
         self._domain_numbering = dict((d, i) for i, d in enumerate(self._integration_domains))
 
     def _analyze_subdomain_data(self):
-        integration_domains = self.domains()
+        integration_domains = self.ufl_domains()
         integrals = self.integrals()
 
         # Make clear data structures to collect subdomain data in
@@ -326,7 +432,7 @@ class Form(object):
 
         for integral in integrals:
             # Get integral properties
-            domain = integral.domain()
+            domain = integral.ufl_domain()
             it = integral.integral_type()
             sd = integral.subdomain_data()
 
@@ -335,7 +441,8 @@ class Form(object):
             if data is None:
                 subdomain_data[domain][it] = sd
             elif sd is not None:
-                ufl_assert(data.ufl_id() == sd.ufl_id(), "Integrals in form have different subdomain_data objects.")
+                if data.ufl_id() != sd.ufl_id():
+                    error("Integrals in form have different subdomain_data objects.")
         self._subdomain_data = subdomain_data
 
     def _analyze_form_arguments(self):
@@ -343,23 +450,13 @@ class Form(object):
         from ufl.algorithms.analysis import extract_arguments_and_coefficients
         arguments, coefficients = extract_arguments_and_coefficients(self)
 
-        # Include coordinate coefficients from integration domains
-        domains = self.domains()
-        coordinates = [c for c in (domain.coordinates() for domain in domains) if c is not None]
-        coefficients.extend(coordinates)
-
-        # TODO: Not including domains from coefficients and arguments here. Will we need that later?
-        #       I believe argument domains must be among integration domains in each integral, anything else is not well defined.
-        #       Furthermore if a coefficient domain differ from the integration domain, it will
-        #       currently be interpolated to the same element on the integration domain in dolfin.
-        #       Therefore their domain should not be included here.
-        #       In the future we may generate code for quadrature point evaluation of these instead,
-        #       and then the coefficient domains are still of no value in the code generation process.
-
         # Define canonical numbering of arguments and coefficients
-        self._arguments = tuple(sorted(set(arguments), key=lambda x: x.number()))
-        self._coefficients = tuple(sorted(set(coefficients), key=lambda x: x.count()))
-        self._coefficient_numbering = dict((c, i) for i, c in enumerate(self._coefficients))
+        self._arguments = tuple(sorted(set(arguments),
+                                       key=lambda x: x.number()))
+        self._coefficients = tuple(sorted(set(coefficients),
+                                          key=lambda x: x.count()))
+        self._coefficient_numbering = dict((c, i) for i,
+                                           c in enumerate(self._coefficients))
 
     def _compute_renumbering(self):
         # Include integration domains and coefficients in renumbering
@@ -369,10 +466,11 @@ class Form(object):
         renumbering.update(dn)
         renumbering.update(cn)
 
-        # Add domains of coefficients, these may include domains not among integration domains
+        # Add domains of coefficients, these may include domains not
+        # among integration domains
         k = len(dn)
         for c in cn:
-            d = c.domain()
+            d = c.ufl_domain()
             if d is not None and d not in renumbering:
                 renumbering[d] = k
                 k += 1
@@ -381,36 +479,38 @@ class Form(object):
 
     def _compute_signature(self):
         from ufl.algorithms.signature import compute_form_signature
-        self._signature = compute_form_signature(self, self._compute_renumbering())
+        self._signature = compute_form_signature(self,
+                                                 self._compute_renumbering())
+
 
 def as_form(form):
     "Convert to form if not a form, otherwise return form."
     if not isinstance(form, Form):
-        error("Unable to convert object to a UFL form: %s" % repr(form))
+        error("Unable to convert object to a UFL form: %s" % ufl_err_str(form))
     return form
 
 
-
-def replace_integral_domains(form, common_domain): # TODO: Move elsewhere
+def replace_integral_domains(form, common_domain):  # TODO: Move elsewhere
     """Given a form and a domain, assign a common integration domain to all integrals.
 
-    Does not modify the input form (Form should always be immutable).
+    Does not modify the input form (``Form`` should always be immutable).
     This is to support ill formed forms with no domain specified,
-    some times occuring in pydolfin, e.g. assemble(1*dx, mesh=mesh).
+    sometimes occurring in pydolfin, e.g. assemble(1*dx, mesh=mesh).
     """
-    domains = form.domains()
+    domains = form.ufl_domains()
     if common_domain is not None:
         gdim = common_domain.geometric_dimension()
         tdim = common_domain.topological_dimension()
-        ufl_assert(all((gdim == domain.geometric_dimension() and
-                        tdim == domain.topological_dimension())
-                        for domain in domains),
-            "Common domain does not share dimensions with form domains.")
+        if not all((gdim == domain.geometric_dimension() and
+                    tdim == domain.topological_dimension())
+                   for domain in domains):
+            error("Common domain does not share dimensions with form domains.")
+
     reconstruct = False
     integrals = []
     for itg in form.integrals():
-        domain = itg.domain()
-        if domain is None or domain.label() != common_domain.label():
+        domain = itg.ufl_domain()
+        if domain != common_domain:
             itg = itg.reconstruct(domain=common_domain)
             reconstruct = True
         integrals.append(itg)
