@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 "Types for representing symbolic expressions for geometric quantities."
 
-# Copyright (C) 2008-2015 Martin Sandve Alnæs
+# Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -21,9 +21,11 @@
 # Modified by Anders Logg, 2009.
 # Modified by Kristian B. Oelgaard, 2009
 # Modified by Marie E. Rognes 2012
+# Modified by Massimiliano Leoni, 2016
 
-from ufl.log import warning, error, deprecate
-from ufl.assertions import ufl_assert
+from ufl.utils.py23 import as_native_str
+from ufl.utils.py23 import as_native_strings
+from ufl.log import error
 from ufl.core.ufl_type import ufl_type
 from ufl.core.terminal import Terminal
 from ufl.domain import as_domain
@@ -95,7 +97,8 @@ Xf = CFK * (X - X0f)
 
 @ufl_type(is_abstract=True)
 class GeometricQuantity(Terminal):
-    __slots__ = ("_domain",)
+    __slots__ = as_native_strings(("_domain",))
+
     def __init__(self, domain):
         Terminal.__init__(self)
         self._domain = as_domain(domain)
@@ -105,10 +108,12 @@ class GeometricQuantity(Terminal):
 
     def is_cellwise_constant(self):
         "Return whether this expression is spatially constant over each cell (or over each facet for facet quantities)."
-        # NB! Geometric quantities are piecewise constant by default. Override if needed.
+        # NB! Geometric quantities are piecewise constant by
+        # default. Override if needed.
         return True
 
-    # NB! Geometric quantities are scalar by default. Override if needed.
+    # NB! Geometric quantities are scalar by default. Override if
+    # needed.
     ufl_shape = ()
 
     def _ufl_signature_data_(self, renumbering):
@@ -119,7 +124,8 @@ class GeometricQuantity(Terminal):
         return self._ufl_class_.name
 
     def __repr__(self):
-        return "%s(%r)" % (self._ufl_class_.__name__, self._domain)
+        r = "%s(%s)" % (self._ufl_class_.__name__, repr(self._domain))
+        return as_native_str(r)
 
     def _ufl_compute_hash_(self):
         return hash((type(self).__name__,) + self._domain._ufl_hash_data_())
@@ -127,13 +133,15 @@ class GeometricQuantity(Terminal):
     def __eq__(self, other):
         return isinstance(other, self._ufl_class_) and other._domain == self._domain
 
+
 @ufl_type(is_abstract=True)
 class GeometricCellQuantity(GeometricQuantity):
-    __slots__ = []
+    __slots__ = ()
+
 
 @ufl_type(is_abstract=True)
 class GeometricFacetQuantity(GeometricQuantity):
-    __slots__ = []
+    __slots__ = ()
 
 
 # --- Coordinate represented in different coordinate systems
@@ -153,6 +161,8 @@ class SpatialCoordinate(GeometricCellQuantity):
 
     @property
     def ufl_shape(self):
+        """Return the number of coordinates defined (i.e. the geometric dimension
+        of the domain)."""
         g = self._domain.geometric_dimension()
         return (g,)
 
@@ -163,6 +173,7 @@ class SpatialCoordinate(GeometricCellQuantity):
         return t == 0
 
     def evaluate(self, x, mapping, component, index_values):
+        "Return the value of the coordinate."
         if component == ():
             if isinstance(x, (tuple, list)):
                 return float(x[0])
@@ -171,6 +182,7 @@ class SpatialCoordinate(GeometricCellQuantity):
         else:
             return float(x[component[0]])
 
+
 @ufl_type()
 class CellCoordinate(GeometricCellQuantity):
     """UFL geometry representation: The coordinate in a reference cell.
@@ -195,6 +207,7 @@ class CellCoordinate(GeometricCellQuantity):
         t = self._domain.topological_dimension()
         return t == 0
 
+
 @ufl_type()
 class FacetCoordinate(GeometricFacetQuantity):
     """UFL geometry representation: The coordinate in a reference cell of a facet.
@@ -211,7 +224,8 @@ class FacetCoordinate(GeometricFacetQuantity):
     def __init__(self, domain):
         GeometricFacetQuantity.__init__(self, domain)
         t = self._domain.topological_dimension()
-        ufl_assert(t > 1, "FacetCoordinate is only defined for topological dimensions > 1.")
+        if t < 2:
+            error("FacetCoordinate is only defined for topological dimensions >= 2.")
 
     @property
     def ufl_shape(self):
@@ -220,7 +234,8 @@ class FacetCoordinate(GeometricFacetQuantity):
 
     def is_cellwise_constant(self):
         "Return whether this expression is spatially constant over each cell."
-        # Only case this is true is if the domain is an interval cell (with a vertex facet).
+        # Only case this is true is if the domain is an interval cell
+        # (with a vertex facet).
         t = self._domain.topological_dimension()
         return t <= 1
 
@@ -241,6 +256,7 @@ class CellOrigin(GeometricCellQuantity):
     def is_cellwise_constant(self):
         return True
 
+
 @ufl_type()
 class FacetOrigin(GeometricFacetQuantity):
     """UFL geometry representation: The spatial coordinate corresponding to origin of a reference facet."""
@@ -252,6 +268,7 @@ class FacetOrigin(GeometricFacetQuantity):
         g = self._domain.geometric_dimension()
         return (g,)
 
+
 @ufl_type()
 class CellFacetOrigin(GeometricFacetQuantity):
     """UFL geometry representation: The reference cell coordinate corresponding to origin of a reference facet."""
@@ -270,13 +287,15 @@ class CellFacetOrigin(GeometricFacetQuantity):
 class Jacobian(GeometricCellQuantity):
     """UFL geometry representation: The Jacobian of the mapping from reference cell to spatial coordinates.
 
-    J_ij = dx_i/dX_j
+    .. math:: J_{ij} = \\frac{dx_i}{dX_j}
     """
     __slots__ = ()
     name = "J"
 
     @property
     def ufl_shape(self):
+        """Return the number of coordinates defined (i.e. the geometric dimension
+        of the domain)."""
         g = self._domain.geometric_dimension()
         t = self._domain.topological_dimension()
         return (g, t)
@@ -286,6 +305,7 @@ class Jacobian(GeometricCellQuantity):
         # Only true for a piecewise linear coordinate field in simplex cells
         return self._domain.is_piecewise_linear_simplex_domain()
 
+
 @ufl_type()
 class FacetJacobian(GeometricFacetQuantity):
     """UFL geometry representation: The Jacobian of the mapping from reference facet to spatial coordinates.
@@ -303,7 +323,8 @@ class FacetJacobian(GeometricFacetQuantity):
     def __init__(self, domain):
         GeometricFacetQuantity.__init__(self, domain)
         t = self._domain.topological_dimension()
-        ufl_assert(t > 1, "FacetJacobian is only defined for topological dimensions > 1.")
+        if t < 2:
+            error("FacetJacobian is only defined for topological dimensions >= 2.")
 
     @property
     def ufl_shape(self):
@@ -313,11 +334,13 @@ class FacetJacobian(GeometricFacetQuantity):
 
     def is_cellwise_constant(self):
         "Return whether this expression is spatially constant over each cell."
-        # Only true for a piecewise linear coordinate field in simplex cells
+        # Only true for a piecewise linear coordinate field in simplex
+        # cells
         return self._domain.is_piecewise_linear_simplex_domain()
 
+
 @ufl_type()
-class CellFacetJacobian(GeometricFacetQuantity): # dX/dXf
+class CellFacetJacobian(GeometricFacetQuantity):  # dX/dXf
     """UFL geometry representation: The Jacobian of the mapping from reference facet to reference cell coordinates.
 
     CFJ_ij = dX_i/dXf_j
@@ -328,7 +351,8 @@ class CellFacetJacobian(GeometricFacetQuantity): # dX/dXf
     def __init__(self, domain):
         GeometricFacetQuantity.__init__(self, domain)
         t = self._domain.topological_dimension()
-        ufl_assert(t > 1, "CellFacetJacobian is only defined for topological dimensions > 1.")
+        if t < 2:
+            error("CellFacetJacobian is only defined for topological dimensions >= 2.")
 
     @property
     def ufl_shape(self):
@@ -337,9 +361,11 @@ class CellFacetJacobian(GeometricFacetQuantity): # dX/dXf
 
     def is_cellwise_constant(self):
         "Return whether this expression is spatially constant over each cell."
-        # This is always a constant mapping between two reference coordinate systems.
+        # This is always a constant mapping between two reference
+        # coordinate systems.
         return True
 
+
 @ufl_type()
 class CellEdgeVectors(GeometricCellQuantity):
     """UFL geometry representation: The vectors between reference cell vertices for each edge in cell."""
@@ -349,7 +375,8 @@ class CellEdgeVectors(GeometricCellQuantity):
     def __init__(self, domain):
         GeometricCellQuantity.__init__(self, domain)
         t = self._domain.topological_dimension()
-        ufl_assert(t > 1, "CellEdgeVectors is only defined for topological dimensions >= 2.")
+        if t < 2:
+            error("CellEdgeVectors is only defined for topological dimensions >= 2.")
 
     @property
     def ufl_shape(self):
@@ -363,6 +390,7 @@ class CellEdgeVectors(GeometricCellQuantity):
         # This is always constant for a given cell type
         return True
 
+
 @ufl_type()
 class FacetEdgeVectors(GeometricFacetQuantity):
     """UFL geometry representation: The vectors between reference cell vertices for each edge in current facet."""
@@ -372,7 +400,8 @@ class FacetEdgeVectors(GeometricFacetQuantity):
     def __init__(self, domain):
         GeometricFacetQuantity.__init__(self, domain)
         t = self._domain.topological_dimension()
-        ufl_assert(t > 2, "FacetEdgeVectors is only defined for topological dimensions >= 3.")
+        if t < 3:
+            error("FacetEdgeVectors is only defined for topological dimensions >= 3.")
 
     @property
     def ufl_shape(self):
@@ -400,9 +429,11 @@ class JacobianDeterminant(GeometricCellQuantity):
 
     def is_cellwise_constant(self):
         "Return whether this expression is spatially constant over each cell."
-        # Only true for a piecewise linear coordinate field in simplex cells
+        # Only true for a piecewise linear coordinate field in simplex
+        # cells
         return self._domain.is_piecewise_linear_simplex_domain()
 
+
 @ufl_type()
 class FacetJacobianDeterminant(GeometricFacetQuantity):
     """UFL geometry representation: The pseudo-determinant of the FacetJacobian."""
@@ -414,6 +445,7 @@ class FacetJacobianDeterminant(GeometricFacetQuantity):
         # Only true for a piecewise linear coordinate field in simplex cells
         return self._domain.is_piecewise_linear_simplex_domain()
 
+
 @ufl_type()
 class CellFacetJacobianDeterminant(GeometricFacetQuantity):
     """UFL geometry representation: The pseudo-determinant of the CellFacetJacobian."""
@@ -422,7 +454,8 @@ class CellFacetJacobianDeterminant(GeometricFacetQuantity):
 
     def is_cellwise_constant(self):
         "Return whether this expression is spatially constant over each cell."
-        # Only true for a piecewise linear coordinate field in simplex cells
+        # Only true for a piecewise linear coordinate field in simplex
+        # cells
         return self._domain.is_piecewise_linear_simplex_domain()
 
 
@@ -439,15 +472,19 @@ class JacobianInverse(GeometricCellQuantity):
 
     @property
     def ufl_shape(self):
+        """Return the number of coordinates defined (i.e. the geometric dimension
+        of the domain)."""
         g = self._domain.geometric_dimension()
         t = self._domain.topological_dimension()
         return (t, g)
 
     def is_cellwise_constant(self):
         "Return whether this expression is spatially constant over each cell."
-        # Only true for a piecewise linear coordinate field in simplex cells
+        # Only true for a piecewise linear coordinate field in simplex
+        # cells
         return self._domain.is_piecewise_linear_simplex_domain()
 
+
 @ufl_type()
 class FacetJacobianInverse(GeometricFacetQuantity):
     """UFL geometry representation: The pseudo-inverse of the FacetJacobian."""
@@ -457,7 +494,8 @@ class FacetJacobianInverse(GeometricFacetQuantity):
     def __init__(self, domain):
         GeometricFacetQuantity.__init__(self, domain)
         t = self._domain.topological_dimension()
-        ufl_assert(t > 1, "FacetJacobianInverse is only defined for topological dimensions > 1.")
+        if t < 2:
+            error("FacetJacobianInverse is only defined for topological dimensions >= 2.")
 
     @property
     def ufl_shape(self):
@@ -467,9 +505,11 @@ class FacetJacobianInverse(GeometricFacetQuantity):
 
     def is_cellwise_constant(self):
         "Return whether this expression is spatially constant over each cell."
-        # Only true for a piecewise linear coordinate field in simplex cells
+        # Only true for a piecewise linear coordinate field in simplex
+        # cells
         return self._domain.is_piecewise_linear_simplex_domain()
 
+
 @ufl_type()
 class CellFacetJacobianInverse(GeometricFacetQuantity):
     """UFL geometry representation: The pseudo-inverse of the CellFacetJacobian."""
@@ -479,7 +519,8 @@ class CellFacetJacobianInverse(GeometricFacetQuantity):
     def __init__(self, domain):
         GeometricFacetQuantity.__init__(self, domain)
         t = self._domain.topological_dimension()
-        ufl_assert(t > 1, "CellFacetJacobianInverse is only defined for topological dimensions > 1.")
+        if t < 2:
+            error("CellFacetJacobianInverse is only defined for topological dimensions >= 2.")
 
     @property
     def ufl_shape(self):
@@ -502,16 +543,21 @@ class FacetNormal(GeometricFacetQuantity):
 
     @property
     def ufl_shape(self):
+        """Return the number of coordinates defined (i.e. the geometric dimension
+        of the domain)."""
         g = self._domain.geometric_dimension()
         return (g,)
 
     def is_cellwise_constant(self):
         "Return whether this expression is spatially constant over each cell."
-        # For product cells, this is only true for some but not all facets. Seems like too much work to fix right now.
-        # Only true for a piecewise linear coordinate field with simplex _facets_.
+        # For product cells, this is only true for some but not all
+        # facets. Seems like too much work to fix right now.  Only
+        # true for a piecewise linear coordinate field with simplex
+        # _facets_.
         is_piecewise_linear = self._domain.ufl_coordinate_element().degree() == 1
         return is_piecewise_linear and self._domain.ufl_cell().has_simplex_facets()
 
+
 @ufl_type()
 class CellNormal(GeometricCellQuantity):
     """UFL geometry representation: The upwards pointing normal vector of the current manifold cell."""
@@ -520,11 +566,14 @@ class CellNormal(GeometricCellQuantity):
 
     @property
     def ufl_shape(self):
+        """Return the number of coordinates defined (i.e. the geometric dimension
+        of the domain)."""
         g = self._domain.geometric_dimension()
-        #t = self._domain.topological_dimension()
-        #return (g-t,g) # TODO: Should it be CellNormals? For interval in 3D we have two!
+        # t = self._domain.topological_dimension()
+        # return (g-t,g) # TODO: Should it be CellNormals? For interval in 3D we have two!
         return (g,)
 
+
 @ufl_type()
 class ReferenceNormal(GeometricFacetQuantity):
     """UFL geometry representation: The outwards pointing normal vector of the current facet on the reference cell"""
@@ -537,16 +586,17 @@ class ReferenceNormal(GeometricFacetQuantity):
         return (t,)
 
 # TODO: Implement in apply_geometry_lowering and enable
-#@ufl_type()
-#class FacetTangents(GeometricFacetQuantity):
-#    """UFL geometry representation: The tangent vectors of the current facet."""
-#    __slots__ = ()
-#    name = "t"
+# @ufl_type()
+# class FacetTangents(GeometricFacetQuantity):
+#     """UFL geometry representation: The tangent vectors of the current facet."""
+#     __slots__ = ()
+#     name = "t"
 #
-#    def __init__(self, domain):
-#        GeometricFacetQuantity.__init__(self, domain)
-#        t = self._domain.topological_dimension()
-#        ufl_assert(t > 1, "FacetTangents is only defined for topological dimensions > 1.")
+#     def __init__(self, domain):
+#         GeometricFacetQuantity.__init__(self, domain)
+#         t = self._domain.topological_dimension()
+#         if t < 2:
+#             error("FacetTangents is only defined for topological dimensions >= 2.")
 #
 #    @property
 #    def ufl_shape(self):
@@ -562,8 +612,8 @@ class ReferenceNormal(GeometricFacetQuantity):
 #        return is_piecewise_linear and self._domain.ufl_cell().has_simplex_facets()
 
 # TODO: Implement in apply_geometry_lowering and enable
-#@ufl_type()
-#class CellTangents(GeometricCellQuantity):
+# @ufl_type()
+# class CellTangents(GeometricCellQuantity):
 #    """UFL geometry representation: The tangent vectors of the current manifold cell."""
 #    __slots__ = ()
 #    name = "cell_tangents"
@@ -578,8 +628,8 @@ class ReferenceNormal(GeometricFacetQuantity):
 # --- Types representing midpoint coordinates
 
 # TODO: Implement in the rest of fenics
-#@ufl_type()
-#class CellMidpoint(GeometricCellQuantity):
+# @ufl_type()
+# class CellMidpoint(GeometricCellQuantity):
 #    """UFL geometry representation: The midpoint coordinate of the current cell."""
 #    __slots__ = ()
 #    name = "cell_midpoint"
@@ -590,8 +640,8 @@ class ReferenceNormal(GeometricFacetQuantity):
 #        return (g,)
 
 # TODO: Implement in the rest of fenics
-#@ufl_type()
-#class FacetMidpoint(GeometricFacetQuantity):
+# @ufl_type()
+# class FacetMidpoint(GeometricFacetQuantity):
 #    """UFL geometry representation: The midpoint coordinate of the current facet."""
 #    __slots__ = ()
 #    name = "facet_midpoint"
@@ -606,60 +656,69 @@ class ReferenceNormal(GeometricFacetQuantity):
 
 # TODO: Clean up this set of types? Document!
 
+
 @ufl_type()
 class ReferenceCellVolume(GeometricCellQuantity):
     """UFL geometry representation: The volume of the reference cell."""
     __slots__ = ()
     name = "reference_cell_volume"
 
+
 @ufl_type()
 class ReferenceFacetVolume(GeometricFacetQuantity):
     """UFL geometry representation: The volume of the reference cell of the current facet."""
     __slots__ = ()
     name = "reference_facet_volume"
 
+
 @ufl_type()
 class CellVolume(GeometricCellQuantity):
     """UFL geometry representation: The volume of the cell."""
     __slots__ = ()
     name = "volume"
 
+
 @ufl_type()
 class Circumradius(GeometricCellQuantity):
     """UFL geometry representation: The circumradius of the cell."""
     __slots__ = ()
     name = "circumradius"
 
-#@ufl_type()
-#class CellSurfaceArea(GeometricCellQuantity):
+# @ufl_type()
+# class CellSurfaceArea(GeometricCellQuantity):
 #    """UFL geometry representation: The total surface area of the cell."""
 #    __slots__ = ()
 #    name = "surfacearea"
 
+
 @ufl_type()
-class FacetArea(GeometricFacetQuantity): # FIXME: Should this be allowed for interval domain?
+class FacetArea(GeometricFacetQuantity):  # FIXME: Should this be allowed for interval domain?
     """UFL geometry representation: The area of the facet."""
     __slots__ = ()
     name = "facetarea"
 
+
 @ufl_type()
 class MinCellEdgeLength(GeometricCellQuantity):
     """UFL geometry representation: The minimum edge length of the cell."""
     __slots__ = ()
     name = "mincelledgelength"
 
+
 @ufl_type()
 class MaxCellEdgeLength(GeometricCellQuantity):
     """UFL geometry representation: The maximum edge length of the cell."""
     __slots__ = ()
     name = "maxcelledgelength"
 
+
 @ufl_type()
 class MinFacetEdgeLength(GeometricFacetQuantity):
     """UFL geometry representation: The minimum edge length of the facet."""
     __slots__ = ()
     name = "minfacetedgelength"
 
+
 @ufl_type()
 class MaxFacetEdgeLength(GeometricFacetQuantity):
     """UFL geometry representation: The maximum edge length of the facet."""
@@ -683,13 +742,16 @@ class CellOrientation(GeometricCellQuantity):
     __slots__ = ()
     name = "cell_orientation"
 
+
 @ufl_type()
 class FacetOrientation(GeometricFacetQuantity):
     """UFL geometry representation: The orientation (+1/-1) of the current facet relative to the reference cell."""
     __slots__ = ()
     name = "facet_orientation"
 
-# This doesn't quite fit anywhere. Make a special set of symbolic terminal types instead?
+
+# This doesn't quite fit anywhere. Make a special set of symbolic
+# terminal types instead?
 @ufl_type()
 class QuadratureWeight(GeometricQuantity):
     """UFL geometry representation: The current quadrature weight.
