@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """This module provides basic mathematical functions."""
 
-# Copyright (C) 2008-2015 Martin Sandve Alnæs
+# Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -22,12 +22,11 @@
 # Modified by Kristian B. Oelgaard, 2011
 
 import math
+from ufl.utils.py23 import as_native_strings
 from ufl.log import warning, error
-from ufl.assertions import ufl_assert
 from ufl.core.operator import Operator
-from ufl.constantvalue import is_true_ufl_scalar, ScalarValue, Zero, FloatValue, IntValue, as_ufl
-from ufl.utils.dicts import EmptyDict
 from ufl.core.ufl_type import ufl_type
+from ufl.constantvalue import is_true_ufl_scalar, ScalarValue, Zero, FloatValue, IntValue, as_ufl
 
 """
 TODO: Include additional functions available in <cmath> (need derivatives as well):
@@ -50,17 +49,20 @@ Implementation in C++ std::tr1:: or boost::math::tr1::
 - BesselY: cyl_neumann(nu, x)
 """
 
-#--- Function representations ---
+
+# --- Function representations ---
 
 @ufl_type(is_abstract=True, is_scalar=True, num_ops=1)
 class MathFunction(Operator):
     "Base class for all unary scalar math functions."
     # Freeze member variables for objects in this class
-    __slots__ = ("_name",)
+    __slots__ = as_native_strings(("_name",))
+
     def __init__(self, name, argument):
         Operator.__init__(self, (argument,))
-        ufl_assert(is_true_ufl_scalar(argument), "Expecting scalar argument.")
-        self._name     = name
+        if not is_true_ufl_scalar(argument):
+            error("Expecting scalar argument.")
+        self._name = name
 
     def evaluate(self, x, mapping, component, index_values):
         a = self.ufl_operands[0].evaluate(x, mapping, component, index_values)
@@ -74,12 +76,11 @@ class MathFunction(Operator):
     def __str__(self):
         return "%s(%s)" % (self._name, self.ufl_operands[0])
 
-    def __repr__(self):
-        return "%s(%r)" % (self._name, self.ufl_operands[0])
 
 @ufl_type()
 class Sqrt(MathFunction):
     __slots__ = ()
+
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.sqrt(float(argument)))
@@ -88,9 +89,11 @@ class Sqrt(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "sqrt", argument)
 
+
 @ufl_type()
 class Exp(MathFunction):
     __slots__ = ()
+
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.exp(float(argument)))
@@ -99,9 +102,11 @@ class Exp(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "exp", argument)
 
+
 @ufl_type()
 class Ln(MathFunction):
     __slots__ = ()
+
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.log(float(argument)))
@@ -114,9 +119,11 @@ class Ln(MathFunction):
         a = self.ufl_operands[0].evaluate(x, mapping, component, index_values)
         return math.log(a)
 
+
 @ufl_type()
 class Cos(MathFunction):
     __slots__ = ()
+
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.cos(float(argument)))
@@ -125,9 +132,11 @@ class Cos(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "cos", argument)
 
+
 @ufl_type()
 class Sin(MathFunction):
     __slots__ = ()
+
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.sin(float(argument)))
@@ -136,9 +145,11 @@ class Sin(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "sin", argument)
 
+
 @ufl_type()
 class Tan(MathFunction):
     __slots__ = ()
+
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.tan(float(argument)))
@@ -147,9 +158,11 @@ class Tan(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "tan", argument)
 
+
 @ufl_type()
 class Cosh(MathFunction):
     __slots__ = ()
+
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.cosh(float(argument)))
@@ -158,9 +171,11 @@ class Cosh(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "cosh", argument)
 
+
 @ufl_type()
 class Sinh(MathFunction):
     __slots__ = ()
+
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.sinh(float(argument)))
@@ -169,9 +184,11 @@ class Sinh(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "sinh", argument)
 
+
 @ufl_type()
 class Tanh(MathFunction):
     __slots__ = ()
+
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.tanh(float(argument)))
@@ -180,9 +197,11 @@ class Tanh(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "tanh", argument)
 
+
 @ufl_type()
 class Acos(MathFunction):
     __slots__ = ()
+
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.acos(float(argument)))
@@ -191,9 +210,11 @@ class Acos(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "acos", argument)
 
+
 @ufl_type()
 class Asin(MathFunction):
     __slots__ = ()
+
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.asin(float(argument)))
@@ -202,9 +223,11 @@ class Asin(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "asin", argument)
 
+
 @ufl_type()
 class Atan(MathFunction):
     __slots__ = ()
+
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             return FloatValue(math.atan(float(argument)))
@@ -213,6 +236,7 @@ class Atan(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "atan", argument)
 
+
 @ufl_type(is_scalar=True, num_ops=2)
 class Atan2(Operator):
     __slots__ = ()
@@ -224,8 +248,10 @@ class Atan2(Operator):
 
     def __init__(self, arg1, arg2):
         Operator.__init__(self, (arg1, arg2))
-        ufl_assert(is_true_ufl_scalar(arg1), "Expecting scalar argument 1.")
-        ufl_assert(is_true_ufl_scalar(arg2), "Expecting scalar argument 2.")
+        if not is_true_ufl_scalar(arg1):
+            error("Expecting scalar argument 1.")
+        if not is_true_ufl_scalar(arg2):
+            error("Expecting scalar argument 2.")
 
     def evaluate(self, x, mapping, component, index_values):
         a = self.ufl_operands[0].evaluate(x, mapping, component, index_values)
@@ -240,9 +266,6 @@ class Atan2(Operator):
     def __str__(self):
         return "atan_2(%s,%s)" % (self.ufl_operands[0], self.ufl_operands[1])
 
-    def __repr__(self):
-        return "atan_2(%s,%s)" % (self.ufl_operands[0], self.ufl_operands[1])
-
 
 def _find_erf():
     import math
@@ -253,9 +276,11 @@ def _find_erf():
         return scipy.special.erf
     return None
 
+
 @ufl_type()
 class Erf(MathFunction):
     __slots__ = ()
+
     def __new__(cls, argument):
         if isinstance(argument, (ScalarValue, Zero)):
             erf = _find_erf()
@@ -273,14 +298,17 @@ class Erf(MathFunction):
             error("No python implementation of erf available on this system, cannot evaluate. Upgrade python or install scipy.")
         return erf(a)
 
+
 @ufl_type(is_abstract=True, is_scalar=True, num_ops=2)
 class BesselFunction(Operator):
     "Base class for all bessel functions"
-    # Freeze member variables for objects in this class
-    __slots__ = ("_name", "_classname")
+    __slots__ = as_native_strings(("_name", "_classname"))
+
     def __init__(self, name, classname, nu, argument):
-        ufl_assert(is_true_ufl_scalar(nu), "Expecting scalar nu.")
-        ufl_assert(is_true_ufl_scalar(argument), "Expecting scalar argument.")
+        if not is_true_ufl_scalar(nu):
+            error("Expecting scalar nu.")
+        if not is_true_ufl_scalar(argument):
+            error("Expecting scalar argument.")
 
         # Use integer representation if suitable
         fnu = float(nu)
@@ -293,7 +321,7 @@ class BesselFunction(Operator):
         Operator.__init__(self, (nu, argument))
 
         self._classname = classname
-        self._name     = name
+        self._name = name
 
     def evaluate(self, x, mapping, component, index_values):
         a = self.ufl_operands[1].evaluate(x, mapping, component, index_values)
@@ -306,37 +334,44 @@ class BesselFunction(Operator):
             nu = int(self.ufl_operands[0])
             functype = 'n' if name != 'i' else 'v'
         else:
-            nu = self.ufl_operands[0].evaluate(x, mapping, component, index_values)
+            nu = self.ufl_operands[0].evaluate(x, mapping, component,
+                                               index_values)
             functype = 'v'
         func = getattr(scipy.special, name + functype)
         return func(nu, a)
 
     def __str__(self):
-        return "%s(%s, %s)" % (self._name, self.ufl_operands[0], self.ufl_operands[1])
+        return "%s(%s, %s)" % (self._name, self.ufl_operands[0],
+                               self.ufl_operands[1])
 
-    def __repr__(self):
-        return "%s(%r, %r)" % (self._classname, self.ufl_operands[0], self.ufl_operands[1])
 
 @ufl_type()
 class BesselJ(BesselFunction):
     __slots__ = ()
+
     def __init__(self, nu, argument):
         BesselFunction.__init__(self, "cyl_bessel_j", "BesselJ", nu, argument)
 
+
 @ufl_type()
 class BesselY(BesselFunction):
     __slots__ = ()
+
     def __init__(self, nu, argument):
         BesselFunction.__init__(self, "cyl_bessel_y", "BesselY", nu, argument)
 
+
 @ufl_type()
 class BesselI(BesselFunction):
     __slots__ = ()
+
     def __init__(self, nu, argument):
         BesselFunction.__init__(self, "cyl_bessel_i", "BesselI", nu, argument)
 
+
 @ufl_type()
 class BesselK(BesselFunction):
     __slots__ = ()
+
     def __init__(self, nu, argument):
         BesselFunction.__init__(self, "cyl_bessel_k", "BesselK", nu, argument)
