@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """The Measure class."""
 
-# Copyright (C) 2008-2015 Martin Sandve Alnæs
+# Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -18,10 +18,16 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
-# Modified by Anders Logg 2008-2015
+# Modified by Anders Logg 2008-2016
+# Modified by Massimiliano Leoni, 2016.
 
-from ufl.assertions import ufl_assert
-from ufl.log import error, warning, deprecate
+#import six
+from six import string_types
+import numbers
+
+from ufl.utils.py23 import as_native_strings
+from ufl.utils.py23 import as_native_str
+from ufl.log import error, deprecate
 from ufl.core.expr import Expr
 from ufl.checks import is_true_ufl_scalar
 from ufl.constantvalue import as_ufl
@@ -31,7 +37,7 @@ from ufl.protocols import id_or_none, metadata_equal, metadata_hashdata
 
 
 # Export list for ufl.classes
-__all_classes__ = ["Measure", "MeasureSum", "MeasureProduct"]
+__all_classes__ = as_native_strings(["Measure", "MeasureSum", "MeasureProduct"])
 
 
 # TODO: Design a class IntegralType(name, shortname, codim, num_cells, ...)?
@@ -41,68 +47,80 @@ __all_classes__ = ["Measure", "MeasureSum", "MeasureProduct"]
 _integral_types = [
     # === Integration over full topological dimension:
     ("cell", "dx"),                     # Over cells of a mesh
-    #("macro_cell", "dE"),              # Over a group of adjacent cells (TODO: Arbitrary cell group? Not currently used.)
+    # ("macro_cell", "dE"),              # Over a group of adjacent cells (TODO: Arbitrary cell group? Not currently used.)
 
     # === Integration over topological dimension - 1:
     ("exterior_facet", "ds"),           # Over one-sided exterior facets of a mesh
     ("interior_facet", "dS"),           # Over two-sided facets between pairs of adjacent cells of a mesh
 
     # === Integration over topological dimension 0
-    ("vertex", "dP"),                   # Over vertices of a mesh
-    #("vertex", "dV"),                  # TODO: Use this over vertices?
-    #("point", "dP"),                   # TODO: Use this over arbitrary points inside cells?
+    ("vertex", "dP"),                    # Over vertices of a mesh
+    # ("vertex", "dV"),                  # TODO: Use this over vertices?
+    # ("point", "dP"),                   # TODO: Use this over arbitrary points inside cells?
 
     # === Integration over custom domains
-    ("custom",    "dc"),                # Over custom user-defined domains (run-time quadrature points)
-    ("cutcell",   "dC"),                # Over a cell with some part cut away (run-time quadrature points)
-    ("interface", "dI"),                # Over a facet fragment overlapping with two or more cells (run-time quadrature points)
-    ("overlap",   "dO"),                # Over a cell fragment overlapping with two or more cells (run-time quadrature points)
+    ("custom", "dc"),                 # Over custom user-defined domains (run-time quadrature points)
+    ("cutcell", "dC"),                # Over a cell with some part cut away (run-time quadrature points)
+    ("interface", "dI"),              # Over a facet fragment overlapping with two or more cells (run-time quadrature points)
+    ("overlap", "dO"),                # Over a cell fragment overlapping with two or more cells (run-time quadrature points)
 
     # === Firedrake specific hacks on the way out:
-    # TODO: Remove these, firedrake can use metadata instead and create the measure objects in firedrake:
+    # TODO: Remove these, firedrake can use metadata instead
+    # and create the measure objects in firedrake:
     ("exterior_facet_bottom", "ds_b"),  # Over bottom facets on extruded mesh
-    ("exterior_facet_top",    "ds_t"),  # Over top facets on extruded mesh
-    ("exterior_facet_vert",   "ds_v"),  # Over side facets of an extruded mesh
-    ("interior_facet_horiz",  "dS_h"),  # Over horizontal facets of an extruded mesh
-    ("interior_facet_vert",   "dS_v"),  # Over vertical facets of an extruded mesh
-    ]
+    ("exterior_facet_top", "ds_t"),     # Over top facets on extruded mesh
+    ("exterior_facet_vert", "ds_v"),    # Over side facets of an extruded mesh
+    ("interior_facet_horiz", "dS_h"),   # Over horizontal facets of an extruded mesh
+    ("interior_facet_vert", "dS_v"),  # Over vertical facets of an extruded mesh
+]
+
 integral_type_to_measure_name = dict((l, s) for l, s in _integral_types)
 measure_name_to_integral_type = dict((s, l) for l, s in _integral_types)
+
 custom_integral_types = ("custom", "cutcell", "interface", "overlap")
+point_integral_types = ("vertex",)  # "point")
+facet_integral_types = ("exterior_facet", "interior_facet")
+
 
 def register_integral_type(integral_type, measure_name):
     global integral_type_to_measure_name, measure_name_to_integral_type
-    ufl_assert(measure_name == integral_type_to_measure_name.get(integral_type, measure_name),
-               "Integral type already added with different measure name!")
-    ufl_assert(integral_type == measure_name_to_integral_type.get(measure_name, integral_type),
-               "Measure name already used for another domain type!")
+    if measure_name != integral_type_to_measure_name.get(integral_type, measure_name):
+        error("Integral type already added with different measure name!")
+    if integral_type != measure_name_to_integral_type.get(measure_name, integral_type):
+        error("Measure name already used for another domain type!")
     integral_type_to_measure_name[integral_type] = measure_name
     measure_name_to_integral_type[measure_name] = integral_type
 
+
 def as_integral_type(integral_type):
     "Map short name to long name and require a valid one."
     integral_type = integral_type.replace(" ", "_")
-    integral_type = measure_name_to_integral_type.get(integral_type, integral_type)
-    ufl_assert(integral_type in integral_type_to_measure_name,
-               "Invalid integral_type.")
+    integral_type = measure_name_to_integral_type.get(integral_type,
+                                                      integral_type)
+    if integral_type not in integral_type_to_measure_name:
+        error("Invalid integral_type.")
     return integral_type
 
+
 def integral_types():
     "Return a tuple of all domain type strings."
     return tuple(sorted(integral_type_to_measure_name.keys()))
 
+
 def measure_names():
     "Return a tuple of all measure name strings."
     return tuple(sorted(measure_name_to_integral_type.keys()))
 
+
+# @six.python_2_unicode_compatible
 class Measure(object):
-    __slots__ = (
+    __slots__ = as_native_strings((
         "_integral_type",
         "_domain",
         "_subdomain_id",
         "_metadata",
         "_subdomain_data",
-        )
+        ))
     """Representation of an integration measure.
 
     The Measure object holds information about integration properties to be
@@ -110,12 +128,11 @@ class Measure(object):
     """
 
     def __init__(self,
-                 integral_type, # "dx" etc
+                 integral_type,  # "dx" etc
                  domain=None,
                  subdomain_id="everywhere",
                  metadata=None,
-                 subdomain_data=None
-                 ):
+                 subdomain_data=None):
         """
         integral_type:
             str, one of "cell", etc.,
@@ -142,25 +159,32 @@ class Measure(object):
 
         # Check that we either have a proper AbstractDomain or none
         self._domain = None if domain is None else as_domain(domain)
-        ufl_assert(self._domain is None or isinstance(self._domain, AbstractDomain), "Invalid domain.")
+        if not (self._domain is None
+                or isinstance(self._domain, AbstractDomain)):
+            error("Invalid domain.")
 
         # Store subdomain data
         self._subdomain_data = subdomain_data
-        # FIXME: Cannot require this (yet) because we currently have no way to implement ufl_id for dolfin SubDomain
-        #ufl_assert(self._subdomain_data is None or hasattr(self._subdomain_data, "ufl_id"),
-        #           "Invalid domain data, missing ufl_id() implementation.")
+        # FIXME: Cannot require this (yet) because we currently have
+        # no way to implement ufl_id for dolfin SubDomain
+        # if not (self._subdomain_data is None or hasattr(self._subdomain_data, "ufl_id")):
+        #     error("Invalid domain data, missing ufl_id() implementation.")
 
-        # Accept "everywhere", single subdomain, or multiple subdomains
-        ufl_assert(subdomain_id in ("everywhere",)
-                   or isinstance(subdomain_id, int)
-                   or (isinstance(subdomain_id, tuple)
-                       and all(isinstance(did, int) for did in subdomain_id)),
-                   "Invalid subdomain_id.")
+        # Accept "everywhere", single subdomain, or multiple
+        # subdomains
+        if isinstance(subdomain_id, tuple):
+            for did in subdomain_id:
+                if not isinstance(did, numbers.Integral):
+                    error("Invalid subdomain_id %s." % (did,))
+        else:
+            if not (subdomain_id in ("everywhere",)
+                    or isinstance(subdomain_id, numbers.Integral)):
+                error("Invalid subdomain_id %s." % (subdomain_id,))
         self._subdomain_id = subdomain_id
 
         # Validate compiler options are None or dict
-        ufl_assert(metadata is None or isinstance(metadata, dict),
-                   "Invalid metadata.")
+        if metadata is not None and not isinstance(metadata, dict):
+            error("Invalid metadata.")
         self._metadata = metadata or EmptyDict
 
     def integral_type(self):
@@ -170,9 +194,10 @@ class Measure(object):
         """
         return self._integral_type
 
-    def domain(self):
-        deprecate("Measure.domain() is deprecated, please use .ufl_domain() instead.")
-        return self.ufl_domain()
+    #def domain(self):
+    #    "Deprecated, please use .ufl_domain() instead."
+    #    deprecate("Measure.domain() is deprecated, please use .ufl_domain() instead.")
+    #    return self.ufl_domain()
 
     def ufl_domain(self):
         """Return the domain associated with this measure.
@@ -221,39 +246,58 @@ class Measure(object):
                        metadata=metadata, subdomain_data=subdomain_data)
 
     def subdomain_data(self):
-        """Return the integral subdomain_data. This data is not interpreted by UFL.
-        Its intension is to give a context in which the domain id is interpreted."""
+        """Return the integral subdomain_data. This data is not interpreted by
+        UFL.  Its intension is to give a context in which the domain
+        id is interpreted.
+
+        """
         return self._subdomain_data
 
-    # Note: Must keep the order of the first two arguments here (subdomain_id, metadata) for
-    # backwards compatibility, because some tutorials write e.g. dx(0, {...}) to set metadata.
-    def __call__(self, subdomain_id=None, metadata=None, domain=None, subdomain_data=None, degree=None, rule=None):
+    # Note: Must keep the order of the first two arguments here
+    # (subdomain_id, metadata) for backwards compatibility, because
+    # some tutorials write e.g. dx(0, {...}) to set metadata.
+    def __call__(self, subdomain_id=None, metadata=None, domain=None,
+                 subdomain_data=None, degree=None, scheme=None, rule=None):
         """Reconfigure measure with new domain specification or metadata."""
 
+        # Deprecation of 'rule' in favour of 'scheme'
+        if rule is not None:
+            deprecate("Measure argument 'rule' has been renamed to 'scheme'.")
+            assert scheme is None or scheme == rule
+            scheme = rule
+
         # Let syntax dx() mean integral over everywhere
-        all_args = (subdomain_id, metadata, domain, subdomain_data, degree, rule)
+        all_args = (subdomain_id, metadata, domain, subdomain_data,
+                    degree, scheme)
         if all(arg is None for arg in all_args):
             return self.reconstruct(subdomain_id="everywhere")
 
-        # Let syntax dx(domain) or dx(domain, metadata) mean integral over entire domain.
-        # To do this we need to hijack the first argument:
-        if subdomain_id is not None and (isinstance(subdomain_id, AbstractDomain) or hasattr(subdomain_id, 'ufl_domain')):
-            ufl_assert(domain is None, "Ambiguous: setting domain both as keyword argument and first argument.")
+        # Let syntax dx(domain) or dx(domain, metadata) mean integral
+        # over entire domain.  To do this we need to hijack the first
+        # argument:
+        if subdomain_id is not None and (isinstance(subdomain_id,
+                                                    AbstractDomain) or
+                                         hasattr(subdomain_id, 'ufl_domain')):
+            if domain is not None:
+                error("Ambiguous: setting domain both as keyword argument and first argument.")
             subdomain_id, domain = "everywhere", as_domain(subdomain_id)
 
-        # If degree or rule is set, inject into metadata. This is a quick fix to enable
-        # the dx(..., degree=3) notation. TODO: Make degree and rule properties of integrals.
-        if (degree, rule) != (None, None):
+        # If degree or scheme is set, inject into metadata. This is a
+        # quick fix to enable the dx(..., degree=3) notation.
+        # TODO: Make degree and scheme properties of integrals instead of adding to metadata.
+        if (degree, scheme) != (None, None):
             metadata = {} if metadata is None else metadata.copy()
             if degree is not None:
                 metadata["quadrature_degree"] = degree
-            if rule is not None:
-                metadata["quadrature_rule"] = rule
+            if scheme is not None:
+                metadata["quadrature_rule"] = scheme
 
         # If we get any keywords, use them to reconstruct Measure.
-        # Note that if only one argument is given, it is the subdomain_id,
-        # e.g. dx(3) == dx(subdomain_id=3)
-        return self.reconstruct(subdomain_id=subdomain_id, domain=domain, metadata=metadata, subdomain_data=subdomain_data)
+        # Note that if only one argument is given, it is the
+        # subdomain_id, e.g. dx(3) == dx(subdomain_id=3)
+        return self.reconstruct(subdomain_id=subdomain_id, domain=domain,
+                                metadata=metadata,
+                                subdomain_data=subdomain_data)
 
     def __getitem__(self, data):
         """This operator supports legacy syntax in python dolfin programs.
@@ -267,6 +311,10 @@ class Measure(object):
         deprecate("Notation dx[meshfunction] is deprecated. Please use dx(subdomain_data=meshfunction) instead.")
         return self(subdomain_data=data)
 
+    def __unicode__(self):
+        # Only in python 2
+        return str(self).decode("utf-8")
+
     def __str__(self):
         global integral_type_to_measure_name
         name = integral_type_to_measure_name[self._integral_type]
@@ -276,7 +324,7 @@ class Measure(object):
             args.append("subdomain_id=%s" % (self._subdomain_id,))
         if self._domain is not None:
             args.append("domain=%s" % (self._domain,))
-        if self._metadata: # Stored as EmptyDict if None
+        if self._metadata:  # Stored as EmptyDict if None
             args.append("metadata=%s" % (self._metadata,))
         if self._subdomain_data is not None:
             args.append("subdomain_data=%s" % (self._subdomain_data,))
@@ -286,21 +334,21 @@ class Measure(object):
     def __repr__(self):
         "Return a repr string for this Measure."
         global integral_type_to_measure_name
-        d = integral_type_to_measure_name[self._integral_type]
 
         args = []
         args.append(repr(self._integral_type))
 
         if self._subdomain_id is not None:
-            args.append("subdomain_id=%r" % (self._subdomain_id,))
+            args.append("subdomain_id=%s" % repr(self._subdomain_id))
         if self._domain is not None:
-            args.append("domain=%r" % (self._domain,))
-        if self._metadata: # Stored as EmptyDict if None
-            args.append("metadata=%r" % (self._metadata,))
+            args.append("domain=%s" % repr(self._domain))
+        if self._metadata:  # Stored as EmptyDict if None
+            args.append("metadata=%s" % repr(self._metadata))
         if self._subdomain_data is not None:
-            args.append("subdomain_data=%r" % (self._subdomain_data,))
+            args.append("subdomain_data=%s" % repr(self._subdomain_data))
 
-        return "%s(%s)" % (type(self).__name__, ', '.join(args))
+        r = "%s(%s)" % (type(self).__name__, ', '.join(args))
+        return as_native_str(r)
 
     def __hash__(self):
         "Return a hash value for this Measure."
@@ -313,12 +361,12 @@ class Measure(object):
 
     def __eq__(self, other):
         "Checks if two Measures are equal."
-        return (isinstance(other, Measure)
-                and self._integral_type == other._integral_type
-                and self._subdomain_id == other._subdomain_id
-                and self._domain == other._domain
-                and id_or_none(self._subdomain_data) == id_or_none(other._subdomain_data)
-                and metadata_equal(self._metadata, other._metadata))
+        return (isinstance(other, Measure) and
+                self._integral_type == other._integral_type and
+                self._subdomain_id == other._subdomain_id and
+                self._domain == other._domain and
+                id_or_none(self._subdomain_data) == id_or_none(other._subdomain_data) and
+                metadata_equal(self._metadata, other._metadata))
 
     def __add__(self, other):
         """Add two measures (self+other).
@@ -351,13 +399,15 @@ class Measure(object):
             return NotImplemented
 
     def __rmul__(self, integrand):
-        """Multiply a scalar expression with measure to construct a form with a single integral.
+        """Multiply a scalar expression with measure to construct a form with
+a single integral.
 
         This is to implement the notation
 
             form = integrand * self
 
         Integration properties are taken from this Measure object.
+
         """
         # Avoid circular imports
         from ufl.integral import Integral
@@ -367,28 +417,30 @@ class Measure(object):
         if isinstance(integrand, (int, float)):
             integrand = as_ufl(integrand)
 
-        # Let other types implement multiplication with Measure
-        # if they want to (to support the dolfin-adjoint TimeMeasure)
+        # Let other types implement multiplication with Measure if
+        # they want to (to support the dolfin-adjoint TimeMeasure)
         if not isinstance(integrand, Expr):
             return NotImplemented
 
         # Allow only scalar integrands
         if not is_true_ufl_scalar(integrand):
-            msg = ("Can only integrate scalar expressions. The integrand is a " +
-                   "tensor expression with value rank %d and free indices %r.")
-            error(msg % (len(integrand.ufl_shape), integrand.ufl_free_indices))
+            error("Can only integrate scalar expressions. The integrand is a "
+                  "tensor expression with value shape %s and free indices with labels %s." %
+                    (integrand.ufl_shape, integrand.ufl_free_indices))
 
-        # If we have a tuple of domain ids, delegate composition to Integral.__add__:
+        # If we have a tuple of domain ids, delegate composition to
+        # Integral.__add__:
         subdomain_id = self.subdomain_id()
         if isinstance(subdomain_id, tuple):
             return sum(integrand*self.reconstruct(subdomain_id=d) for d in subdomain_id)
 
         # Check that we have an integer subdomain or a string
         # ("everywhere" or "otherwise", any more?)
-        ufl_assert(isinstance(subdomain_id, (int, str)),
-                   "Expecting integer or string domain id.")
+        if not isinstance(subdomain_id, string_types + (numbers.Integral,)):
+            error("Expecting integer or string domain id.")
 
-        # If we don't have an integration domain, try to find one in integrand
+        # If we don't have an integration domain, try to find one in
+        # integrand
         domain = self.ufl_domain()
         if domain is None:
             domains = extract_domains(integrand)
@@ -408,6 +460,8 @@ class Measure(object):
                             subdomain_data=self.subdomain_data())
         return Form([integral])
 
+
+# @six.python_2_unicode_compatible
 class MeasureSum(object):
     """Represents a sum of measures.
 
@@ -419,7 +473,8 @@ class MeasureSum(object):
 
         f*ds(1) + f*ds(3)
     """
-    __slots__ = ("_measures",)
+    __slots__ = as_native_strings(("_measures",))
+
     def __init__(self, *measures):
         self._measures = measures
 
@@ -434,9 +489,14 @@ class MeasureSum(object):
             return MeasureSum(*(self._measures + other._measures))
         return NotImplemented
 
+    def __unicode__(self):
+        # Only in python 2
+        return str(self).decode("utf-8")
+
     def __str__(self):
         return "{\n    " + "\n  + ".join(map(str, self._measures)) + "\n}"
 
+
 class MeasureProduct(object):
     """Represents a product of measures.
 
@@ -447,11 +507,13 @@ class MeasureProduct(object):
     This is work in progress and not functional. It needs support
     in other parts of ufl and the rest of the code generation chain.
     """
-    __slots__ = ("_measures",)
+    __slots__ = as_native_strings(("_measures",))
+
     def __init__(self, *measures):
         "Create MeasureProduct from given list of measures."
         self._measures = measures
-        ufl_assert(len(self._measures) > 1, "Expecting at least two measures.")
+        if len(self._measures) < 2:
+            error("Expecting at least two measures.")
 
     def __mul__(self, other):
         """Flatten multiplication of product measures.
