@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """Compound tensor algebra operations."""
 
-# Copyright (C) 2008-2015 Martin Sandve Alnæs
+# Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -18,17 +18,17 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 
-from ufl.log import warning
-from ufl.assertions import ufl_assert
+from ufl.log import error
+from ufl.utils.py23 import as_native_strings
+from ufl.core.expr import ufl_err_str
+from ufl.core.ufl_type import ufl_type
 from ufl.constantvalue import Zero
 from ufl.algebra import Operator
 from ufl.precedence import parstr
 from ufl.sorting import sorted_expr
-from ufl.core.ufl_type import ufl_type
 from ufl.index_combination_utils import merge_nonoverlapping_indices
 
-
-### Algebraic operations on tensors:
+# Algebraic operations on tensors:
 # FloatValues:
 #   dot(a,b)      = a*b
 #   inner(a,b)    = a*b
@@ -53,11 +53,13 @@ from ufl.index_combination_utils import merge_nonoverlapping_indices
 #   dot(x,y):   last index of x has same dimension as first index of y
 #   inner(x,y): shape of x equals the shape of y
 
+
 # --- Classes representing compound tensor algebra operations ---
 
 @ufl_type(is_abstract=True)
 class CompoundTensorOperator(Operator):
     __slots__ = ()
+
     def __init__(self, operands):
         Operator.__init__(self, operands)
 
@@ -65,33 +67,34 @@ class CompoundTensorOperator(Operator):
 #       This would simplify some algorithms. The only
 #       problem is we can't use + in many algorithms because
 #       this type should be expanded by expand_compounds.
-#class TensorSum(CompoundTensorOperator):
-#    "Sum of nonscalar expressions."
-#    pass
+# class TensorSum(CompoundTensorOperator):
+#     "Sum of nonscalar expressions."
+#     pass
 
 # TODO: Use this similarly to TensorSum?
 #       This would simplify some algorithms. The only
 #       problem is we can't use / in many algorithms because
 #       this type should be expanded by expand_compounds.
-#class TensorDivision(CompoundTensorOperator):
-#    "Division of nonscalar expression with a scalar expression."
-#    pass
+# class TensorDivision(CompoundTensorOperator):
+#     "Division of nonscalar expression with a scalar expression."
+#     pass
 
 # TODO: Use this similarly to TensorSum?
 #       This would simplify some algorithms. The only
 #       problem is we can't use * in many algorithms because
 #       this type should be expanded by expand_compounds.
-#class MatrixProduct(CompoundTensorOperator):
-#    "Product of a matrix with a matrix or vector."
-#    pass
+# class MatrixProduct(CompoundTensorOperator):
+#     "Product of a matrix with a matrix or vector."
+#     pass
 
 # TODO: Use this similarly to TensorSum?
 #       This would simplify some algorithms. The only
 #       problem is we can't use abs in many algorithms because
 #       this type should be expanded by expand_compounds.
-#class TensorAbs(CompoundTensorOperator):
-#    "Absolute value of nonscalar expression."
-#    pass
+# class TensorAbs(CompoundTensorOperator):
+#     "Absolute value of nonscalar expression."
+#     pass
+
 
 @ufl_type(is_shaping=True, num_ops=1, inherit_indices_from_operand=0)
 class Transposed(CompoundTensorOperator):
@@ -105,7 +108,8 @@ class Transposed(CompoundTensorOperator):
 
     def __init__(self, A):
         CompoundTensorOperator.__init__(self, (A,))
-        ufl_assert(len(A.ufl_shape) == 2, "Transposed is only defined for rank 2 tensors.")
+        if len(A.ufl_shape) != 2:
+            error("Transposed is only defined for rank 2 tensors.")
 
     @property
     def ufl_shape(self):
@@ -115,12 +119,13 @@ class Transposed(CompoundTensorOperator):
     def __str__(self):
         return "%s^T" % parstr(self.ufl_operands[0], self)
 
-    def __repr__(self):
-        return "Transposed(%r)" % self.ufl_operands[0]
 
 @ufl_type(num_ops=2)
 class Outer(CompoundTensorOperator):
-    __slots__ = ("ufl_free_indices", "ufl_index_dimensions")
+    __slots__ = as_native_strings((
+        "ufl_free_indices",
+        "ufl_index_dimensions",
+        ))
 
     def __new__(cls, a, b):
         ash, bsh = a.ufl_shape, b.ufl_shape
@@ -142,19 +147,22 @@ class Outer(CompoundTensorOperator):
         return self.ufl_operands[0].ufl_shape + self.ufl_operands[1].ufl_shape
 
     def __str__(self):
-        return "%s (X) %s" % (parstr(self.ufl_operands[0], self), parstr(self.ufl_operands[1], self))
+        return "%s (X) %s" % (parstr(self.ufl_operands[0], self),
+                              parstr(self.ufl_operands[1], self))
 
-    def __repr__(self):
-        return "Outer(%r, %r)" % (self.ufl_operands[0], self.ufl_operands[1])
 
 @ufl_type(num_ops=2)
 class Inner(CompoundTensorOperator):
-    __slots__ = ("ufl_free_indices", "ufl_index_dimensions")
+    __slots__ = as_native_strings((
+        "ufl_free_indices",
+        "ufl_index_dimensions",
+        ))
 
     def __new__(cls, a, b):
         # Checks
         ash, bsh = a.ufl_shape, b.ufl_shape
-        ufl_assert(ash == bsh, "Shape mismatch.")
+        if ash != bsh:
+            error("Shapes do not match: %s and %s." % (ufl_err_str(a), ufl_err_str(b)))
 
         # Simplification
         if isinstance(a, Zero) or isinstance(b, Zero):
@@ -180,14 +188,16 @@ class Inner(CompoundTensorOperator):
     ufl_shape = ()
 
     def __str__(self):
-        return "%s : %s" % (parstr(self.ufl_operands[0], self), parstr(self.ufl_operands[1], self))
+        return "%s : %s" % (parstr(self.ufl_operands[0], self),
+                            parstr(self.ufl_operands[1], self))
 
-    def __repr__(self):
-        return "Inner(%r, %r)" % (self.ufl_operands[0], self.ufl_operands[1])
 
 @ufl_type(num_ops=2)
 class Dot(CompoundTensorOperator):
-    __slots__ = ("ufl_free_indices", "ufl_index_dimensions")
+    __slots__ = as_native_strings((
+        "ufl_free_indices",
+        "ufl_index_dimensions",
+        ))
 
     def __new__(cls, a, b):
         ash = a.ufl_shape
@@ -196,18 +206,18 @@ class Dot(CompoundTensorOperator):
         scalar = (ar == 0 and br == 0)
 
         # Checks
-        ufl_assert((ar >= 1 and br >= 1) or scalar,
-            "Dot product requires non-scalar arguments, "
-            "got arguments with ranks %d and %d." %
-            (ar, br))
-        ufl_assert(scalar or ash[-1] == bsh[0], "Dimension mismatch in dot product.")
+        if not ((ar >= 1 and br >= 1) or scalar):
+            error("Dot product requires non-scalar arguments, "
+                  "got arguments with ranks %d and %d." % (ar, br))
+        if not (scalar or ash[-1] == bsh[0]):
+            error("Dimension mismatch in dot product.")
 
         # Simplification
         if isinstance(a, Zero) or isinstance(b, Zero):
             shape = ash[:-1] + bsh[1:]
             fi, fid = merge_nonoverlapping_indices(a, b)
             return Zero(shape, fi, fid)
-        elif scalar: # TODO: Move this to def dot()?
+        elif scalar:  # TODO: Move this to def dot()?
             return a * b
 
         return CompoundTensorOperator.__new__(cls)
@@ -223,22 +233,25 @@ class Dot(CompoundTensorOperator):
         return self.ufl_operands[0].ufl_shape[:-1] + self.ufl_operands[1].ufl_shape[1:]
 
     def __str__(self):
-        return "%s . %s" % (parstr(self.ufl_operands[0], self), parstr(self.ufl_operands[1], self))
+        return "%s . %s" % (parstr(self.ufl_operands[0], self),
+                            parstr(self.ufl_operands[1], self))
 
-    def __repr__(self):
-        return "Dot(%r, %r)" % (self.ufl_operands[0], self.ufl_operands[1])
 
 @ufl_type(num_ops=2)
 class Cross(CompoundTensorOperator):
-    __slots__ = ("ufl_free_indices", "ufl_index_dimensions")
+    __slots__ = as_native_strings((
+        "ufl_free_indices",
+        "ufl_index_dimensions",
+        ))
 
     def __new__(cls, a, b):
         ash = a.ufl_shape
         bsh = b.ufl_shape
 
         # Checks
-        ufl_assert(len(ash) == 1 and ash == bsh,
-            "Cross product requires arguments of rank 1.")
+        if not (len(ash) == 1 and ash == bsh):
+            error("Cross product requires arguments of rank 1, got %s and %s." % (
+                ufl_err_str(a), ufl_err_str(b)))
 
         # Simplification
         if isinstance(a, Zero) or isinstance(b, Zero):
@@ -256,10 +269,9 @@ class Cross(CompoundTensorOperator):
     ufl_shape = (3,)
 
     def __str__(self):
-        return "%s x %s" % (parstr(self.ufl_operands[0], self), parstr(self.ufl_operands[1], self))
+        return "%s x %s" % (parstr(self.ufl_operands[0], self),
+                            parstr(self.ufl_operands[1], self))
 
-    def __repr__(self):
-        return "Cross(%r, %r)" % (self.ufl_operands[0], self.ufl_operands[1])
 
 @ufl_type(num_ops=1, inherit_indices_from_operand=0)
 class Trace(CompoundTensorOperator):
@@ -267,7 +279,8 @@ class Trace(CompoundTensorOperator):
 
     def __new__(cls, A):
         # Checks
-        ufl_assert(len(A.ufl_shape) == 2, "Trace of tensor with rank != 2 is undefined.")
+        if len(A.ufl_shape) != 2:
+            error("Trace of tensor with rank != 2 is undefined.")
 
         # Simplification
         if isinstance(A, Zero):
@@ -283,8 +296,6 @@ class Trace(CompoundTensorOperator):
     def __str__(self):
         return "tr(%s)" % self.ufl_operands[0]
 
-    def __repr__(self):
-        return "Trace(%r)" % self.ufl_operands[0]
 
 @ufl_type(is_scalar=True, num_ops=1)
 class Determinant(CompoundTensorOperator):
@@ -296,12 +307,12 @@ class Determinant(CompoundTensorOperator):
         Afi = A.ufl_free_indices
 
         # Checks
-        ufl_assert(r == 0 or r == 2,
-            "Determinant of tensor with rank != 2 is undefined.")
-        ufl_assert(r == 0 or sh[0] == sh[1],
-            "Cannot take determinant of rectangular rank 2 tensor.")
-        ufl_assert(not Afi,
-            "Not expecting free indices in determinant.")
+        if r not in (0, 2):
+            error("Determinant of tensor with rank != 2 is undefined.")
+        if r == 2 and sh[0] != sh[1]:
+            error("Cannot take determinant of rectangular rank 2 tensor.")
+        if Afi:
+            error("Not expecting free indices in determinant.")
 
         # Simplification
         if isinstance(A, Zero):
@@ -317,10 +328,9 @@ class Determinant(CompoundTensorOperator):
     def __str__(self):
         return "det(%s)" % self.ufl_operands[0]
 
-    def __repr__(self):
-        return "Determinant(%r)" % self.ufl_operands[0]
 
-# TODO: Drop Inverse and represent it as product of Determinant and Cofactor?
+# TODO: Drop Inverse and represent it as product of Determinant and
+# Cofactor?
 @ufl_type(is_index_free=True, num_ops=1)
 class Inverse(CompoundTensorOperator):
     __slots__ = ()
@@ -343,7 +353,7 @@ class Inverse(CompoundTensorOperator):
         if r != 2:
             error("Inverse of tensor with rank != 2 is undefined.")
         if sh[0] != sh[1]:
-            error("Cannot take inverse of rectangular matrix with dimensions %s." % repr(sh))
+            error("Cannot take inverse of rectangular matrix with dimensions %s." % (sh,))
 
         return CompoundTensorOperator.__new__(cls)
 
@@ -357,8 +367,6 @@ class Inverse(CompoundTensorOperator):
     def __str__(self):
         return "%s^-1" % parstr(self.ufl_operands[0], self)
 
-    def __repr__(self):
-        return "Inverse(%r)" % self.ufl_operands[0]
 
 @ufl_type(is_index_free=True, num_ops=1)
 class Cofactor(CompoundTensorOperator):
@@ -369,11 +377,14 @@ class Cofactor(CompoundTensorOperator):
 
         # Checks
         sh = A.ufl_shape
-        ufl_assert(len(sh) == 2, "Cofactor of tensor with rank != 2 is undefined.")
+        if len(sh) != 2:
+            error("Cofactor of tensor with rank != 2 is undefined.")
         if sh[0] != sh[1]:
-            error("Cannot take cofactor of rectangular matrix with dimensions %s." % repr(sh))
-        ufl_assert(not A.ufl_free_indices, "Not expecting free indices in Cofactor.")
-        ufl_assert(not isinstance(A, Zero), "Cannot take cofactor of zero matrix.")
+            error("Cannot take cofactor of rectangular matrix with dimensions %s." % (sh,))
+        if A.ufl_free_indices:
+            error("Not expecting free indices in Cofactor.")
+        if isinstance(A, Zero):
+            error("Cannot take cofactor of zero matrix.")
 
     @property
     def ufl_shape(self):
@@ -382,8 +393,6 @@ class Cofactor(CompoundTensorOperator):
     def __str__(self):
         return "cofactor(%s)" % self.ufl_operands[0]
 
-    def __repr__(self):
-        return "Cofactor(%r)" % self.ufl_operands[0]
 
 @ufl_type(num_ops=1, inherit_shape_from_operand=0, inherit_indices_from_operand=0)
 class Deviatoric(CompoundTensorOperator):
@@ -393,10 +402,12 @@ class Deviatoric(CompoundTensorOperator):
         sh = A.ufl_shape
 
         # Checks
-        ufl_assert(len(sh) == 2, "Deviatoric part of tensor with rank != 2 is undefined.")
+        if len(sh) != 2:
+            error("Deviatoric part of tensor with rank != 2 is undefined.")
         if sh[0] != sh[1]:
-            error("Cannot take deviatoric part of rectangular matrix with dimensions %s." % repr(sh))
-        ufl_assert(not A.ufl_free_indices, "Not expecting free indices in Deviatoric.")
+            error("Cannot take deviatoric part of rectangular matrix with dimensions %s." % (sh,))
+        if A.ufl_free_indices:
+            error("Not expecting free indices in Deviatoric.")
 
         # Simplification
         if isinstance(A, Zero):
@@ -410,8 +421,6 @@ class Deviatoric(CompoundTensorOperator):
     def __str__(self):
         return "dev(%s)" % self.ufl_operands[0]
 
-    def __repr__(self):
-        return "Deviatoric(%r)" % self.ufl_operands[0]
 
 @ufl_type(num_ops=1, inherit_shape_from_operand=0, inherit_indices_from_operand=0)
 class Skew(CompoundTensorOperator):
@@ -422,10 +431,12 @@ class Skew(CompoundTensorOperator):
         Afi = A.ufl_free_indices
 
         # Checks
-        ufl_assert(len(sh) == 2, "Skew symmetric part of tensor with rank != 2 is undefined.")
+        if len(sh) != 2:
+            error("Skew symmetric part of tensor with rank != 2 is undefined.")
         if sh[0] != sh[1]:
-            error("Cannot take skew part of rectangular matrix with dimensions %s." % repr(sh))
-        ufl_assert(not Afi, "Not expecting free indices in Skew.")
+            error("Cannot take skew part of rectangular matrix with dimensions %s." % (sh,))
+        if Afi:
+            error("Not expecting free indices in Skew.")
 
         # Simplification
         if isinstance(A, Zero):
@@ -439,8 +450,6 @@ class Skew(CompoundTensorOperator):
     def __str__(self):
         return "skew(%s)" % self.ufl_operands[0]
 
-    def __repr__(self):
-        return "Skew(%r)" % self.ufl_operands[0]
 
 @ufl_type(num_ops=1, inherit_shape_from_operand=0, inherit_indices_from_operand=0)
 class Sym(CompoundTensorOperator):
@@ -451,10 +460,12 @@ class Sym(CompoundTensorOperator):
         Afi = A.ufl_free_indices
 
         # Checks
-        ufl_assert(len(sh) == 2, "Symmetric part of tensor with rank != 2 is undefined.")
+        if len(sh) != 2:
+            error("Symmetric part of tensor with rank != 2 is undefined.")
         if sh[0] != sh[1]:
-            error("Cannot take symmetric part of rectangular matrix with dimensions %s." % repr(sh))
-        ufl_assert(not Afi, "Not expecting free indices in Sym.")
+            error("Cannot take symmetric part of rectangular matrix with dimensions %s." % (sh,))
+        if Afi:
+            error("Not expecting free indices in Sym.")
 
         # Simplification
         if isinstance(A, Zero):
@@ -467,6 +478,3 @@ class Sym(CompoundTensorOperator):
 
     def __str__(self):
         return "sym(%s)" % self.ufl_operands[0]
-
-    def __repr__(self):
-        return "Sym(%r)" % self.ufl_operands[0]
