@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """Classes used to group scalar expressions into expressions with rank > 0."""
 
-# Copyright (C) 2008-2015 Martin Sandve Alnæs
+# Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -17,13 +17,14 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
+#
+# Modified by Massimiliano Leoni, 2016.
 
 from six.moves import zip
 from six.moves import xrange as range
 
-from ufl.log import warning, error
-from ufl.utils.dicts import subdict, EmptyDict
-from ufl.assertions import ufl_assert
+from ufl.utils.py23 import as_native_strings
+from ufl.log import error
 from ufl.core.ufl_type import ufl_type
 from ufl.core.expr import Expr
 from ufl.core.operator import Operator
@@ -32,6 +33,7 @@ from ufl.core.multiindex import Index, FixedIndex, MultiIndex, indices
 from ufl.indexed import Indexed
 from ufl.index_combination_utils import remove_indices
 
+
 # --- Classes representing tensors of UFL expressions ---
 
 @ufl_type(is_shaping=True, num_ops="varying", inherit_indices_from_operand=0)
@@ -40,15 +42,16 @@ class ListTensor(Operator):
     __slots__ = ()
 
     def __new__(cls, *expressions):
-        # All lists and tuples should already be unwrapped in as_tensor
+        # All lists and tuples should already be unwrapped in
+        # as_tensor
         if any(not isinstance(e, Expr) for e in expressions):
             error("Expecting only UFL expressions in ListTensor constructor.")
 
         # Get properties of the first expression
         e0 = expressions[0]
-        sh    = e0.ufl_shape
-        fi    = e0.ufl_free_indices
-        fid   = e0.ufl_index_dimensions
+        sh = e0.ufl_shape
+        fi = e0.ufl_free_indices
+        fid = e0.ufl_index_dimensions
 
         # Obviously, each subexpression must have the same shape
         if any(sh != e.ufl_shape for e in expressions[1:]):
@@ -70,17 +73,17 @@ class ListTensor(Operator):
 
         # Checks
         indexset = set(self.ufl_operands[0].ufl_free_indices)
-        ufl_assert(all(not (indexset ^ set(e.ufl_free_indices)) for e in self.ufl_operands),
-            "Can't combine subtensor expressions with different sets of free indices.")
+        if not all(not (indexset ^ set(e.ufl_free_indices)) for e in self.ufl_operands):
+            error("Can't combine subtensor expressions with different sets of free indices.")
 
     @property
     def ufl_shape(self):
         return (len(self.ufl_operands),) + self.ufl_operands[0].ufl_shape
 
     def evaluate(self, x, mapping, component, index_values, derivatives=()):
-        ufl_assert(len(component) == len(self.ufl_shape),
-                   "Can only evaluate scalars, expecting a component "
-                   "tuple of length %d, not %s." % (len(self.ufl_shape), component))
+        if len(component) != len(self.ufl_shape):
+            error("Can only evaluate scalars, expecting a component "
+                  "tuple of length %d, not %s." % (len(self.ufl_shape), component))
         a = self.ufl_operands[component[0]]
         component = component[1:]
         if derivatives:
@@ -119,13 +122,11 @@ class ListTensor(Operator):
                 return "%s[%s]" % (ind, s)
         return substring(self.ufl_operands, 0)
 
-    def __repr__(self):
-        return "ListTensor(%s)" % ", ".join(repr(e) for e in self.ufl_operands)
 
 @ufl_type(is_shaping=True, num_ops="varying")
 class ComponentTensor(Operator):
     """UFL operator type: Maps the free indices of a scalar valued expression to tensor axes."""
-    __slots__ = ("ufl_shape", "ufl_free_indices", "ufl_index_dimensions")
+    __slots__ = as_native_strings(("ufl_shape", "ufl_free_indices", "ufl_index_dimensions"))
 
     def __new__(cls, expression, indices):
 
@@ -140,17 +141,20 @@ class ComponentTensor(Operator):
         return Operator.__new__(cls)
 
     def __init__(self, expression, indices):
-        ufl_assert(isinstance(expression, Expr), "Expecting ufl expression.")
-        ufl_assert(expression.ufl_shape == (), "Expecting scalar valued expression.")
-        ufl_assert(isinstance(indices, MultiIndex), "Expecting a MultiIndex.")
-        ufl_assert(all(isinstance(i, Index) for i in indices),
-           "Expecting sequence of Index objects, not %s." % repr(indices))
+        if not isinstance(expression, Expr):
+            error("Expecting ufl expression.")
+        if expression.ufl_shape != ():
+            error("Expecting scalar valued expression.")
+        if not isinstance(indices, MultiIndex):
+            error("Expecting a MultiIndex.")
+        if not all(isinstance(i, Index) for i in indices):
+            error("Expecting sequence of Index objects, not %s." % indices._ufl_err_str_())
 
         Operator.__init__(self, (expression, indices))
 
         fi, fid, sh = remove_indices(expression.ufl_free_indices,
-                                 expression.ufl_index_dimensions,
-                                 [ind.count() for ind in indices])
+                                     expression.ufl_index_dimensions,
+                                     [ind.count() for ind in indices])
         self.ufl_free_indices = fi
         self.ufl_index_dimensions = fid
         self.ufl_shape = sh
@@ -160,7 +164,6 @@ class ComponentTensor(Operator):
         if isinstance(expressions, Indexed):
             A, ii = expressions.ufl_operands
             if indices == ii:
-                #print "RETURNING", A, "FROM", expressions, indices, "SELF IS", self
                 return A
         return Operator._ufl_expr_reconstruct_(self, expressions, indices)
 
@@ -171,8 +174,8 @@ class ComponentTensor(Operator):
         indices = self.ufl_operands[1]
         a = self.ufl_operands[0]
 
-        ufl_assert(len(indices) == len(component),
-                   "Expecting a component matching the indices tuple.")
+        if len(indices) != len(component):
+            error("Expecting a component matching the indices tuple.")
 
         # Map component to indices
         for i, c in zip(indices, component):
@@ -188,8 +191,6 @@ class ComponentTensor(Operator):
     def __str__(self):
         return "{ A | A_{%s} = %s }" % (self.ufl_operands[1], self.ufl_operands[0])
 
-    def __repr__(self):
-        return "ComponentTensor(%r, %r)" % (self.ufl_operands[0], self.ufl_operands[1])
 
 # --- User-level functions to wrap expressions in the correct way ---
 
@@ -199,6 +200,7 @@ def numpy2nestedlists(arr):
         return arr
     return [numpy2nestedlists(arr[k]) for k in range(arr.shape[0])]
 
+
 def _as_list_tensor(expressions):
     if isinstance(expressions, (list, tuple)):
         expressions = [_as_list_tensor(e) for e in expressions]
@@ -206,6 +208,7 @@ def _as_list_tensor(expressions):
     else:
         return as_ufl(expressions)
 
+
 def from_numpy_to_lists(expressions):
     try:
         import numpy
@@ -215,20 +218,21 @@ def from_numpy_to_lists(expressions):
         pass
     return expressions
 
+
 def as_tensor(expressions, indices=None):
     """UFL operator: Make a tensor valued expression.
 
     This works in two different ways, by using indices or lists.
 
-    1) Returns A such that A[indices] = expressions.
-    If indices are provided, expressions must be a scalar
+    1) Returns :math:`A` such that :math:`A` [*indices*] = *expressions*.
+    If *indices* are provided, *expressions* must be a scalar
     valued expression with all the provided indices among
     its free indices. This operator will then map each of these
     indices to a tensor axis, thereby making a tensor valued
     expression from a scalar valued expression with free indices.
 
-    2) Returns A such that A[k,...] = expressions[k].
-    If no indices are provided, expressions must be a list
+    2) Returns :math:`A` such that :math:`A[k,...]` = *expressions*[k].
+    If no indices are provided, *expressions* must be a list
     or tuple of expressions. The expressions can also consist
     of recursively nested lists to build higher rank tensors.
     """
@@ -245,7 +249,8 @@ def as_tensor(expressions, indices=None):
         if not isinstance(expressions, (list, tuple)):
             error("Expecting nested list or tuple.")
 
-        # Recursive conversion from nested lists to nested ListTensor objects
+        # Recursive conversion from nested lists to nested ListTensor
+        # objects
         return _as_list_tensor(expressions)
     else:
         # Make sure we have a tuple of indices
@@ -269,12 +274,14 @@ def as_tensor(expressions, indices=None):
         # Make a tensor from given scalar expression with free indices
         return ComponentTensor(expressions, indices)
 
-def as_matrix(expressions, indices = None):
-    "UFL operator: As as_tensor(), but limited to rank 2 tensors."
+
+def as_matrix(expressions, indices=None):
+    "UFL operator: As *as_tensor()*, but limited to rank 2 tensors."
     if indices is None:
         # Allow as_matrix(as_matrix(A)) in user code
         if isinstance(expressions, Expr):
-            ufl_assert(len(expressions.ufl_shape) == 2, "Expecting rank 2 tensor.")
+            if len(expressions.ufl_shape) != 2:
+                error("Expecting rank 2 tensor.")
             return expressions
 
         # To avoid importing numpy unneeded, it's quite slow...
@@ -282,21 +289,24 @@ def as_matrix(expressions, indices = None):
             expressions = from_numpy_to_lists(expressions)
 
         # Check for expected list structure
-        ufl_assert(isinstance(expressions, (list, tuple)),
-            "Expecting nested list or tuple of Exprs.")
-        ufl_assert(isinstance(expressions[0], (list, tuple)),
-            "Expecting nested list or tuple of Exprs.")
+        if not isinstance(expressions, (list, tuple)):
+            error("Expecting nested list or tuple of Exprs.")
+        if not isinstance(expressions[0], (list, tuple)):
+            error("Expecting nested list or tuple of Exprs.")
     else:
-        ufl_assert(len(indices) == 2, "Expecting exactly two indices.")
+        if len(indices) != 2:
+            error("Expecting exactly two indices.")
 
     return as_tensor(expressions, indices)
 
-def as_vector(expressions, index = None):
-    "UFL operator: As as_tensor(), but limited to rank 1 tensors."
+
+def as_vector(expressions, index=None):
+    "UFL operator: As ``as_tensor()``, but limited to rank 1 tensors."
     if index is None:
         # Allow as_vector(as_vector(v)) in user code
         if isinstance(expressions, Expr):
-            ufl_assert(len(expressions.ufl_shape) == 1, "Expecting rank 1 tensor.")
+            if len(expressions.ufl_shape) != 1:
+                error("Expecting rank 1 tensor.")
             return expressions
 
         # To avoid importing numpy unneeded, it's quite slow...
@@ -304,14 +314,16 @@ def as_vector(expressions, index = None):
             expressions = from_numpy_to_lists(expressions)
 
         # Check for expected list structure
-        ufl_assert(isinstance(expressions, (list, tuple)),
-            "Expecting nested list or tuple of Exprs.")
+        if not isinstance(expressions, (list, tuple)):
+            error("Expecting nested list or tuple of Exprs.")
     else:
-        ufl_assert(isinstance(index, Index), "Expecting a single Index object.")
+        if not isinstance(index, Index):
+            error("Expecting a single Index object.")
         index = (index,)
 
     return as_tensor(expressions, index)
 
+
 def as_scalar(expression):
     """Given a scalar or tensor valued expression A, returns either of the tuples::
 
@@ -324,6 +336,7 @@ def as_scalar(expression):
         expression = expression[ii]
     return expression, ii
 
+
 def as_scalars(*expressions):
     """Given multiple scalar or tensor valued expressions A, returns either of the tuples::
 
@@ -336,52 +349,65 @@ def as_scalars(*expressions):
         expressions = [expression[ii] for expression in expressions]
     return expressions, ii
 
+
 def relabel(A, indexmap):
-    "UFL operator: Relabel free indices of A with new indices, using the given mapping."
+    "UFL operator: Relabel free indices of :math:`A` with new indices, using the given mapping."
     ii = tuple(sorted(indexmap.keys()))
     jj = tuple(indexmap[i] for i in ii)
-    ufl_assert(all(isinstance(i, Index) for i in ii), "Expecting Index objects.")
-    ufl_assert(all(isinstance(j, Index) for j in jj), "Expecting Index objects.")
+    if not all(isinstance(i, Index) for i in ii):
+        error("Expecting Index objects.")
+    if not all(isinstance(j, Index) for j in jj):
+        error("Expecting Index objects.")
     return as_tensor(A, ii)[jj]
 
+
 # --- Experimental support for dyadic notation:
 
 def unit_list(i, n):
     return [(1 if i == j else 0) for j in range(n)]
 
+
 def unit_list2(i, j, n):
     return [[(1 if (i == i0 and j == j0) else 0) for j0 in range(n)] for i0 in range(n)]
 
+
 def unit_vector(i, d):
-    "UFL value: A constant unit vector in direction i with dimension d."
+    "UFL value: A constant unit vector in direction *i* with dimension *d*."
     return as_vector(unit_list(i, d))
 
+
 def unit_vectors(d):
-    "UFL value: A tuple of constant unit vectors in all directions with dimension d."
+    """UFL value: A tuple of constant unit vectors in all directions with
+    dimension *d*."""
     return tuple(unit_vector(i, d) for i in range(d))
 
+
 def unit_matrix(i, j, d):
-    "UFL value: A constant unit matrix in direction i,j with dimension d."
+    "UFL value: A constant unit matrix in direction *i*,*j* with dimension *d*."
     return as_matrix(unit_list2(i, j, d))
 
+
 def unit_matrices(d):
-    "UFL value: A tuple of constant unit matrices in all directions with dimension d."
+    """UFL value: A tuple of constant unit matrices in all directions with
+    dimension *d*."""
     return tuple(unit_matrix(i, j, d) for i in range(d) for j in range(d))
 
+
 def dyad(d, *iota):
     "TODO: Develop this concept, can e.g. write A[i,j]*dyad(j,i) for the transpose."
     from ufl.constantvalue import Identity
-    from ufl.operators import outer # a bit of circular dependency issue here
+    from ufl.operators import outer  # a bit of circular dependency issue here
     I = Identity(d)
     i = iota[0]
-    e = as_vector(I[i,:], i)
+    e = as_vector(I[i, :], i)
     for i in iota[1:]:
-        e = outer(e, as_vector(I[i,:], i))
+        e = outer(e, as_vector(I[i, :], i))
     return e
 
+
 def unit_indexed_tensor(shape, component):
     from ufl.constantvalue import Identity
-    from ufl.operators import outer # a bit of circular dependency issue here
+    from ufl.operators import outer  # a bit of circular dependency issue here
     r = len(shape)
     if r == 0:
         return 0, ()
@@ -398,6 +424,7 @@ def unit_indexed_tensor(shape, component):
         E = outer(E, e)
     return E, jj
 
+
 def unwrap_list_tensor(lt):
     components = []
     sh = lt.ufl_shape
