@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 "Various string formatting utilities."
 
-# Copyright (C) 2008-2015 Martin Sandve Alnæs and Anders Logg
+# Copyright (C) 2008-2016 Martin Sandve Alnæs and Anders Logg
 #
 # This file is part of UFL.
 #
@@ -18,6 +18,9 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 
+from six import string_types
+
+
 def camel2underscore(name):
     "Convert a CamelCaps string to underscore_syntax."
     letters = []
@@ -33,6 +36,7 @@ def camel2underscore(name):
         letters.append(l)
     return "".join(letters)
 
+
 def lstr(l):
     "Pretty-print list or tuple, invoking str() on items instead of repr() like str() does."
     if isinstance(l, list):
@@ -41,11 +45,13 @@ def lstr(l):
         return "(" + ", ".join(lstr(item) for item in l) + ")"
     return str(l)
 
+
 def dstr(d, colsize=80):
     "Pretty-print dictionary of key-value pairs."
     sorted_keys = sorted(d.keys())
     return tstr([(key, d[key]) for key in sorted_keys], colsize)
 
+
 def tstr(t, colsize=80):
     "Pretty-print list of tuples of key-value pairs."
     if not t:
@@ -62,7 +68,7 @@ def tstr(t, colsize=80):
     s = ""
     for (key, value) in t:
         key = str(key)
-        if isinstance(value, str):
+        if isinstance(value, string_types):
             value = "'%s'" % value
         else:
             value = str(value)
@@ -75,10 +81,12 @@ def tstr(t, colsize=80):
             space = " "*(keylen + 2)
     return s
 
+
 def sstr(s):
     "Pretty-print set."
     return ", ".join(str(x) for x in s)
 
+
 def istr(o):
     "Format object as string, inserting ? for None."
     if o is None:
@@ -86,6 +94,7 @@ def istr(o):
     else:
         return str(o)
 
+
 def estr(elements):
     "Format list of elements for printing."
     return ", ".join(e.shortstr() for e in elements)
