@@ -20,6 +20,8 @@
 
 from ufl.log import warning
 from six import itervalues, iteritems
+from six import string_types
+
 
 def topological_sorting(nodes, edges):
     """
@@ -69,7 +71,8 @@ def sorted_by_ufl_id(seq):
 
 def sorted_by_key(mapping):
     "Sort dict items by key, allowing different key types."
-    # Python3 doesn't allow comparing builtins of different type, therefore the typename trick here
+    # Python3 doesn't allow comparing builtins of different type,
+    # therefore the typename trick here
     def _key(x):
         return (type(x[0]).__name__, x[0])
     return sorted(iteritems(mapping), key=_key)
@@ -77,7 +80,8 @@ def sorted_by_key(mapping):
 
 def sorted_by_tuple_key(mapping):
     "Sort dict items by tuple valued keys, allowing different types as items of the key tuples."
-    # Python3 doesn't allow comparing builtins of different type, therefore the typename trick here
+    # Python3 doesn't allow comparing builtins of different type,
+    # therefore the typename trick here
     def _tuple_key(x):
         return tuple((type(k).__name__, k) for k in x[0])
     return sorted(iteritems(mapping), key=_tuple_key)
@@ -97,7 +101,7 @@ def canonicalize_metadata(metadata):
 
     if isinstance(metadata, dict):
         keys = sorted(metadata.keys())
-        assert all(isinstance(key, str) for key in keys)
+        assert all(isinstance(key, string_types) for key in keys)
         values = [metadata[key] for key in keys]
     elif isinstance(metadata, (tuple, list)):
         values = metadata
@@ -106,7 +110,7 @@ def canonicalize_metadata(metadata):
     for value in values:
         if isinstance(value, (dict, list, tuple)):
             value = canonicalize_metadata(value)
-        elif isinstance(value, (int, float, str)):
+        elif isinstance(value, (int, float, string_types)):
             value = str(value)
         else:
             warning("Applying str() to a metadata value of type {0}, don't know if this is safe.".format(type(value).__name__))
