@@ -1,7 +1,8 @@
+# -*- coding: utf-8 -*-
 """Defines the Variable and Label classes, used to label
 expressions as variables for differentiation."""
 
-# Copyright (C) 2008-2014 Martin Sandve Alnes
+# Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -18,9 +19,10 @@ expressions as variables for differentiation."""
 # You should have received a copy of the GNU Lesser General Public License
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 
-from ufl.common import counted_init
+from ufl.utils.counted import counted_init
+from ufl.utils.py23 import as_native_str
+from ufl.utils.py23 import as_native_strings
 from ufl.log import error
-from ufl.assertions import ufl_assert
 from ufl.core.expr import Expr
 from ufl.core.ufl_type import ufl_type
 from ufl.core.terminal import Terminal
@@ -30,7 +32,7 @@ from ufl.constantvalue import as_ufl
 
 @ufl_type()
 class Label(Terminal):
-    __slots__ = ("_count",)
+    __slots__ = as_native_strings(("_count",))
 
     _globalcount = 0
 
@@ -45,7 +47,8 @@ class Label(Terminal):
         return "Label(%d)" % self._count
 
     def __repr__(self):
-        return "Label(%d)" % self._count
+        r = "Label(%d)" % self._count
+        return as_native_str(r)
 
     @property
     def ufl_shape(self):
@@ -60,10 +63,9 @@ class Label(Terminal):
         error("Label has no free indices (it is not a tensor expression).")
 
     def is_cellwise_constant(self):
-        error("Asking if a Label is cellwise constant makes no sense (it is not a tensor expression).")
-        #return True # Could also just return True, after all it doesn't change with the cell
+        return True
 
-    def domains(self):
+    def ufl_domains(self):
         "Return tuple of domains related to this terminal object."
         return ()
 
@@ -90,17 +92,17 @@ class Variable(Operator):
             label = Label()
 
         # Checks
-        ufl_assert(isinstance(expression, Expr), "Expecting Expr.")
-        ufl_assert(isinstance(label, Label), "Expecting a Label.")
-        ufl_assert(not expression.ufl_free_indices, "Variable cannot wrap an expression with free indices.")
+        if not isinstance(expression, Expr):
+            error("Expecting Expr.")
+        if not isinstance(label, Label):
+            error("Expecting a Label.")
+        if expression.ufl_free_indices:
+            error("Variable cannot wrap an expression with free indices.")
 
         Operator.__init__(self, (expression, label))
 
-    def domains(self):
-        return self.ufl_operands[0].domains()
-
-    def is_cellwise_constant(self):
-        return self.ufl_operands[0].is_cellwise_constant()
+    def ufl_domains(self):
+        return self.ufl_operands[0].ufl_domains()
 
     def evaluate(self, x, mapping, component, index_values):
         a = self.ufl_operands[0].evaluate(x, mapping, component, index_values)
@@ -113,12 +115,10 @@ class Variable(Operator):
         return self.ufl_operands[1]
 
     def __eq__(self, other):
-        return (isinstance(other, Variable)
-                and self.ufl_operands[1] == other.ufl_operands[1]
-                and self.ufl_operands[0] == other.ufl_operands[0])
+        return (isinstance(other, Variable) and
+                self.ufl_operands[1] == other.ufl_operands[1] and
+                self.ufl_operands[0] == other.ufl_operands[0])
 
     def __str__(self):
-        return "var%d(%s)" % (self.ufl_operands[1].count(), self.ufl_operands[0])
-
-    def __repr__(self):
-        return "Variable(%r, %r)" % (self.ufl_operands[0], self.ufl_operands[1])
+        return "var%d(%s)" % (self.ufl_operands[1].count(),
+                              self.ufl_operands[0])
